@@ -5,11 +5,11 @@
  *
  * by Darrin Smith (beldin@light.iinet.net.au)
  *
- * $Id: botmsg.c,v 1.37 2008-02-16 21:41:03 guppy Exp $
+ * $Id: botmsg.c,v 1.39 2010/01/03 13:27:31 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -925,7 +925,7 @@ int add_note(char *to, char *from, char *msg, int idx, int echo)
   if (Tcl_VarEval(interp, "storenote", " $_from $_to $_data $_idx", NULL) ==
       TCL_OK) {
 
-    if (interp->result && interp->result[0])
+    if (!tcl_resultempty())
       status = NOTE_FWD;
 
     /* User is away in all sessions -- just notify the user that a
