@@ -7,11 +7,11 @@
  *   linking, unlinking, and relaying to another bot
  *   pinging the bots periodically and checking leaf status
  *
- * $Id: botnet.c,v 1.62 2008-02-16 21:41:03 guppy Exp $
+ * $Id: botnet.c,v 1.65 2010/06/29 15:52:24 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -437,12 +437,12 @@ void answer_local_whom(int idx, int chan)
             BOT_LOCALCHAN);
   else if (chan > 0) {
     simple_sprintf(idle, "assoc %d", chan);
-    if ((Tcl_Eval(interp, idle) != TCL_OK) || !interp->result[0])
+    if ((Tcl_Eval(interp, idle) != TCL_OK) || tcl_resultempty())
       dprintf(idx, "%s %s%d:\n", BOT_USERSONCHAN,
               (chan < GLOBAL_CHANS) ? "" : "*", chan % GLOBAL_CHANS);
     else
       dprintf(idx, "%s '%s%s' (%s%d):\n", BOT_USERSONCHAN,
-              (chan < GLOBAL_CHANS) ? "" : "*", interp->result,
+              (chan < GLOBAL_CHANS) ? "" : "*", tcl_resultstring(),
               (chan < GLOBAL_CHANS) ? "" : "*", chan % GLOBAL_CHANS);
   }
   /* Find longest nick and botnick */
@@ -1008,7 +1008,7 @@ int botlink(char *linker, int idx, char *nick)
         dprintf(idx, "%s .chaddr %s %s\n",
                 MISC_USEFORMAT, nick, MISC_CHADDRFORMAT);
       }
-    } else if (dcc_total == max_dcc) {
+    } else if (dcc_total == max_dcc && increase_socks_max()) {
       if (idx >= 0)
         dprintf(idx, "%s\n", DCC_TOOMANYDCCS1);
     } else {
