@@ -7,11 +7,11 @@
  *   telling the current programmed settings
  *   initializing a lot of stuff and loading the tcl scripts
  *
- * $Id: chanprog.c,v 1.61 2008-02-16 21:41:03 guppy Exp $
+ * $Id: chanprog.c,v 1.67 2010/06/29 15:52:24 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -51,7 +51,7 @@ extern char ver[], botnetnick[], firewall[], motdfile[], userfile[], helpdir[],
 extern time_t now, online_since;
 extern int backgrd, term_z, con_chan, cache_hit, cache_miss, firewallport,
            default_flags, max_logs, conmask, protect_readonly, make_userfile,
-           noshare, ignore_time;
+           noshare, ignore_time, max_socks;
 
 tcl_timer_t *timer = NULL;         /* Minutely timer               */
 tcl_timer_t *utimer = NULL;        /* Secondly timer               */
@@ -346,19 +346,18 @@ void tell_verbose_status(int idx)
   /* info library */
   dprintf(idx, "%s %s\n", MISC_TCLLIBRARY,
           ((interp) && (Tcl_Eval(interp, "info library") == TCL_OK)) ?
-          interp->result : "*unknown*");
+          tcl_resultstring() : "*unknown*");
 
   /* info tclversion/patchlevel */
   dprintf(idx, "%s %s (%s %s)\n", MISC_TCLVERSION,
           ((interp) && (Tcl_Eval(interp, "info patchlevel") == TCL_OK)) ?
-          interp->result : (Tcl_Eval(interp, "info tclversion") == TCL_OK) ?
-          interp->result : "*unknown*", MISC_TCLHVERSION,
+          tcl_resultstring() : (Tcl_Eval(interp, "info tclversion") == TCL_OK) ?
+          tcl_resultstring() : "*unknown*", MISC_TCLHVERSION,
           TCL_PATCH_LEVEL ? TCL_PATCH_LEVEL : "*unknown*");
 
-#ifdef HAVE_TCL_THREADS
-  dprintf(idx, "Tcl is threaded.\n");
-#endif
-
+  if (tcl_threaded())
+    dprintf(idx, "Tcl is threaded.\n");
+  dprintf(idx, "Socket table: %d/%d\n", threaddata()->MAXSOCKS, max_socks);
 }
 
 /* Show all internal state variables
@@ -668,7 +667,7 @@ int isowner(char *name)
 {
   register char *ptr = NULL, *s = NULL, *n = NULL;
 
-  if (!owner || !name)
+  if (!name)
     return 0;
 
   ptr = owner - 1;
@@ -679,7 +678,8 @@ int isowner(char *name)
       if (!s)
         s = ptr;
     } else if (s) {
-      for (n = name; *n && *s && s < ptr && tolower(*n) == tolower(*s); n++, s++);
+      for (n = name; *n && *s && s < ptr &&
+           tolower((unsigned) *n) == tolower((unsigned) *s); n++, s++);
 
       if (s == ptr && !*n)
         return 1;
