@@ -3,11 +3,11 @@
  *   commands from a user via dcc
  *   (split in 2, this portion contains no-irc commands)
  *
- * $Id: cmds.c,v 1.116 2008-02-16 21:41:03 guppy Exp $
+ * $Id: cmds.c,v 1.128 2010/07/01 16:10:49 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -97,13 +97,13 @@ static void tell_who(struct userrec *u, int idx, int chan)
             "^ = halfop)\n", BOT_PARTYMEMBS);
   else {
     simple_sprintf(s, "assoc %d", chan);
-    if ((Tcl_Eval(interp, s) != TCL_OK) || !interp->result[0])
+    if ((Tcl_Eval(interp, s) != TCL_OK) || tcl_resultempty())
       dprintf(idx, "%s %s%d: (* = owner, + = master, %% = botmaster, @ = op, "
               "^ = halfop)\n", BOT_PEOPLEONCHAN, (chan < GLOBAL_CHANS) ? "" :
               "*", chan % GLOBAL_CHANS);
     else
       dprintf(idx, "%s '%s' (%s%d): (* = owner, + = master, %% = botmaster, @ = op, "
-              "^ = halfop)\n", BOT_PEOPLEONCHAN, interp->result,
+              "^ = halfop)\n", BOT_PEOPLEONCHAN, tcl_resultstring(),
               (chan < GLOBAL_CHANS) ? "" : "*", chan % GLOBAL_CHANS);
   }
 
@@ -306,10 +306,10 @@ static void cmd_whom(struct userrec *u, int idx, char *par)
     int chan = -1;
 
     if ((par[0] < '0') || (par[0] > '9')) {
-      Tcl_SetVar(interp, "chan", par, 0);
-      if ((Tcl_VarEval(interp, "assoc ", "$chan", NULL) == TCL_OK) &&
-          interp->result[0]) {
-        chan = atoi(interp->result);
+      Tcl_SetVar(interp, "_chan", par, 0);
+      if ((Tcl_VarEval(interp, "assoc ", "$_chan", NULL) == TCL_OK) &&
+          !tcl_resultempty()) {
+        chan = tcl_resultint();
       }
       if (chan <= 0) {
         dprintf(idx, "No such channel exists.\n");
@@ -513,24 +513,13 @@ static void cmd_who(struct userrec *u, int idx, char *par)
 
 static void cmd_whois(struct userrec *u, int idx, char *par)
 {
-  struct flag_record fr = { FR_GLOBAL | FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0 };
-  char *handle;
-
   if (!par[0]) {
     dprintf(idx, "Usage: whois <handle>\n");
     return;
   }
 
-  handle = newsplit(&par);
-  get_user_flagrec(u, &fr, NULL);
-  if (egg_strcasecmp(handle, dcc[idx].nick) && !glob_botmast(fr) &&
-      !glob_op(fr) && !chan_master(fr)) {
-    dprintf(idx, "You do not have access to whois handles other than your "
-            "own.\n");
-    return;
-  }
-  putlog(LOG_CMDS, "*", "#%s# whois %s", dcc[idx].nick, handle);
-  tell_user_ident(idx, handle, u ? (u->flags & USER_MASTER) : 0);
+  putlog(LOG_CMDS, "*", "#%s# whois %s", dcc[idx].nick, par);
+  tell_user_ident(idx, par, u ? (u->flags & USER_MASTER) : 0);
 }
 
 static void cmd_match(struct userrec *u, int idx, char *par)
@@ -657,7 +646,7 @@ static void cmd_boot(struct userrec *u, int idx, char *par)
 static void cmd_console(struct userrec *u, int idx, char *par)
 {
   char *nick, s[2], s1[512];
-  int dest = 0, i, ok = 0, pls, md;
+  int dest = 0, i, ok = 0, pls;
   struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
   module_entry *me;
 
@@ -705,9 +694,8 @@ static void cmd_console(struct userrec *u, int idx, char *par)
     }
     strncpyz(dcc[dest].u.chat->con_chan, nick, 81);
     nick[0] = 0;
-    if ((dest == idx) && !glob_master(fr) && !chan_master(fr))
-      /* Consoling to another channel for self */
-      dcc[dest].u.chat->con_flags &= ~(LOG_MISC | LOG_CMDS | LOG_WALL);
+    if (dest != idx)
+      get_user_flagrec(dcc[dest].user, &fr, dcc[dest].u.chat->con_chan);
   }
   if (!nick[0])
     nick = newsplit(&par);
@@ -723,28 +711,15 @@ static void cmd_console(struct userrec *u, int idx, char *par)
       else {
         s[0] = *nick;
         s[1] = 0;
-        md = logmodes(s);
-        if ((dest == idx) && !glob_master(fr) && pls) {
-          if (chan_master(fr))
-            md &= ~(LOG_FILES | LOG_LEV1 | LOG_LEV2 | LOG_LEV3 |
-                    LOG_LEV4 | LOG_LEV5 | LOG_LEV6 | LOG_LEV7 |
-                    LOG_LEV8 | LOG_DEBUG);
-          else
-            md &= ~(LOG_MISC | LOG_CMDS | LOG_FILES | LOG_LEV1 |
-                    LOG_LEV2 | LOG_LEV3 | LOG_LEV4 | LOG_LEV5 |
-                    LOG_LEV6 | LOG_LEV7 | LOG_LEV8 | LOG_WALL | LOG_DEBUG);
-        }
-        if (!glob_owner(fr) && pls)
-          md &= ~(LOG_RAW | LOG_SRVOUT | LOG_BOTNET | LOG_BOTSHARE);
-        if (!glob_botmast(fr) && pls)
-          md &= ~LOG_BOTS;
         if (pls)
-          dcc[dest].u.chat->con_flags |= md;
+          dcc[dest].u.chat->con_flags |= logmodes(s);
         else
-          dcc[dest].u.chat->con_flags &= ~md;
+          dcc[dest].u.chat->con_flags &= ~logmodes(s);
       }
     }
   }
+  dcc[dest].u.chat->con_flags = check_conflags(&fr,
+                                               dcc[dest].u.chat->con_flags);
   putlog(LOG_CMDS, "*", "#%s# console %s", dcc[idx].nick, s1);
   if (dest == idx) {
     dprintf(idx, "Set your console to %s: %s (%s).\n",
@@ -859,7 +834,7 @@ static void cmd_chhandle(struct userrec *u, int idx, char *par)
     return;
   }
   for (i = 0; i < strlen(newhand); i++)
-    if ((newhand[i] <= 32) || (newhand[i] >= 127) || (newhand[i] == '@'))
+    if (((unsigned char) newhand[i] <= 32) || (newhand[i] == '@'))
       newhand[i] = '?';
   if (strchr(BADHANDCHARS, newhand[0]) != NULL)
     dprintf(idx, "Bizarre quantum forces prevent nicknames from starting with "
@@ -903,7 +878,7 @@ static void cmd_handle(struct userrec *u, int idx, char *par)
     return;
   }
   for (i = 0; i < strlen(newhandle); i++)
-    if ((newhandle[i] <= 32) || (newhandle[i] >= 127) || (newhandle[i] == '@'))
+    if (((unsigned char) newhandle[i] <= 32) || (newhandle[i] == '@'))
       newhandle[i] = '?';
   if (strchr(BADHANDCHARS, newhandle[0]) != NULL)
     dprintf(idx,
@@ -987,7 +962,7 @@ static void cmd_chaddr(struct userrec *u, int idx, char *par)
     dprintf(idx, "This command is only useful for tandem bots.\n");
     return;
   }
-  if ((bot_flags(u1) & BOT_SHARE) && (!u || !u->flags & USER_OWNER)) {
+  if ((bot_flags(u1) & BOT_SHARE) && (!u || !(u->flags & USER_OWNER))) {
     dprintf(idx, "You can't change a share bot's address.\n");
     return;
   }
@@ -1280,6 +1255,7 @@ static void cmd_banner(struct userrec *u, int idx, char *par)
 int check_dcc_attrs(struct userrec *u, int oatr)
 {
   int i, stat;
+  struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
 
   if (!u)
     return 0;
@@ -1297,16 +1273,6 @@ int check_dcc_attrs(struct userrec *u, int oatr)
         botnet_send_join_idx(i, -1);
       }
       if ((oatr & USER_MASTER) && !(u->flags & USER_MASTER)) {
-        struct flag_record fr = { FR_CHAN | FR_ANYWH, 0, 0, 0, 0, 0 };
-
-        dcc[i].u.chat->con_flags &= ~(LOG_MISC | LOG_CMDS | LOG_RAW |
-                                      LOG_FILES | LOG_LEV1 | LOG_LEV2 |
-                                      LOG_LEV3 | LOG_LEV4 | LOG_LEV5 |
-                                      LOG_LEV6 | LOG_LEV7 | LOG_LEV8 |
-                                      LOG_WALL | LOG_DEBUG);
-        get_user_flagrec(u, &fr, NULL);
-        if (!chan_master(fr))
-          dcc[i].u.chat->con_flags |= (LOG_MISC | LOG_CMDS);
         dprintf(i, "*** POOF! ***\n");
         dprintf(i, "You are no longer a master on this bot.\n");
       }
@@ -1331,6 +1297,9 @@ int check_dcc_attrs(struct userrec *u, int oatr)
         dprintf(i, "@@@ POOF! @@@\n");
         dprintf(i, "You are no longer an owner of this bot.\n");
       }
+      get_user_flagrec(u, &fr, dcc[i].u.chat->con_chan);
+      dcc[i].u.chat->con_flags = check_conflags(&fr,
+                                                dcc[i].u.chat->con_flags);
       if ((stat & STAT_PARTY) && (u->flags & USER_OP))
         stat &= ~STAT_PARTY;
       if (!(stat & STAT_PARTY) && !(u->flags & USER_OP) &&
@@ -1390,7 +1359,8 @@ int check_dcc_attrs(struct userrec *u, int oatr)
 int check_dcc_chanattrs(struct userrec *u, char *chname, int chflags,
                         int ochatr)
 {
-  int i, found = 0, atr = u ? u->flags : 0;
+  int i, found = 0;
+  struct flag_record fr = { FR_CHAN, 0, 0, 0, 0, 0 };
   struct chanset_t *chan;
 
   if (!u)
@@ -1403,19 +1373,11 @@ int check_dcc_chanattrs(struct userrec *u, char *chname, int chflags,
           (ochatr & (USER_OP | USER_MASTER | USER_OWNER))))
         botnet_send_join_idx(i, -1);
       if ((ochatr & USER_MASTER) && !(chflags & USER_MASTER)) {
-        if (!(atr & USER_MASTER))
-          dcc[i].u.chat->con_flags &= ~(LOG_MISC | LOG_CMDS);
         dprintf(i, "*** POOF! ***\n");
         dprintf(i, "You are no longer a master on %s.\n", chname);
       }
       if (!(ochatr & USER_MASTER) && (chflags & USER_MASTER)) {
         dcc[i].u.chat->con_flags |= conmask;
-        if (!(atr & USER_MASTER))
-          dcc[i].u.chat->con_flags &= ~(LOG_LEV1 | LOG_LEV2 | LOG_LEV3 |
-                                        LOG_LEV4 | LOG_LEV5 | LOG_LEV6 |
-                                        LOG_LEV7 | LOG_LEV8 | LOG_RAW |
-                                        LOG_DEBUG | LOG_WALL | LOG_FILES |
-                                        LOG_SRVOUT);
         dprintf(i, "*** POOF! ***\n");
         dprintf(i, "You are now a master on %s.\n", chname);
       }
@@ -1431,7 +1393,6 @@ int check_dcc_chanattrs(struct userrec *u, char *chname, int chflags,
           (!(chflags & (USER_OP | USER_MASTER | USER_OWNER)))) ||
           ((chflags & (USER_OP | USER_MASTER | USER_OWNER)) &&
           (!(ochatr & (USER_OP | USER_MASTER | USER_OWNER))))) {
-        struct flag_record fr = { FR_CHAN, 0, 0, 0, 0, 0 };
 
         for (chan = chanset; chan && !found; chan = chan->next) {
           get_user_flagrec(u, &fr, chan->dname);
@@ -1445,6 +1406,10 @@ int check_dcc_chanattrs(struct userrec *u, char *chname, int chflags,
         else
           strcpy(dcc[i].u.chat->con_chan, "*");
       }
+      fr.match = (FR_CHAN | FR_GLOBAL);
+      get_user_flagrec(u, &fr, dcc[i].u.chat->con_chan);
+      dcc[i].u.chat->con_flags = check_conflags(&fr,
+                                                dcc[i].u.chat->con_flags);
     }
   }
   return chflags;
@@ -1626,7 +1591,6 @@ static void cmd_chattr(struct userrec *u, int idx, char *par)
   }
   if (chg && (me = module_find("irc", 0, 0))) {
     Function *func = me->funcs;
-
     (func[IRC_CHECK_THIS_USER]) (hand, 0, NULL);
   }
   if (tmpchg)
@@ -1768,6 +1732,7 @@ static void cmd_botattr(struct userrec *u, int idx, char *par)
     user.match = FR_CHAN;
     get_user_flagrec(u2, &user, par);
     user.chan &= BOT_SHARE;
+    user.udef_chan = 0; /* udef chan flags are user only */
     build_flags(work, &user, NULL);
     if (work[0] != '-')
       dprintf(idx, "Bot flags for %s on %s are now +%s.\n", hand,
@@ -1807,10 +1772,10 @@ static void cmd_chat(struct userrec *u, int idx, char *par)
         if (!arg[1])
           newchan = 0;
         else {
-          Tcl_SetVar(interp, "chan", arg, 0);
-          if ((Tcl_VarEval(interp, "assoc ", "$chan", NULL) == TCL_OK) &&
-              interp->result[0])
-            newchan = atoi(interp->result);
+          Tcl_SetVar(interp, "_chan", arg, 0);
+          if ((Tcl_VarEval(interp, "assoc ", "$_chan", NULL) == TCL_OK) &&
+              !tcl_resultempty())
+            newchan = tcl_resultint();
           else
             newchan = -1;
         }
@@ -1830,10 +1795,10 @@ static void cmd_chat(struct userrec *u, int idx, char *par)
         if (!egg_strcasecmp(arg, "on"))
           newchan = 0;
         else {
-          Tcl_SetVar(interp, "chan", arg, 0);
-          if ((Tcl_VarEval(interp, "assoc ", "$chan", NULL) == TCL_OK) &&
-              interp->result[0])
-            newchan = atoi(interp->result);
+          Tcl_SetVar(interp, "_chan", arg, 0);
+          if ((Tcl_VarEval(interp, "assoc ", "$_chan", NULL) == TCL_OK) &&
+              !tcl_resultempty())
+            newchan = tcl_resultint();
           else
             newchan = -1;
         }
@@ -1927,7 +1892,7 @@ int stripmodes(char *s)
   int res = 0;
 
   for (; *s; s++)
-    switch (tolower(*s)) {
+    switch (tolower((unsigned) *s)) {
     case 'b':
       res |= STRIP_BOLD;
       break;
@@ -2228,11 +2193,11 @@ static void cmd_tcl(struct userrec *u, int idx, char *msg)
 #ifdef USE_TCL_ENCODING
   /* properly convert string to system encoding. */
   Tcl_DStringInit(&dstr);
-  Tcl_UtfToExternalDString(NULL, interp->result, -1, &dstr);
+  Tcl_UtfToExternalDString(NULL, tcl_resultstring(), -1, &dstr);
   result = Tcl_DStringValue(&dstr);
 #else
   /* use old pre-Tcl 8.1 way. */
-  result = interp->result;
+  result = tcl_resultstring();
 #endif
 
   if (code == TCL_OK)
@@ -2264,7 +2229,7 @@ static void cmd_set(struct userrec *u, int idx, char *msg)
   if (!msg[0]) {
     strcpy(s, "info globals");
     Tcl_Eval(interp, s);
-    dumplots(idx, "Global vars: ", interp->result);
+    dumplots(idx, "Global vars: ", tcl_resultstring());
     return;
   }
   strcpy(s + 4, msg);
@@ -2273,11 +2238,11 @@ static void cmd_set(struct userrec *u, int idx, char *msg)
 #ifdef USE_TCL_ENCODING
   /* properly convert string to system encoding. */
   Tcl_DStringInit(&dstr);
-  Tcl_UtfToExternalDString(NULL, interp->result, -1, &dstr);
+  Tcl_UtfToExternalDString(NULL, tcl_resultstring(), -1, &dstr);
   result = Tcl_DStringValue(&dstr);
 #else
   /* use old pre-Tcl 8.1 way. */
-  result = interp->result;
+  result = tcl_resultstring();
 #endif
 
   if (code == TCL_OK) {
@@ -2360,7 +2325,7 @@ static void cmd_pls_ignore(struct userrec *u, int idx, char *par)
     p = newsplit(&par);
     p_expire = p + 1;
     while (*(++p) != 0) {
-      switch (tolower(*p)) {
+      switch (tolower((unsigned) *p)) {
       case 'd':
         *p = 0;
         expire_foo = strtol(p_expire, NULL, 10);
@@ -2619,7 +2584,8 @@ static void cmd_mns_host(struct userrec *u, int idx, char *par)
   get_user_flagrec(u, &fr, NULL);
   get_user_flagrec(u2, &fr2, NULL);
   /* check to see if user is +d or +k and don't let them remove hosts */
-  if (glob_deop(fr) || glob_kick(fr) || chan_deop(fr) || chan_kick(fr)) {
+  if (((glob_deop(fr) || glob_kick(fr)) && !glob_master(fr)) ||
+      ((chan_deop(fr) || chan_kick(fr)) && !chan_master(fr))) {
     dprintf(idx, "You can't remove hostmasks while having the +d or +k "
             "flag.\n");
     return;
@@ -2792,73 +2758,73 @@ static void cmd_whoami(struct userrec *u, int idx, char *par)
  * As with msg commands, function is responsible for any logging.
  */
 cmd_t C_dcc[] = {
-  {"+bot",      "t",    (Function) cmd_pls_bot,    NULL},
-  {"+host",     "t|m",  (Function) cmd_pls_host,   NULL},
-  {"+ignore",   "m",    (Function) cmd_pls_ignore, NULL},
-  {"+user",     "m",    (Function) cmd_pls_user,   NULL},
-  {"-bot",      "t",    (Function) cmd_mns_user,   NULL},
-  {"-host",     "",     (Function) cmd_mns_host,   NULL},
-  {"-ignore",   "m",    (Function) cmd_mns_ignore, NULL},
-  {"-user",     "m",    (Function) cmd_mns_user,   NULL},
-  {"addlog",    "to|o", (Function) cmd_addlog,     NULL},
-  {"away",      "",     (Function) cmd_away,       NULL},
-  {"back",      "",     (Function) cmd_back,       NULL},
-  {"backup",    "m|m",  (Function) cmd_backup,     NULL},
-  {"banner",    "t",    (Function) cmd_banner,     NULL},
-  {"binds",     "m",    (Function) cmd_binds,      NULL},
-  {"boot",      "t",    (Function) cmd_boot,       NULL},
-  {"botattr",   "t",    (Function) cmd_botattr,    NULL},
-  {"botinfo",   "",     (Function) cmd_botinfo,    NULL},
-  {"bots",      "",     (Function) cmd_bots,       NULL},
-  {"bottree",   "",     (Function) cmd_bottree,    NULL},
-  {"chaddr",    "t",    (Function) cmd_chaddr,     NULL},
-  {"chat",      "",     (Function) cmd_chat,       NULL},
-  {"chattr",    "m|m",  (Function) cmd_chattr,     NULL},
-  {"chhandle",  "t",    (Function) cmd_chhandle,   NULL},
-  {"chnick",    "t",    (Function) cmd_chhandle,   NULL},
-  {"chpass",    "t",    (Function) cmd_chpass,     NULL},
-  {"comment",   "m",    (Function) cmd_comment,    NULL},
-  {"console",   "to|o", (Function) cmd_console,    NULL},
-  {"dccstat",   "t",    (Function) cmd_dccstat,    NULL},
-  {"debug",     "m",    (Function) cmd_debug,      NULL},
-  {"die",       "n",    (Function) cmd_die,        NULL},
-  {"echo",      "",     (Function) cmd_echo,       NULL},
-  {"fixcodes",  "",     (Function) cmd_fixcodes,   NULL},
-  {"help",      "",     (Function) cmd_help,       NULL},
-  {"ignores",   "m",    (Function) cmd_ignores,    NULL},
-  {"link",      "t",    (Function) cmd_link,       NULL},
-  {"loadmod",   "n",    (Function) cmd_loadmod,    NULL},
-  {"match",     "to|o", (Function) cmd_match,      NULL},
-  {"me",        "",     (Function) cmd_me,         NULL},
-  {"module",    "m",    (Function) cmd_module,     NULL},
-  {"modules",   "n",    (Function) cmd_modules,    NULL},
-  {"motd",      "",     (Function) cmd_motd,       NULL},
-  {"newpass",   "",     (Function) cmd_newpass,    NULL},
-  {"handle",    "",     (Function) cmd_handle,     NULL},
-  {"nick",      "",     (Function) cmd_handle,     NULL},
-  {"page",      "",     (Function) cmd_page,       NULL},
-  {"quit",      "",     (Function) CMD_LEAVE,      NULL},
-  {"rehash",    "m",    (Function) cmd_rehash,     NULL},
-  {"rehelp",    "n",    (Function) cmd_rehelp,     NULL},
-  {"relay",     "o",    (Function) cmd_relay,      NULL},
-  {"reload",    "m|m",  (Function) cmd_reload,     NULL},
-  {"restart",   "m",    (Function) cmd_restart,    NULL},
-  {"save",      "m|m",  (Function) cmd_save,       NULL},
-  {"set",       "n",    (Function) cmd_set,        NULL},
-  {"simul",     "n",    (Function) cmd_simul,      NULL},
-  {"status",    "m|m",  (Function) cmd_status,     NULL},
-  {"strip",     "",     (Function) cmd_strip,      NULL},
-  {"su",        "",     (Function) cmd_su,         NULL},
-  {"tcl",       "n",    (Function) cmd_tcl,        NULL},
-  {"trace",     "t",    (Function) cmd_trace,      NULL},
-  {"unlink",    "t",    (Function) cmd_unlink,     NULL},
-  {"unloadmod", "n",    (Function) cmd_unloadmod,  NULL},
-  {"uptime",    "m|m",  (Function) cmd_uptime,     NULL},
-  {"vbottree",  "",     (Function) cmd_vbottree,   NULL},
-  {"who",       "",     (Function) cmd_who,        NULL},
-  {"whois",     "",     (Function) cmd_whois,      NULL},
-  {"whom",      "",     (Function) cmd_whom,       NULL},
-  {"traffic",   "m|m",  (Function) cmd_traffic,    NULL},
-  {"whoami",    "",     (Function) cmd_whoami,     NULL},
+  {"+bot",      "t",    (IntFunc) cmd_pls_bot,    NULL},
+  {"+host",     "t|m",  (IntFunc) cmd_pls_host,   NULL},
+  {"+ignore",   "m",    (IntFunc) cmd_pls_ignore, NULL},
+  {"+user",     "m",    (IntFunc) cmd_pls_user,   NULL},
+  {"-bot",      "t",    (IntFunc) cmd_mns_user,   NULL},
+  {"-host",     "",     (IntFunc) cmd_mns_host,   NULL},
+  {"-ignore",   "m",    (IntFunc) cmd_mns_ignore, NULL},
+  {"-user",     "m",    (IntFunc) cmd_mns_user,   NULL},
+  {"addlog",    "to|o", (IntFunc) cmd_addlog,     NULL},
+  {"away",      "",     (IntFunc) cmd_away,       NULL},
+  {"back",      "",     (IntFunc) cmd_back,       NULL},
+  {"backup",    "m|m",  (IntFunc) cmd_backup,     NULL},
+  {"banner",    "t",    (IntFunc) cmd_banner,     NULL},
+  {"binds",     "m",    (IntFunc) cmd_binds,      NULL},
+  {"boot",      "t",    (IntFunc) cmd_boot,       NULL},
+  {"botattr",   "t",    (IntFunc) cmd_botattr,    NULL},
+  {"botinfo",   "",     (IntFunc) cmd_botinfo,    NULL},
+  {"bots",      "",     (IntFunc) cmd_bots,       NULL},
+  {"bottree",   "",     (IntFunc) cmd_bottree,    NULL},
+  {"chaddr",    "t",    (IntFunc) cmd_chaddr,     NULL},
+  {"chat",      "",     (IntFunc) cmd_chat,       NULL},
+  {"chattr",    "m|m",  (IntFunc) cmd_chattr,     NULL},
+  {"chhandle",  "t",    (IntFunc) cmd_chhandle,   NULL},
+  {"chnick",    "t",    (IntFunc) cmd_chhandle,   NULL},
+  {"chpass",    "t",    (IntFunc) cmd_chpass,     NULL},
+  {"comment",   "m",    (IntFunc) cmd_comment,    NULL},
+  {"console",   "to|o", (IntFunc) cmd_console,    NULL},
+  {"dccstat",   "t",    (IntFunc) cmd_dccstat,    NULL},
+  {"debug",     "m",    (IntFunc) cmd_debug,      NULL},
+  {"die",       "n",    (IntFunc) cmd_die,        NULL},
+  {"echo",      "",     (IntFunc) cmd_echo,       NULL},
+  {"fixcodes",  "",     (IntFunc) cmd_fixcodes,   NULL},
+  {"help",      "",     (IntFunc) cmd_help,       NULL},
+  {"ignores",   "m",    (IntFunc) cmd_ignores,    NULL},
+  {"link",      "t",    (IntFunc) cmd_link,       NULL},
+  {"loadmod",   "n",    (IntFunc) cmd_loadmod,    NULL},
+  {"match",     "to|o", (IntFunc) cmd_match,      NULL},
+  {"me",        "",     (IntFunc) cmd_me,         NULL},
+  {"module",    "m",    (IntFunc) cmd_module,     NULL},
+  {"modules",   "n",    (IntFunc) cmd_modules,    NULL},
+  {"motd",      "",     (IntFunc) cmd_motd,       NULL},
+  {"newpass",   "",     (IntFunc) cmd_newpass,    NULL},
+  {"handle",    "",     (IntFunc) cmd_handle,     NULL},
+  {"nick",      "",     (IntFunc) cmd_handle,     NULL},
+  {"page",      "",     (IntFunc) cmd_page,       NULL},
+  {"quit",      "",     (IntFunc) CMD_LEAVE,      NULL},
+  {"rehash",    "m",    (IntFunc) cmd_rehash,     NULL},
+  {"rehelp",    "n",    (IntFunc) cmd_rehelp,     NULL},
+  {"relay",     "o",    (IntFunc) cmd_relay,      NULL},
+  {"reload",    "m|m",  (IntFunc) cmd_reload,     NULL},
+  {"restart",   "m",    (IntFunc) cmd_restart,    NULL},
+  {"save",      "m|m",  (IntFunc) cmd_save,       NULL},
+  {"set",       "n",    (IntFunc) cmd_set,        NULL},
+  {"simul",     "n",    (IntFunc) cmd_simul,      NULL},
+  {"status",    "m|m",  (IntFunc) cmd_status,     NULL},
+  {"strip",     "",     (IntFunc) cmd_strip,      NULL},
+  {"su",        "",     (IntFunc) cmd_su,         NULL},
+  {"tcl",       "n",    (IntFunc) cmd_tcl,        NULL},
+  {"trace",     "t",    (IntFunc) cmd_trace,      NULL},
+  {"unlink",    "t",    (IntFunc) cmd_unlink,     NULL},
+  {"unloadmod", "n",    (IntFunc) cmd_unloadmod,  NULL},
+  {"uptime",    "m|m",  (IntFunc) cmd_uptime,     NULL},
+  {"vbottree",  "",     (IntFunc) cmd_vbottree,   NULL},
+  {"who",       "",     (IntFunc) cmd_who,        NULL},
+  {"whois",     "to|o", (IntFunc) cmd_whois,      NULL},
+  {"whom",      "",     (IntFunc) cmd_whom,       NULL},
+  {"traffic",   "m|m",  (IntFunc) cmd_traffic,    NULL},
+  {"whoami",    "",     (IntFunc) cmd_whoami,     NULL},
   {NULL,        NULL,   NULL,                      NULL}
 };
