@@ -2,11 +2,11 @@
  * cmdt.h
  *   stuff for builtin commands
  *
- * $Id: cmdt.h,v 1.13 2008-02-16 21:41:03 guppy Exp $
+ * $Id: cmdt.h,v 1.15 2010/02/18 09:52:29 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,13 +29,13 @@
 typedef struct {
   char *name;
   char *flags;
-  Function func;
+  IntFunc func;
   char *funcname;
 } cmd_t;
 
 typedef struct {
   char *name;
-  Function func;
+  IntFunc func;
 } botcmd_t;
 
 #endif /* _EGG_CMDT_H */
