@@ -4,12 +4,12 @@
  *   provides the code used by the bot if the DNS module is not loaded
  *   DNS Tcl commands
  *
- * $Id: dns.c,v 1.35 2008-02-16 21:41:03 guppy Exp $
+ * $Id: dns.c,v 1.38 2010/01/03 13:27:32 pseudo Exp $
  */
 /*
  * Written by Fabian Knittel <fknittel@gmx.de>
  *
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -64,7 +64,7 @@ void eof_dcc_dnswait(int idx)
 
 static void display_dcc_dnswait(int idx, char *buf)
 {
-  sprintf(buf, "dns   waited %lis", now - dcc[idx].timeval);
+  sprintf(buf, "dns   waited %lis", (long) now - dcc[idx].timeval);
 }
 
 static int expmem_dcc_dnswait(void *x)
@@ -256,10 +256,10 @@ static void dns_tcl_iporhostres(IP ip, char *hostn, int ok, void *other)
     output = Tcl_Concat(2, argv);
 
     if (Tcl_Eval(interp, output) == TCL_ERROR)
-      putlog(LOG_MISC, "*", DCC_TCLERROR, tclinfo->proc, interp->result);
+      putlog(LOG_MISC, "*", DCC_TCLERROR, tclinfo->proc, tcl_resultstring());
     Tcl_Free(output);
   } else if (Tcl_Eval(interp, Tcl_DStringValue(&list)) == TCL_ERROR)
-    putlog(LOG_MISC, "*", DCC_TCLERROR, tclinfo->proc, interp->result);
+    putlog(LOG_MISC, "*", DCC_TCLERROR, tclinfo->proc, tcl_resultstring());
 
   Tcl_DStringFree(&list);
 
