@@ -4,11 +4,11 @@
  *
  *   IF YOU ALTER THIS FILE, YOU NEED TO RECOMPILE THE BOT.
  *
- * $Id: eggdrop.h,v 1.67 2008-02-16 21:41:03 guppy Exp $
+ * $Id: eggdrop.h,v 1.78 2010/06/29 15:52:24 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,18 +34,7 @@
  */
 #undef NO_OLD_BOTNET
 
-/*
- * Undefine this to completely disable context debugging.
- * WARNING: DO NOT send in bug reports if you undefine this!
- */
-#define DEBUG_CONTEXT
-
-/*
- * Set the following to the timestamp for the logfile entries.
- * Popular times might be "[%H:%M]" (hour, min), or "[%H:%M:%S]" (hour, min, sec)
- * Read `man strftime' for more formatting options.  Keep it below 32 chars.
- */
-#define LOG_TS "[%H:%M]"
+/* DEBUG_CONTEXT is now set in config.h by configure */
 
 /*
  * HANDLEN note:
@@ -158,6 +147,11 @@
 #  endif
 #endif
 
+#ifdef EGG_HANDLEN
+#  undef HANDLEN
+#  define HANDLEN EGG_HANDLEN
+#endif
+
 #if (NICKMAX < 9) || (NICKMAX > 32)
 #  include "Error: Invalid NICKMAX value."
 #endif
@@ -167,7 +161,7 @@
 #endif
 
 #if HANDLEN > NICKMAX
-#  include "Error: HANDLEN MUST BE <= NICKMAX."
+#  include "Error: HANDLEN must be <= NICKMAX."
 #endif
 
 #ifdef HAVE_SYS_PARAM_H
@@ -197,13 +191,63 @@
 #  endif
 #endif
 
-#ifndef HAVE_SRANDOM
-#  define srandom(x) srand(x)
-#endif
 
-#ifndef HAVE_RANDOM
-#  define random() (rand()/16)
-#endif
+/* Yikes...who would have thought finding a usable random() would be so much
+ * trouble?
+ * Note: random(), rand(), and lrand48() are *not* thread safe.
+ */
+#ifdef HAVE_RANDOM
+  /* On systems with random(), RANDOM_MAX may or may not be defined.
+   *
+   * If RANDOM_MAX isn't defined, we use 0x7FFFFFFF (2^31-1), or 2147483647
+   * since this follows the 4.3BSD and POSIX.1-2001 standards. This of course
+   * assumes random() uses a 32 bit long int type per the standards.
+   */
+#  ifndef RANDOM_MAX
+#    define RANDOM_MAX 0x7FFFFFFF  /* random() -- 2^31-1 */
+#  endif
+#else                              /* !HAVE_RANDOM */
+   /* This shouldn't exist in this case, but just to be safe... */
+#  ifdef RANDOM_MAX
+#    undef RANDOM_MAX
+#  endif
+  /* If we don't have random() it's safe to assume we also don't have
+   * srandom(), and we need both.
+   */
+#  ifdef HAVE_RAND
+#    define random() rand()
+#    define srandom(x) srand(x)
+    /* Depending on the system int size, RAND_MAX can be either 0x7FFFFFFF
+     * (2^31-1), or 2147483647 for a 32 bit int, or 0x7FFF (2^15-1), or
+     * 32767 for a 16 bit int. The standards only state that RAND_MAX must
+     * be _at least_ 32767 but some systems with 16 bit int define it as
+     * 32767. See: SVr4, 4.3BSD, C89, C99, POSIX.1-2001.
+     */
+#    define RANDOM_MAX RAND_MAX    /* rand() -- 2^31-1 or 2^15-1 */
+#  else                            /* !HAVE_RAND */
+#    ifdef  HAVE_LRAND48
+#      define random() lrand48()
+#      define srandom(x) srand48(x)
+      /* For lrand48() we define RANDOM_MAX as 0x7FFFFFFF (2^31-1), or
+       * 2147483647 since this is what the SVr4 and POSIX.1-2001 standards
+       * call for. Note: SVID 3 declares these functions as obsolete and
+       * states rand() should be used instead.
+       */
+#      define RANDOM_MAX 0x7FFFFFFF /* lrand48() -- 2^31-1 */
+#    else                          /* !HAVE_LRAND48 */
+#      include "Error: Must define one of HAVE_RANDOM, HAVE_RAND, or HAVE_LRAND48"
+#    endif                         /* HAVE_LRAND48 */
+#  endif                           /* HAVE_RAND */
+#endif                             /* HAVE_RANDOM */
+
+
+/* Use high-order bits for getting the random integer. With a modern
+ * random() implmentation, modulo would probably be sufficient, but on
+ * systems lacking random(), it may just be a macro for an older rand()
+ * fucntion.
+ */
+#define randint(n) (unsigned long) (random() / (RANDOM_MAX + 1.0) * n)
+
 
 #ifndef HAVE_SIGACTION /* old "weird signals" */
 #  define sigaction sigvec
@@ -234,8 +278,8 @@ typedef int socklen_t;
 #  define Context           eggContext(__FILE__, __LINE__, NULL)
 #  define ContextNote(note) eggContextNote(__FILE__, __LINE__, NULL, note)
 #else
-#  define Context           {}
-#  define ContextNote(note) {}
+#  define Context           do {} while (0)
+#  define ContextNote(note) do {} while (0)
 #endif
 
 #ifdef DEBUG_ASSERT
@@ -244,8 +288,7 @@ typedef int socklen_t;
             eggAssert(__FILE__, __LINE__, NULL);                        \
 } while (0)
 #else
-#  define Assert(expr) do {                                             \
-} while (0)
+#  define Assert(expr) do {} while (0)
 #endif
 
 #ifndef COMPILING_MEM
@@ -272,6 +315,7 @@ typedef unsigned char u_8bit_t;
 /* IP type */
 typedef u_32bit_t IP;
 
+/* Debug logging macros */
 #define debug0(x)             putlog(LOG_DEBUG,"*",x)
 #define debug1(x,a1)          putlog(LOG_DEBUG,"*",x,a1)
 #define debug2(x,a1,a2)       putlog(LOG_DEBUG,"*",x,a1,a2)
@@ -288,7 +332,8 @@ typedef u_32bit_t IP;
 /***********************************************************************/
 
 /* It's used in so many places, let's put it here */
-typedef int (*Function) ();
+typedef intptr_t (*Function) ();
+typedef int (*IntFunc) ();
 
 /* Public structure for the listening port map */
 struct portmap {
@@ -322,8 +367,8 @@ struct dcc_t {
   char nick[NICKLEN];
   char host[UHOSTLEN];
   struct dcc_table *type;
-  time_t timeval;               /* This is used for timeout checking.  */
-  unsigned long status;         /* A LOT of dcc types have status things; makes it more avaliabe. */
+  time_t timeval;               /* This is used for timeout checking. */
+  unsigned long status;         /* A LOT of dcc types have status things; makes it more availabe. */
   union {
     struct chat_info *chat;
     struct file_info *file;
@@ -478,8 +523,8 @@ struct dupwait_info {
 #define STRIP_REV    0x00004    /* remove reverse video codes           */
 #define STRIP_UNDER  0x00008    /* remove underline codes               */
 #define STRIP_ANSI   0x00010    /* remove ALL ANSI codes                */
-#define STRIP_BELLS  0x00020    /* remote ctrl-g's                      */
-#define STRIP_ALL    0x00040    /* remove every damn thing!             */
+#define STRIP_BELLS  0x00020    /* remove ctrl-g's                      */
+#define STRIP_ALL    0x0007F    /* remove every damn thing!             */
 
 /* For dcc bot links. */
 #define STAT_PINGED  0x00001    /* waiting for ping to return            */
@@ -492,7 +537,7 @@ struct dupwait_info {
 #define STAT_LEAF    0x00080    /* this bot is a leaf only               */
 #define STAT_LINKING 0x00100    /* the bot is currently going through
                                  * the linking stage                     */
-#define STAT_AGGRESSIVE   0x200 /* aggressively sharing with this bot    */
+#define STAT_AGGRESSIVE 0x00200 /* aggressively sharing with this bot    */
 
 /* Flags for listening sockets */
 #define LSTN_PUBLIC  0x000001   /* No access restrictions               */
@@ -528,8 +573,8 @@ typedef struct {
 
 /* Logfile display flags
  */
-#define LOG_MSGS     0x000001   /* m   msgs/notice/ctcps                */
-#define LOG_PUBLIC   0x000002   /* p   public msg/notice/ctcps          */
+#define LOG_MSGS     0x000001   /* m   private msgs/notices/ctcps       */
+#define LOG_PUBLIC   0x000002   /* p   public  msgs/notices/ctcps       */
 #define LOG_JOIN     0x000004   /* j   channel joins/parts/etc          */
 #define LOG_MODES    0x000008   /* k   mode changes/kicks/bans          */
 #define LOG_CMDS     0x000010   /* c   user dcc or msg commands         */
@@ -576,6 +621,7 @@ typedef struct {
                                  * of traffic                           */
 #define SOCK_VIRTUAL    0x0200  /* not-connected socket (dont read it!) */
 #define SOCK_BUFFER     0x0400  /* buffer data; don't notify dcc funcs  */
+#define SOCK_TCL        0x0800  /* tcl socket, don't do anything on it  */
 
 /* Flags to sock_has_data
  */
@@ -616,16 +662,29 @@ enum {
 #define HELP_TEXT       2
 #define HELP_IRC        16
 
-/* This is used by the net module to keep track of sockets and what's
+/* These are used by the net module to keep track of sockets and what's
  * queued on them
  */
-typedef struct {
-  int sock;
-  short flags;
+struct sock_handler {
   char *inbuf;
   char *outbuf;
   unsigned long outbuflen;      /* Outbuf could be binary data  */
   unsigned long inbuflen;       /* Inbuf could be binary data   */
+};
+
+struct tclsock_handler {
+  int mask;                     /* desired events               */
+  Tcl_FileProc *proc;
+  ClientData cd;
+};
+
+typedef struct sock_list {
+  int sock;
+  short flags;
+  union {
+    struct sock_handler sock;
+    struct tclsock_handler tclsock;
+  } handler;
 } sock_list;
 
 enum {
@@ -635,6 +694,8 @@ enum {
 
 /* Telnet codes.  See "TELNET Protocol Specification" (RFC 854) and
  * "TELNET Echo Option" (RFC 875) for details. */
+
+/* Telnet commands */
 #define TLN_AYT         246     /* Are You There        */
 #define TLN_WILL        251     /* Will                 */
 #define TLN_WILL_C      "\373"
@@ -647,7 +708,10 @@ enum {
 #define TLN_IAC         255     /* Interpret As Command */
 #define TLN_IAC_C       "\377"
 
-#define TLN_ECHO        1       /* Echo                 */
+/* Telnet options */
+#define TLN_ECHO        1       /* Echo (RFC 875)        */
 #define TLN_ECHO_C      "\001"
+#define TLN_STATUS      5       /* STATUS (RFC 859)      */
+#define TLN_STATUS_C    "\005"
 
 #endif /* _EGG_EGGDROP_H */
