@@ -2,11 +2,11 @@
  * flags.c -- handles:
  *   all the flag matching/conversion functions in one neat package :)
  *
- * $Id: flags.c,v 1.34 2008-02-16 21:41:03 guppy Exp $
+ * $Id: flags.c,v 1.36 2010/01/03 13:27:32 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -479,6 +479,8 @@ static int bot2str(char *string, int bot)
     x++;
     bot >>= 1;
   }
+  if (string == old)
+    *string++ = '-';
   return string - old;
 }
 
@@ -516,16 +518,6 @@ int build_flags(char *string, struct flag_record *plus,
       string += flag2str(string, minus->global, minus->udef_chan);
     }
   }
-  if ((plus->match & (FR_BOT | FR_CHAN)) == (FR_BOT | FR_CHAN)) {
-    *string++ = (plus->match & FR_AND) ? '&' : '|';
-    if (minus && plus->bot)
-      *string++ = '+';
-    string += bot2str(string, plus->bot);
-    if (minus && minus->bot) {
-      *string++ = '-';
-      string += bot2str(string, minus->bot);
-    }
-  }
   if (string == old) {
     *string++ = '-';
     *string = 0;
