@@ -1,11 +1,11 @@
 /*
  * flags.h
  *
- * $Id: flags.h,v 1.18 2008-02-16 21:41:03 guppy Exp $
+ * $Id: flags.h,v 1.20 2010/02/18 09:52:29 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,7 @@ struct flag_record {
   int match;
   int global;
   int udef_global;
-  int bot;
+  intptr_t bot;
   int chan;
   int udef_chan;
 };
