@@ -2,11 +2,11 @@
  * lang.h
  *   Conversion definitions for language support
  *
- * $Id: lang.h,v 1.37 2008-02-16 21:41:03 guppy Exp $
+ * $Id: lang.h,v 1.39 2010/01/03 13:27:32 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -95,7 +95,7 @@
 /* was: MISC_TRYINGMISTAKE      0x525            */
 #define MISC_PENDING            get_language(0x526)
 #define MISC_WANTOPS            get_language(0x527)
-#define MISC_LURKING            get_language(0x528)
+/* was: MISC_LURKING            0x528            */
 #define MISC_BACKGROUND         get_language(0x529)
 #define MISC_TERMMODE           get_language(0x52a)
 #define MISC_STATMODE           get_language(0x52b)
