@@ -5,11 +5,11 @@
  *   command line arguments
  *   context and assert debugging
  *
- * $Id: main.c,v 1.121 2008-02-16 21:41:03 guppy Exp $
+ * $Id: main.c,v 1.138 2010/07/12 15:40:52 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -50,7 +50,7 @@
 #  endif
 #endif
 
-#ifdef STOP_UAC                         /* osf/1 complains a lot */
+#ifdef STOP_UAC                         /* OSF/1 complains a lot */
 #  include <sys/sysinfo.h>
 #  define UAC_NOPRINT 0x00000001        /* Don't report unaligned fixups */
 #endif
@@ -60,8 +60,8 @@
 #include "tandem.h"
 #include "bg.h"
 
-#ifndef ENABLE_STRIP
-#  include <sys/resource.h>
+#ifdef DEBUG                            /* For debug compile */
+#  include <sys/resource.h>             /* setrlimit() */
 #endif
 
 #ifdef CYGWIN_HACKS
@@ -69,11 +69,12 @@
 #endif
 
 #ifndef _POSIX_SOURCE
-#  define _POSIX_SOURCE 1 /* Solaris needs this */
+#  define _POSIX_SOURCE 1               /* Solaris needs this */
 #endif
 
 extern char origbotname[], userfile[], botnetnick[];
-extern int dcc_total, conmask, cache_hit, cache_miss, max_logs, quick_logs;
+extern int dcc_total, conmask, cache_hit, cache_miss, max_logs, quick_logs,
+           quiet_save;
 extern struct dcc_t *dcc;
 extern struct userrec *userlist;
 extern struct chanset_t *chanset;
@@ -82,6 +83,8 @@ extern Tcl_Interp *interp;
 extern tcl_timer_t *timer, *utimer;
 extern sigjmp_buf alarmret;
 time_t now;
+static int argc;
+static char **argv;
 
 /*
  * Please use the PATCH macro instead of directly altering the version
@@ -90,8 +93,8 @@ time_t now;
  * modified versions of this bot.
  */
 
-char egg_version[1024] = "1.6.19";
-int egg_numver = 1061800;
+char egg_version[1024] = "1.6.20";
+int egg_numver = 1062000;
 
 char notify_new[121] = "";      /* Person to send a note to for new users */
 int default_flags = 0;          /* Default user flags                     */
@@ -222,10 +225,12 @@ static void check_expired_dcc()
     }
 }
 
-#ifdef DEBUG_CONTEXT
+#ifndef DEBUG_CONTEXT
+#define write_debug() do {} while (0)
+#else
 static int nested_debug = 0;
 
-void write_debug()
+static void write_debug()
 {
   int x;
   char s[25];
@@ -278,18 +283,17 @@ void write_debug()
     /* info library */
     dprintf(-x, "Tcl library: %s\n",
             ((interp) && (Tcl_Eval(interp, "info library") == TCL_OK)) ?
-            interp->result : "*unknown*");
+            tcl_resultstring() : "*unknown*");
 
     /* info tclversion/patchlevel */
     dprintf(-x, "Tcl version: %s (header version %s)\n",
             ((interp) && (Tcl_Eval(interp, "info patchlevel") == TCL_OK)) ?
-            interp->result : (Tcl_Eval(interp, "info tclversion") == TCL_OK) ?
-            interp->result : "*unknown*", TCL_PATCH_LEVEL ? TCL_PATCH_LEVEL :
+            tcl_resultstring() : (Tcl_Eval(interp, "info tclversion") == TCL_OK) ?
+            tcl_resultstring() : "*unknown*", TCL_PATCH_LEVEL ? TCL_PATCH_LEVEL :
             "*unknown*");
 
-#ifdef HAVE_TCL_THREADS
-    dprintf(-x, "Tcl is threaded\n");
-#endif
+    if (tcl_threaded())
+      dprintf(-x, "Tcl is threaded\n");
 
 #ifdef CCFLAGS
     dprintf(-x, "Compile flags: %s\n", CCFLAGS);
@@ -322,9 +326,7 @@ void write_debug()
 
 static void got_bus(int z)
 {
-#ifdef DEBUG_CONTEXT
   write_debug();
-#endif
   fatal("BUS ERROR -- CRASHING!", 1);
 #ifdef SA_RESETHAND
   kill(getpid(), SIGBUS);
@@ -336,9 +338,7 @@ static void got_bus(int z)
 
 static void got_segv(int z)
 {
-#ifdef DEBUG_CONTEXT
   write_debug();
-#endif
   fatal("SEGMENT VIOLATION -- CRASHING!", 1);
 #ifdef SA_RESETHAND
   kill(getpid(), SIGSEGV);
@@ -350,9 +350,7 @@ static void got_segv(int z)
 
 static void got_fpe(int z)
 {
-#ifdef DEBUG_CONTEXT
   write_debug();
-#endif
   fatal("FLOATING POINT ERROR -- CRASHING!", 0);
 }
 
@@ -407,7 +405,8 @@ static void got_ill(int z)
 }
 
 #ifdef DEBUG_CONTEXT
-/* Context */
+/* Called from the Context macro.
+ */
 void eggContext(const char *file, int line, const char *module)
 {
   char x[31], *p;
@@ -447,9 +446,7 @@ void eggContextNote(const char *file, int line, const char *module,
  */
 void eggAssert(const char *file, int line, const char *module)
 {
-#ifdef DEBUG_CONTEXT
   write_debug();
-#endif
   if (!module)
     putlog(LOG_MISC, "*", "* In file %s, line %u", file, line);
   else
@@ -506,7 +503,8 @@ void backup_userfile(void)
 {
   char s[125];
 
-  putlog(LOG_MISC, "*", USERF_BACKUP);
+  if (quiet_save < 2)
+    putlog(LOG_MISC, "*", USERF_BACKUP);
   egg_snprintf(s, sizeof s, "%s~bak", userfile);
   copyfile(userfile, s);
 }
@@ -570,7 +568,8 @@ static void core_secondly()
         int j;
 
         strncpyz(s, ctime(&now), sizeof s);
-        putlog(LOG_ALL, "*", "--- %.11s%s", s, s + 20);
+        if (quiet_save < 3)
+          putlog(LOG_ALL, "*", "--- %.11s%s", s, s + 20);
         call_hook(HOOK_BACKUP);
         for (j = 0; j < max_logs; j++) {
           if (logs[j].filename != NULL && logs[j].f != NULL) {
@@ -588,7 +587,8 @@ static void core_secondly()
     if (miltime == switch_logfiles_at) {
       call_hook(HOOK_DAILY);
       if (!keep_all_logs) {
-        putlog(LOG_MISC, "*", MISC_LOGSWITCH);
+        if (quiet_save < 3)
+          putlog(LOG_MISC, "*", MISC_LOGSWITCH);
         for (i = 0; i < max_logs; i++)
           if (logs[i].filename) {
             char s[1024];
@@ -609,6 +609,7 @@ static void core_secondly()
 static void core_minutely()
 {
   check_tcl_time(&nowtm);
+  check_tcl_cron(&nowtm);
   do_check_timers(&timer);
   if (quick_logs != 0) {
     flushlogs();
@@ -674,17 +675,16 @@ void check_static(char *, char *(*)());
 
 #include "mod/static.h"
 #endif
+int init_threaddata(int);
 int init_mem();
-int init_dcc_max();
 int init_userent();
 int init_misc();
 int init_bots();
-int init_net();
 int init_modules();
 int init_tcl(int, char **);
 int init_language(int);
 
-void patch(const char *str)
+static void patch(const char *str)
 {
   char *p = strchr(egg_version, '+');
 
@@ -705,48 +705,270 @@ static inline void garbage_collect(void)
     run_cnt++;
 }
 
-int main(int argc, char **argv)
+int mainloop(int toplevel)
 {
-  int xx, i;
-  char buf[520], s[25];
+  static int socket_cleanup = 0;
+  int xx, i, eggbusy = 1, tclbusy = 0;
+  char buf[520];
+
+  /* Lets move some of this here, reducing the numer of actual
+   * calls to periodic_timers
+   */
+  now = time(NULL);
+  /*
+   * FIXME: Get rid of this, it's ugly and wastes lots of cpu.
+   *
+   * pre-1.3.0 Eggdrop had random() in the once a second block below.
+   *
+   * This attempts to keep random() more random by constantly
+   * calling random() and updating the state information.
+   */
+  random();                /* Woop, lets really jumble things */
+
+  /* If we want to restart, we have to unwind to the toplevel.
+   * Tcl will Panic if we kill the interp with Tcl_Eval in progress.
+   * This is done by returning -1 in tickle_WaitForEvent.
+   */
+  if (do_restart && do_restart != -2 && !toplevel)
+    return -1;
+
+  /* Once a second */
+  if (now != then) {
+    call_hook(HOOK_SECONDLY);
+    then = now;
+  }
+
+  /* Only do this every so often. */
+  if (!socket_cleanup) {
+    socket_cleanup = 5;
+
+    /* Remove dead dcc entries. */
+    dcc_remove_lost();
+
+    /* Check for server or dcc activity. */
+    dequeue_sockets();
+  } else
+    socket_cleanup--;
+
+  /* Free unused structures. */
+  garbage_collect();
+
+  xx = sockgets(buf, &i);
+  if (xx >= 0) {              /* Non-error */
+    int idx;
+
+    for (idx = 0; idx < dcc_total; idx++)
+      if (dcc[idx].sock == xx) {
+        if (dcc[idx].type && dcc[idx].type->activity) {
+          /* Traffic stats */
+          if (dcc[idx].type->name) {
+            if (!strncmp(dcc[idx].type->name, "BOT", 3))
+              itraffic_bn_today += strlen(buf) + 1;
+            else if (!strcmp(dcc[idx].type->name, "SERVER"))
+              itraffic_irc_today += strlen(buf) + 1;
+            else if (!strncmp(dcc[idx].type->name, "CHAT", 4))
+              itraffic_dcc_today += strlen(buf) + 1;
+            else if (!strncmp(dcc[idx].type->name, "FILES", 5))
+              itraffic_dcc_today += strlen(buf) + 1;
+            else if (!strcmp(dcc[idx].type->name, "SEND"))
+              itraffic_trans_today += strlen(buf) + 1;
+            else if (!strncmp(dcc[idx].type->name, "GET", 3))
+              itraffic_trans_today += strlen(buf) + 1;
+            else
+              itraffic_unknown_today += strlen(buf) + 1;
+          }
+          dcc[idx].type->activity(idx, buf, i);
+        } else
+          putlog(LOG_MISC, "*",
+                 "!!! untrapped dcc activity: type %s, sock %d",
+                 dcc[idx].type->name, dcc[idx].sock);
+        break;
+      }
+  } else if (xx == -1) {        /* EOF from someone */
+    int idx;
+
+    if (i == STDOUT && !backgrd)
+      fatal("END OF FILE ON TERMINAL", 0);
+    for (idx = 0; idx < dcc_total; idx++)
+      if (dcc[idx].sock == i) {
+        if (dcc[idx].type && dcc[idx].type->eof)
+          dcc[idx].type->eof(idx);
+        else {
+          putlog(LOG_MISC, "*",
+                 "*** ATTENTION: DEAD SOCKET (%d) OF TYPE %s UNTRAPPED",
+                 i, dcc[idx].type ? dcc[idx].type->name : "*UNKNOWN*");
+          killsock(i);
+          lostdcc(idx);
+        }
+        idx = dcc_total + 1;
+      }
+    if (idx == dcc_total) {
+      putlog(LOG_MISC, "*",
+             "(@) EOF socket %d, not a dcc socket, not anything.", i);
+      close(i);
+      killsock(i);
+    }
+  } else if (xx == -2 && errno != EINTR) {      /* select() error */
+    putlog(LOG_MISC, "*", "* Socket error #%d; recovering.", errno);
+    for (i = 0; i < dcc_total; i++) {
+      if ((fcntl(dcc[i].sock, F_GETFD, 0) == -1) && (errno == EBADF)) {
+        putlog(LOG_MISC, "*",
+               "DCC socket %d (type %d, name '%s') expired -- pfft",
+               dcc[i].sock, dcc[i].type, dcc[i].nick);
+        killsock(dcc[i].sock);
+        lostdcc(i);
+        i--;
+      }
+    }
+  } else if (xx == -3) {
+    call_hook(HOOK_IDLE);
+    socket_cleanup = 0;       /* If we've been idle, cleanup & flush */
+    eggbusy = 0;
+  } else if (xx == -5) {
+    eggbusy = 0;
+    tclbusy = 1;
+  }
+
+  if (do_restart) {
+    if (do_restart == -2)
+      rehash();
+    else if (!toplevel)
+      return -1; /* Unwind to toplevel before restarting */
+    else {
+      /* Unload as many modules as possible */
+      int f = 1;
+      module_entry *p;
+      Function startfunc;
+      char name[256];
+
+      /* oops, I guess we should call this event before tcl is restarted */
+      check_tcl_event("prerestart");
+
+      while (f) {
+        f = 0;
+        for (p = module_list; p != NULL; p = p->next) {
+          dependancy *d = dependancy_list;
+          int ok = 1;
+
+          while (ok && d) {
+            if (d->needed == p)
+              ok = 0;
+            d = d->next;
+          }
+          if (ok) {
+            strcpy(name, p->name);
+            if (module_unload(name, botnetnick) == NULL) {
+              f = 1;
+              break;
+            }
+          }
+        }
+      }
+
+      /* Make sure we don't have any modules left hanging around other than
+       * "eggdrop" and the two that are supposed to be.
+       */
+      for (f = 0, p = module_list; p; p = p->next) {
+        if (strcmp(p->name, "eggdrop") && strcmp(p->name, "encryption") &&
+            strcmp(p->name, "uptime")) {
+          f++;
+        }
+      }
+      if (f != 0) {
+        putlog(LOG_MISC, "*", MOD_STAGNANT);
+      }
+
+      flushlogs();
+      kill_tcl();
+      init_tcl(argc, argv);
+      init_language(0);
+
+      /* this resets our modules which we didn't unload (encryption and uptime) */
+      for (p = module_list; p; p = p->next) {
+        if (p->funcs) {
+          startfunc = p->funcs[MODCALL_START];
+          startfunc(NULL);
+        }
+      }
+
+      rehash();
+      restart_chons();
+      call_hook(HOOK_LOADED);
+    }
+    eggbusy = 1;
+    do_restart = 0;
+  }
+
+#ifdef USE_TCL_EVENTS
+  if (!eggbusy) {
+/* Process all pending tcl events */
+#  ifdef REPLACE_NOTIFIER
+    if (Tcl_ServiceAll())
+      tclbusy = 1;
+#  else
+    while (Tcl_DoOneEvent(TCL_DONT_WAIT | TCL_ALL_EVENTS))
+      tclbusy = 1;
+#  endif /* REPLACE_NOTIFIER */
+#endif   /* USE_TCL_EVENTS   */
+  }
+
+  return (eggbusy || tclbusy);
+}
+
+int main(int arg_c, char **arg_v)
+{
+  int i, xx;
+  char s[25];
   FILE *f;
   struct sigaction sv;
   struct chanset_t *chan;
-
-#ifndef ENABLE_STRIP
-  /* Make sure it can write core, if you make debug. Else it's pretty
-   * useless (dw)
-   */
-  {
-    struct rlimit cdlim;
-
-    cdlim.rlim_cur = RLIM_INFINITY;
-    cdlim.rlim_max = RLIM_INFINITY;
-    setrlimit(RLIMIT_CORE, &cdlim);
-  }
+#ifdef DEBUG
+  struct rlimit cdlim;
+#endif
+#ifdef STOP_UAC
+  int nvpair[2];
 #endif
 
+/* Make sure it can write core, if you make debug. Else it's pretty
+ * useless (dw)
+ *
+ * Only allow unlimited size core files when compiled with DEBUG defined.
+ * This is not a good idea for normal builds -- in these cases, use the
+ * default system resource limits instead.
+ */
+#ifdef DEBUG
+  cdlim.rlim_cur = RLIM_INFINITY;
+  cdlim.rlim_max = RLIM_INFINITY;
+  setrlimit(RLIMIT_CORE, &cdlim);
+#endif
+
+#ifdef DEBUG_CONTEXT
   /* Initialise context list */
   for (i = 0; i < 16; i++)
     Context;
+#endif
 
+/* Include patch.h header for patch("...") */
 #include "patch.h"
+
+  argc = arg_c;
+  argv = arg_v;
+
   /* Version info! */
   egg_snprintf(ver, sizeof ver, "eggdrop v%s", egg_version);
   egg_snprintf(version, sizeof version,
-               "Eggdrop v%s (C) 1997 Robey Pointer (C) 2008 Eggheads",
+               "Eggdrop v%s (C) 1997 Robey Pointer (C) 2010 Eggheads",
                egg_version);
   /* Now add on the patchlevel (for Tcl) */
   sprintf(&egg_version[strlen(egg_version)], " %u", egg_numver);
   strcat(egg_version, egg_xtra);
-#ifdef STOP_UAC
-  {
-    int nvpair[2];
 
-    nvpair[0] = SSIN_UACPROC;
-    nvpair[1] = UAC_NOPRINT;
-    setsysinfo(SSI_NVPAIRS, (char *) nvpair, 1, NULL, 0);
-  }
+/* For OSF/1 */
+#ifdef STOP_UAC
+  /* Don't print "unaligned access fixup" warning to the user */
+  nvpair[0] = SSIN_UACPROC;
+  nvpair[1] = UAC_NOPRINT;
+  setsysinfo(SSI_NVPAIRS, (char *) nvpair, 1, NULL, 0);
 #endif
 
   /* Set up error traps: */
@@ -789,17 +1011,24 @@ int main(int argc, char **argv)
   if (argc > 1)
     for (i = 1; i < argc; i++)
       do_arg(argv[i]);
+
   printf("\n%s\n", version);
 
-  /* Don't allow eggdrop to run as root */
+#ifndef CYGWIN_HACKS
+  /* Don't allow eggdrop to run as root
+   * This check isn't useful under cygwin and has been
+   * reported to cause trouble in some situations.
+   */
   if (((int) getuid() == 0) || ((int) geteuid() == 0))
     fatal("ERROR: Eggdrop will not run as root!", 0);
+#endif
 
-  init_dcc_max();
+#ifndef REPLACE_NOTIFIER
+  init_threaddata(1);
+#endif
   init_userent();
   init_misc();
   init_bots();
-  init_net();
   init_modules();
   if (backgrd)
     bg_prepare_split();
@@ -832,9 +1061,9 @@ int main(int argc, char **argv)
   if (f != NULL) {
     fgets(s, 10, f);
     xx = atoi(s);
-    kill(xx, SIGCHLD);          /* Meaningless kill to determine if pid
+    i = kill(xx, SIGCHLD);      /* Meaningless kill to determine if pid
                                  * is used */
-    if (errno != ESRCH) {
+    if (i == 0 || errno != ESRCH) {
       printf(EGG_RUNNING1, botnetnick);
       printf(EGG_RUNNING2, pid_file);
       bg_send_quit(BG_ABORT);
@@ -844,10 +1073,8 @@ int main(int argc, char **argv)
 
   /* Move into background? */
   if (backgrd) {
-#ifndef CYGWIN_HACKS
     bg_do_split();
   } else {                        /* !backgrd */
-#endif
     xx = getpid();
     if (xx != 0) {
       FILE *fp;
@@ -866,16 +1093,13 @@ int main(int argc, char **argv)
           fclose(fp);
       } else
         printf(EGG_NOWRITE, pid_file);
-#ifdef CYGWIN_HACKS
-      printf("Launched into the background  (pid: %d)\n\n", xx);
-#endif
     }
   }
 
   use_stderr = 0;               /* Stop writing to stderr now */
   if (backgrd) {
     /* Ok, try to disassociate from controlling terminal (finger cross) */
-#if defined(HAVE_SETPGID) && !defined(CYGWIN_HACKS)
+#ifdef HAVE_SETPGID
     setpgid(0, 0);
 #endif
     /* Tcl wants the stdin, stdout and stderr file handles kept open. */
@@ -932,176 +1156,6 @@ int main(int argc, char **argv)
 
   debug0("main: entering loop");
   while (1) {
-    int socket_cleanup = 0;
-
-#ifdef USE_TCL_EVENTS
-    /* Process a single tcl event */
-    Tcl_DoOneEvent(TCL_ALL_EVENTS | TCL_DONT_WAIT);
-#endif /* USE_TCL_EVENTS */
-
-    /* Lets move some of this here, reducing the numer of actual
-     * calls to periodic_timers
-     */
-    now = time(NULL);
-    random();                   /* Woop, lets really jumble things */
-    if (now != then) {          /* Once a second */
-      call_hook(HOOK_SECONDLY);
-      then = now;
-    }
-
-    /* Only do this every so often. */
-    if (!socket_cleanup) {
-      socket_cleanup = 5;
-
-      /* Remove dead dcc entries. */
-      dcc_remove_lost();
-
-      /* Check for server or dcc activity. */
-      dequeue_sockets();
-    } else
-      socket_cleanup--;
-
-    /* Free unused structures. */
-    garbage_collect();
-
-    xx = sockgets(buf, &i);
-    if (xx >= 0) {              /* Non-error */
-      int idx;
-
-      for (idx = 0; idx < dcc_total; idx++)
-        if (dcc[idx].sock == xx) {
-          if (dcc[idx].type && dcc[idx].type->activity) {
-            /* Traffic stats */
-            if (dcc[idx].type->name) {
-              if (!strncmp(dcc[idx].type->name, "BOT", 3))
-                itraffic_bn_today += strlen(buf) + 1;
-              else if (!strcmp(dcc[idx].type->name, "SERVER"))
-                itraffic_irc_today += strlen(buf) + 1;
-              else if (!strncmp(dcc[idx].type->name, "CHAT", 4))
-                itraffic_dcc_today += strlen(buf) + 1;
-              else if (!strncmp(dcc[idx].type->name, "FILES", 5))
-                itraffic_dcc_today += strlen(buf) + 1;
-              else if (!strcmp(dcc[idx].type->name, "SEND"))
-                itraffic_trans_today += strlen(buf) + 1;
-              else if (!strncmp(dcc[idx].type->name, "GET", 3))
-                itraffic_trans_today += strlen(buf) + 1;
-              else
-                itraffic_unknown_today += strlen(buf) + 1;
-            }
-            dcc[idx].type->activity(idx, buf, i);
-          } else
-            putlog(LOG_MISC, "*",
-                   "!!! untrapped dcc activity: type %s, sock %d",
-                   dcc[idx].type->name, dcc[idx].sock);
-          break;
-        }
-    } else if (xx == -1) {        /* EOF from someone */
-      int idx;
-
-      if (i == STDOUT && !backgrd)
-        fatal("END OF FILE ON TERMINAL", 0);
-      for (idx = 0; idx < dcc_total; idx++)
-        if (dcc[idx].sock == i) {
-          if (dcc[idx].type && dcc[idx].type->eof)
-            dcc[idx].type->eof(idx);
-          else {
-            putlog(LOG_MISC, "*",
-                   "*** ATTENTION: DEAD SOCKET (%d) OF TYPE %s UNTRAPPED",
-                   i, dcc[idx].type ? dcc[idx].type->name : "*UNKNOWN*");
-            killsock(i);
-            lostdcc(idx);
-          }
-          idx = dcc_total + 1;
-        }
-      if (idx == dcc_total) {
-        putlog(LOG_MISC, "*",
-               "(@) EOF socket %d, not a dcc socket, not anything.", i);
-        close(i);
-        killsock(i);
-      }
-    } else if (xx == -2 && errno != EINTR) {      /* select() error */
-      putlog(LOG_MISC, "*", "* Socket error #%d; recovering.", errno);
-      for (i = 0; i < dcc_total; i++) {
-        if ((fcntl(dcc[i].sock, F_GETFD, 0) == -1) && (errno == EBADF)) {
-          putlog(LOG_MISC, "*",
-                 "DCC socket %d (type %d, name '%s') expired -- pfft",
-                 dcc[i].sock, dcc[i].type, dcc[i].nick);
-          killsock(dcc[i].sock);
-          lostdcc(i);
-          i--;
-        }
-      }
-    } else if (xx == -3) {
-      call_hook(HOOK_IDLE);
-      socket_cleanup = 0;       /* If we've been idle, cleanup & flush */
-    }
-
-    if (do_restart) {
-      if (do_restart == -2)
-        rehash();
-      else {
-        /* Unload as many modules as possible */
-        int f = 1;
-        module_entry *p;
-        Function startfunc;
-        char name[256];
-
-        /* oops, I guess we should call this event before tcl is restarted */
-        check_tcl_event("prerestart");
-
-        while (f) {
-          f = 0;
-          for (p = module_list; p != NULL; p = p->next) {
-            dependancy *d = dependancy_list;
-            int ok = 1;
-
-            while (ok && d) {
-              if (d->needed == p)
-                ok = 0;
-              d = d->next;
-            }
-            if (ok) {
-              strcpy(name, p->name);
-              if (module_unload(name, botnetnick) == NULL) {
-                f = 1;
-                break;
-              }
-            }
-          }
-        }
-
-        /* Make sure we don't have any modules left hanging around other than
-         * "eggdrop" and the two that are supposed to be.
-         */
-        for (f = 0, p = module_list; p; p = p->next) {
-          if (strcmp(p->name, "eggdrop") && strcmp(p->name, "encryption") &&
-              strcmp(p->name, "uptime")) {
-            f++;
-          }
-        }
-        if (f != 0) {
-          putlog(LOG_MISC, "*", MOD_STAGNANT);
-        }
-
-        flushlogs();
-        kill_tcl();
-        init_tcl(argc, argv);
-        init_language(0);
-
-        /* this resets our modules which we didn't unload (encryption and uptime) */
-        for (p = module_list; p; p = p->next) {
-          if (p->funcs) {
-            startfunc = p->funcs[MODCALL_START];
-            startfunc(NULL);
-          }
-        }
-
-        rehash();
-        restart_chons();
-        call_hook(HOOK_LOADED);
-      }
-
-      do_restart = 0;
-    }
+    mainloop(1);
   }
 }
