@@ -2,11 +2,11 @@
  * main.h
  *   include file to include most other include files
  *
- * $Id: main.h,v 1.36 2008-02-16 21:41:03 guppy Exp $
+ * $Id: main.h,v 1.41 2010/06/29 15:52:24 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -48,6 +48,10 @@
 #  define USE_TCL_ENCODING
 #endif
 
+#if defined(HAVE_TCL_SETNOTIFIER) && defined(HAVE_TCL_GETTHREADDATA)
+#  define REPLACE_NOTIFIER
+#endif
+
 #if (((TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION >= 4)) || (TCL_MAJOR_VERSION > 8))
 #  ifdef CONST
 #    define EGG_CONST CONST
@@ -58,14 +62,16 @@
 #  define EGG_CONST
 #endif
 
+#ifdef CONST86
+#  define TCL_CONST86 CONST86
+#else
+#  define TCL_CONST86
+#endif
+
 /* UGH! Why couldn't Tcl pick a standard? */
 #if defined(USE_TCL_VARARGS) && (defined(__STDC__) || defined(HAS_STDARG))
 #  ifdef HAVE_STDARG_H
 #    include <stdarg.h>
-#  else
-#    ifdef HAVE_STD_ARGS_H
-#      include <std_args.h>
-#    endif
 #  endif
 #  define EGG_VARARGS(type, name) (type name, ...)
 #  define EGG_VARARGS_DEF(type, name) (type name, ...)
@@ -87,6 +93,10 @@
 #  include <strings.h>
 #endif
 
+#ifdef HAVE_INTTYPES_H
+#  include <inttypes.h>
+#endif
+
 #include <sys/types.h>
 #include "lush.h"
 #include "lang.h"
@@ -142,10 +152,4 @@ extern struct dcc_table DCC_CHAT, DCC_BOT, DCC_LOST, DCC_SCRIPT, DCC_BOT_NEW,
 #  define O_NONBLOCK 00000004 /* POSIX non-blocking I/O */
 #endif /* BORGCUBES */
 
-/* Use high-order bits for getting the random integer. With random()
- * modulo would probably be sufficient but on systems lacking random(),
- * the function will be just renamed rand().
- */
-#define randint(n) (unsigned long) (random() / (RANDOM_MAX + 1.0) * ((n) < 0 ? (-(n)) : (n)))
-
 #endif /* _EGG_MAIN_H */
