@@ -2,11 +2,11 @@
  * channels.c -- part of channels.mod
  *   support for channels within the bot
  *
- * $Id: channels.c,v 1.96 2008-02-16 21:41:06 guppy Exp $
+ * $Id: channels.c,v 1.107 2010/07/10 14:54:49 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,10 +36,10 @@ static char *lastdeletedmask;
 
 static struct udef_struct *udef;
 
-static int setstatic, use_info, chan_hack, quiet_save, global_revenge_mode,
+static int use_info, chan_hack, quiet_save, global_revenge_mode,
            global_stopnethack_mode, global_idle_kick, global_aop_min,
            global_aop_max, global_ban_time, global_exempt_time,
-           global_invite_time;
+           global_invite_time, global_ban_type, allow_ps;
 
 /* Global channel settings (drummer/dw) */
 static char glob_chanset[512];
@@ -168,7 +168,7 @@ static void set_mode_protect(struct chanset_t *chan, char *set)
     }
   }
   /* Prevents a +s-p +p-s flood  (fixed by drummer) */
-  if (chan->mode_pls_prot & CHANSEC)
+  if (chan->mode_pls_prot & CHANSEC && !allow_ps)
     chan->mode_pls_prot &= ~CHANPRIV;
 }
 
@@ -258,7 +258,7 @@ static int ismodeline(masklist *m, char *user)
 static int ismasked(masklist *m, char *user)
 {
   for (; m && m->mask[0]; m = m->next)
-    if (wild_match(m->mask, user))
+    if (match_addr(m->mask, user))
       return 1;
   return 0;
 }
@@ -410,19 +410,19 @@ static void write_channels()
     convert_element(chan->need_key, need3);
     convert_element(chan->need_unban, need4);
     convert_element(chan->need_limit, need5);
-    /* Do not indent me (adds extra spaces to chan file). */
     fprintf(f,
-"channel %s %s%schanmode %s idle-kick %d stopnethack-mode %d revenge-mode %d \
-need-op %s need-invite %s need-key %s need-unban %s need-limit %s \
-flood-chan %d:%d flood-ctcp %d:%d flood-join %d:%d flood-kick %d:%d \
-flood-deop %d:%d flood-nick %d:%d aop-delay %d:%d ban-time %d exempt-time %d \
-invite-time %d %cenforcebans %cdynamicbans %cuserbans %cautoop %cautohalfop \
-%cbitch %cgreet %cprotectops %cprotecthalfops %cprotectfriends %cdontkickops \
-%cstatuslog %crevenge %crevengebot %cautovoice %csecret %cshared %ccycle \
-%cseen %cinactive %cdynamicexempts %cuserexempts %cdynamicinvites \
-%cuserinvites %cnodesynch ",
-            channel_static(chan) ? "set" : "add", name, channel_static(chan) ?
-            " " : " { ", w2, chan->idle_kick, chan->stopnethack_mode,
+            "channel add %s { chanmode %s idle-kick %d stopnethack-mode %d "
+            "revenge-mode %d need-op %s need-invite %s need-key %s "
+            "need-unban %s need-limit %s flood-chan %d:%d flood-ctcp %d:%d "
+            "flood-join %d:%d flood-kick %d:%d flood-deop %d:%d "
+            "flood-nick %d:%d aop-delay %d:%d ban-type %d ban-time %d "
+            "exempt-time %d invite-time %d %cenforcebans %cdynamicbans "
+            "%cuserbans %cautoop %cautohalfop %cbitch %cgreet %cprotectops "
+            "%cprotecthalfops %cprotectfriends %cdontkickops %cstatuslog "
+            "%crevenge %crevengebot %cautovoice %csecret %cshared %ccycle "
+            "%cseen %cinactive %cdynamicexempts %cuserexempts %cdynamicinvites "
+            "%cuserinvites %cnodesynch %cstatic }" "\n",
+            name, w2, chan->idle_kick, chan->stopnethack_mode,
             chan->revenge_mode, need1, need2, need3, need4, need5,
             chan->flood_pub_thr, chan->flood_pub_time,
             chan->flood_ctcp_thr, chan->flood_ctcp_time,
@@ -430,7 +430,7 @@ invite-time %d %cenforcebans %cdynamicbans %cuserbans %cautoop %cautohalfop \
             chan->flood_kick_thr, chan->flood_kick_time,
             chan->flood_deop_thr, chan->flood_deop_time,
             chan->flood_nick_thr, chan->flood_nick_time,
-            chan->aop_min, chan->aop_max, chan->ban_time,
+            chan->aop_min, chan->aop_max, chan->ban_type, chan->ban_time,
             chan->exempt_time, chan->invite_time,
             PLSMNS(channel_enforcebans(chan)),
             PLSMNS(channel_dynamicbans(chan)),
@@ -456,16 +456,16 @@ invite-time %d %cenforcebans %cdynamicbans %cuserbans %cautoop %cautohalfop \
             PLSMNS(!channel_nouserexempts(chan)),
             PLSMNS(channel_dynamicinvites(chan)),
             PLSMNS(!channel_nouserinvites(chan)),
-            PLSMNS(channel_nodesynch(chan)));
-    fprintf(f, "%s\n", channel_static(chan) ? "" : "}");
+            PLSMNS(channel_nodesynch(chan)),
+            PLSMNS(channel_static(chan)));
     for (ul = udef; ul; ul = ul->next) {
       if (ul->defined && ul->name) {
         if (ul->type == UDEF_FLAG)
-          fprintf(f, "channel set %s %c%s%s\n", name, getudef(ul->values, chan->dname) ? '+' : '-',
-                  "udef-flag-", ul->name);
+          fprintf(f, "channel set %s %c%s%s\n", name, getudef(ul->values,
+                  chan->dname) ? '+' : '-', "udef-flag-", ul->name);
         else if (ul->type == UDEF_INT)
-          fprintf(f, "channel set %s %s%s %d\n", name, "udef-int-", ul->name, getudef(ul->values,
-                  chan->dname));
+          fprintf(f, "channel set %s %s%s %d\n", name, "udef-int-", ul->name,
+                  (int) getudef(ul->values, chan->dname));
         else if (ul->type == UDEF_STR) {
           char *p = (char *) getudef(ul->values, chan->dname);
 
@@ -488,15 +488,17 @@ invite-time %d %cenforcebans %cdynamicbans %cuserbans %cautoop %cautohalfop \
   movefile(s, chanfile);
 }
 
-static void read_channels(int create)
+static void read_channels(int create, int reload)
 {
   struct chanset_t *chan, *chan_next;
 
   if (!chanfile[0])
     return;
-  for (chan = chanset; chan; chan = chan->next)
-    if (!channel_static(chan))
+
+  if (reload)
+    for (chan = chanset; chan; chan = chan->next)
       chan->status |= CHAN_FLAGGED;
+
   chan_hack = 1;
   if (!readtclprog(chanfile) && create) {
     FILE *f;
@@ -511,6 +513,8 @@ static void read_channels(int create)
       fclose(f);
   }
   chan_hack = 0;
+  if (!reload)
+    return;
   for (chan = chanset; chan; chan = chan_next) {
     chan_next = chan->next;
     if (chan->status & CHAN_FLAGGED) {
@@ -524,175 +528,192 @@ static void backup_chanfile()
 {
   char s[125];
 
-  putlog(LOG_MISC, "*", "Backing up channel file...");
+  if (quiet_save < 2)
+    putlog(LOG_MISC, "*", "Backing up channel file...");
   egg_snprintf(s, sizeof s, "%s~bak", chanfile);
   copyfile(chanfile, s);
 }
 
 static void channels_prerehash()
 {
-  struct chanset_t *chan;
-
-  /* Flag will be cleared as the channels are re-added by the
-   * config file. Any still flagged afterwards will be removed.
-   */
-  for (chan = chanset; chan; chan = chan->next) {
-    chan->status |= CHAN_FLAGGED;
-    /* Flag is only added to channels read from config file */
-    if (chan->status & CHAN_STATIC)
-      chan->status &= ~CHAN_STATIC;
-  }
-  setstatic = 1;
+  write_channels();
 }
 
 static void channels_rehash()
 {
-  struct chanset_t *chan;
-
-  setstatic = 0;
-  read_channels(1);
-  /* Remove any extra channels, by checking the flag. */
-  chan = chanset;
-  for (chan = chanset; chan;) {
-    if (chan->status & CHAN_FLAGGED) {
-      putlog(LOG_MISC, "*", "No longer supporting channel %s", chan->dname);
-      remove_channel(chan);
-      chan = chanset;
-    } else
-      chan = chan->next;
-  }
+  /* add channels from the chanfile but don't remove missing ones */
+  read_channels(1, 0);
+  write_channels();
 }
 
 static cmd_t my_chon[] = {
-  {"*",  "",   (Function) channels_chon, "channels:chon"},
+  {"*",  "",   (IntFunc) channels_chon, "channels:chon"},
   {NULL, NULL, NULL,                                NULL}
 };
 
 static void channels_report(int idx, int details)
 {
-  struct chanset_t *chan;
   int i;
-  char s[1024], s2[100];
+  char s[1024], s1[100], s2[100];
+  struct chanset_t *chan;
   struct flag_record fr = { FR_CHAN | FR_GLOBAL, 0, 0, 0, 0, 0 };
 
   for (chan = chanset; chan; chan = chan->next) {
+
+    /* Get user's flags if output isn't going to stdout */
     if (idx != DP_STDOUT)
       get_user_flagrec(dcc[idx].user, &fr, chan->dname);
-    if ((idx == DP_STDOUT) || glob_master(fr) || chan_master(fr)) {
-      s[0] = 0;
-      if (channel_greet(chan))
-        strcat(s, "greet, ");
-      if (channel_autoop(chan))
-        strcat(s, "auto-op, ");
-      if (channel_bitch(chan))
-        strcat(s, "bitch, ");
-      if (s[0])
-        s[strlen(s) - 2] = 0;
-      if (!s[0])
-        strcpy(s, MISC_LURKING);
+
+    /* Don't show channel information to someone who isn't a master */
+    if ((idx != DP_STDOUT) && !glob_master(fr) && !chan_master(fr))
+      continue;
+
+    s[0] = 0;
+
+    sprintf(s, "    %-20s: ", chan->dname);
+
+    if (channel_inactive(chan))
+      strcat(s, "(inactive)");
+    else if (channel_pending(chan))
+      strcat(s, "(pending)");
+    else if (!channel_active(chan))
+      strcat(s, "(not on channel)");
+    else {
+
+      s1[0] = 0;
+      sprintf(s1, "%3d member%s", chan->channel.members,
+              (chan->channel.members == 1) ? "" : "s");
+      strcat(s, s1);
+
+      s2[0] = 0;
       get_mode_protect(chan, s2);
-      if (!channel_inactive(chan)) {
-        if (channel_active(chan)) {
-          /* If it's a !chan, we want to display it's unique name too <cybah> */
-          if (chan->dname[0] == '!') {
-            dprintf(idx, "    %-20s: %3d member%s enforcing \"%s\" (%s), "
-                    "unique name %s\n", chan->dname, chan->channel.members,
-                    (chan->channel.members == 1) ? "," : "s,", s2, s,
-                    chan->name);
-          } else {
-            dprintf(idx, "    %-20s: %3d member%s enforcing \"%s\" (%s)\n",
-                    chan->dname, chan->channel.members,
-                    chan->channel.members == 1 ? "," : "s,", s2, s);
-          }
-        } else {
-          dprintf(idx, "    %-20s: (%s), enforcing \"%s\"  (%s)\n", chan->dname,
-                  channel_pending(chan) ? "pending" : "not on channel", s2, s);
-        }
-      } else {
-        dprintf(idx, "    %-20s: channel is set +inactive\n", chan->dname);
+
+      if (s2[0]) {
+        s1[0] = 0;
+        sprintf(s1, ", enforcing \"%s\"", s2);
+        strcat(s, s1);
       }
-      if (details) {
-        s[0] = 0;
-        i = 0;
-        if (channel_enforcebans(chan))
-          i += my_strcpy(s + i, "enforcebans ");
-        if (channel_dynamicbans(chan))
-          i += my_strcpy(s + i, "dynamicbans ");
-        if (!channel_nouserbans(chan))
-          i += my_strcpy(s + i, "userbans ");
-        if (channel_autoop(chan))
-          i += my_strcpy(s + i, "autoop ");
-        if (channel_bitch(chan))
-          i += my_strcpy(s + i, "bitch ");
-        if (channel_greet(chan))
-          i += my_strcpy(s + i, "greet ");
-        if (channel_protectops(chan))
-          i += my_strcpy(s + i, "protectops ");
-        if (channel_protecthalfops(chan))
-          i += my_strcpy(s + i, "protecthalfops ");
-        if (channel_protectfriends(chan))
-          i += my_strcpy(s + i, "protectfriends ");
-        if (channel_dontkickops(chan))
-          i += my_strcpy(s + i, "dontkickops ");
-        if (channel_logstatus(chan))
-          i += my_strcpy(s + i, "statuslog ");
-        if (channel_revenge(chan))
-          i += my_strcpy(s + i, "revenge ");
-        if (channel_revenge(chan))
-          i += my_strcpy(s + i, "revengebot ");
-        if (channel_secret(chan))
-          i += my_strcpy(s + i, "secret ");
-        if (channel_shared(chan))
-          i += my_strcpy(s + i, "shared ");
-        if (!channel_static(chan))
-          i += my_strcpy(s + i, "dynamic ");
-        if (channel_autovoice(chan))
-          i += my_strcpy(s + i, "autovoice ");
-        if (channel_autohalfop(chan))
-          i += my_strcpy(s + i, "autohalfop ");
-        if (channel_cycle(chan))
-          i += my_strcpy(s + i, "cycle ");
-        if (channel_seen(chan))
-          i += my_strcpy(s + i, "seen ");
-        if (channel_dynamicexempts(chan))
-          i += my_strcpy(s + i, "dynamicexempts ");
-        if (!channel_nouserexempts(chan))
-          i += my_strcpy(s + i, "userexempts ");
-        if (channel_dynamicinvites(chan))
-          i += my_strcpy(s + i, "dynamicinvites ");
-        if (!channel_nouserinvites(chan))
-          i += my_strcpy(s + i, "userinvites ");
-        if (channel_inactive(chan))
-          i += my_strcpy(s + i, "inactive ");
-        if (channel_nodesynch(chan))
-          i += my_strcpy(s + i, "nodesynch ");
-        dprintf(idx, "      Options: %s\n", s);
-        if (chan->need_op[0])
-          dprintf(idx, "      To get ops, I do: %s\n", chan->need_op);
-        if (chan->need_invite[0])
-          dprintf(idx, "      To get invited, I do: %s\n", chan->need_invite);
-        if (chan->need_limit[0])
-          dprintf(idx, "      To get the channel limit raised, I do: %s\n",
-                  chan->need_limit);
-        if (chan->need_unban[0])
-          dprintf(idx, "      To get unbanned, I do: %s\n", chan->need_unban);
-        if (chan->need_key[0])
-          dprintf(idx, "      To get the channel key, I do: %s\n",
-                  chan->need_key);
-        if (chan->idle_kick)
-          dprintf(idx, "      Kicking idle users after %d minute%s\n",
-                  chan->idle_kick, (chan->idle_kick != 1) ? "s" : "");
-        if (chan->stopnethack_mode)
-          dprintf(idx, "      stopnethack-mode: %d\n", chan->stopnethack_mode);
-        if (chan->revenge_mode)
-          dprintf(idx, "      revenge-mode: %d\n", chan->revenge_mode);
-        dprintf(idx, "      Bans last %d minute%s.\n", chan->ban_time,
-                 (chan->ban_time != 1) ? "s" : "");
-        dprintf(idx, "      Exemptions last %d minute%s.\n", chan->exempt_time,
-                 (chan->exempt_time != 1) ? "s" : "");
-        dprintf(idx, "      Invitations last %d minute%s.\n", chan->invite_time,
-                 (chan->invite_time != 1) ? "s" : "");
+
+      s2[0] = 0;
+
+      if (channel_greet(chan))
+        strcat(s2, "greet, ");
+      if (channel_autoop(chan))
+        strcat(s2, "auto-op, ");
+      if (channel_bitch(chan))
+        strcat(s2, "bitch, ");
+
+      if (s2[0]) {
+        s2[strlen(s2) - 2] = 0;
+
+        s1[0] = 0;
+        sprintf(s1, " (%s)", s2);
+        strcat(s, s1);
       }
+
+      /* If it's a !chan, we want to display it's unique name too <cybah> */
+      if (chan->dname[0] == '!') {
+        s1[0] = 0;
+        sprintf(s1, ", unique name %s", chan->name);
+        strcat(s, s1);
+      }
+    }
+
+    dprintf(idx, "%s\n", s);
+
+    if (details) {
+      s[0] = 0;
+      i = 0;
+
+      if (channel_enforcebans(chan))
+        i += my_strcpy(s + i, "enforcebans ");
+      if (channel_dynamicbans(chan))
+        i += my_strcpy(s + i, "dynamicbans ");
+      if (!channel_nouserbans(chan))
+        i += my_strcpy(s + i, "userbans ");
+      if (channel_autoop(chan))
+        i += my_strcpy(s + i, "autoop ");
+      if (channel_bitch(chan))
+        i += my_strcpy(s + i, "bitch ");
+      if (channel_greet(chan))
+        i += my_strcpy(s + i, "greet ");
+      if (channel_protectops(chan))
+        i += my_strcpy(s + i, "protectops ");
+      if (channel_protecthalfops(chan))
+        i += my_strcpy(s + i, "protecthalfops ");
+      if (channel_protectfriends(chan))
+        i += my_strcpy(s + i, "protectfriends ");
+      if (channel_dontkickops(chan))
+        i += my_strcpy(s + i, "dontkickops ");
+      if (channel_logstatus(chan))
+        i += my_strcpy(s + i, "statuslog ");
+      if (channel_revenge(chan))
+        i += my_strcpy(s + i, "revenge ");
+      if (channel_revenge(chan))
+        i += my_strcpy(s + i, "revengebot ");
+      if (channel_secret(chan))
+        i += my_strcpy(s + i, "secret ");
+      if (channel_shared(chan))
+        i += my_strcpy(s + i, "shared ");
+      if (!channel_static(chan))
+        i += my_strcpy(s + i, "dynamic ");
+      if (channel_autovoice(chan))
+        i += my_strcpy(s + i, "autovoice ");
+      if (channel_autohalfop(chan))
+        i += my_strcpy(s + i, "autohalfop ");
+      if (channel_cycle(chan))
+        i += my_strcpy(s + i, "cycle ");
+      if (channel_seen(chan))
+        i += my_strcpy(s + i, "seen ");
+      if (channel_dynamicexempts(chan))
+        i += my_strcpy(s + i, "dynamicexempts ");
+      if (!channel_nouserexempts(chan))
+        i += my_strcpy(s + i, "userexempts ");
+      if (channel_dynamicinvites(chan))
+        i += my_strcpy(s + i, "dynamicinvites ");
+      if (!channel_nouserinvites(chan))
+        i += my_strcpy(s + i, "userinvites ");
+      if (channel_inactive(chan))
+        i += my_strcpy(s + i, "inactive ");
+      if (channel_nodesynch(chan))
+        i += my_strcpy(s + i, "nodesynch ");
+
+      dprintf(idx, "      Options: %s\n", s);
+
+      if (chan->need_op[0])
+        dprintf(idx, "      To get ops, I do: %s\n", chan->need_op);
+
+      if (chan->need_invite[0])
+        dprintf(idx, "      To get invited, I do: %s\n", chan->need_invite);
+
+      if (chan->need_limit[0])
+        dprintf(idx, "      To get the channel limit raised, I do: %s\n",
+                chan->need_limit);
+
+      if (chan->need_unban[0])
+        dprintf(idx, "      To get unbanned, I do: %s\n", chan->need_unban);
+
+      if (chan->need_key[0])
+        dprintf(idx, "      To get the channel key, I do: %s\n",
+                chan->need_key);
+
+      if (chan->idle_kick)
+        dprintf(idx, "      Kicking idle users after %d minute%s\n",
+                chan->idle_kick, (chan->idle_kick != 1) ? "s" : "");
+
+      if (chan->stopnethack_mode)
+        dprintf(idx, "      stopnethack-mode: %d\n", chan->stopnethack_mode);
+
+      if (chan->revenge_mode)
+        dprintf(idx, "      revenge-mode: %d\n", chan->revenge_mode);
+
+      dprintf(idx, "      ban-type: %d\n", chan->ban_type);
+      dprintf(idx, "      Bans last %d minute%s.\n", chan->ban_time,
+               (chan->ban_time == 1) ? "" : "s");
+      dprintf(idx, "      Exemptions last %d minute%s.\n", chan->exempt_time,
+               (chan->exempt_time == 1) ? "" : "s");
+      dprintf(idx, "      Invitations last %d minute%s.\n", chan->invite_time,
+               (chan->invite_time == 1) ? "" : "s");
     }
   }
 }
@@ -785,12 +806,14 @@ static tcl_ints my_tcl_ints[] = {
   {"share-greet",             NULL,                     0},
   {"use-info",                &use_info,                0},
   {"quiet-save",              &quiet_save,              0},
+  {"allow-ps",                &allow_ps,                0},
   {"global-stopnethack-mode", &global_stopnethack_mode, 0},
   {"global-revenge-mode",     &global_revenge_mode,     0},
   {"global-idle-kick",        &global_idle_kick,        0},
   {"global-ban-time",         &global_ban_time,         0},
   {"global-exempt-time",      &global_exempt_time,      0},
   {"global-invite-time",      &global_invite_time,      0},
+  {"global-ban-type",         &global_ban_type,         0},
   /* keeping [ban|exempt|invite]-time for compatability <Wcc[07/20/02]> */
   {"ban-time",                &global_ban_time,         0},
   {"exempt-time",             &global_exempt_time,      0},
@@ -927,7 +950,7 @@ char *channels_start(Function *global_funcs)
   global_idle_kick = 0;
   global_aop_min = 5;
   global_aop_max = 30;
-  setstatic = 0;
+  allow_ps = 0;
   lastdeletedmask = 0;
   use_info = 1;
   strcpy(chanfile, "chanfile");
@@ -937,6 +960,7 @@ char *channels_start(Function *global_funcs)
   udef = NULL;
   global_stopnethack_mode = 0;
   global_revenge_mode = 0;
+  global_ban_type = 3;
   global_ban_time = 120;
   global_exempt_time = 60;
   global_invite_time = 60;
@@ -965,11 +989,12 @@ char *channels_start(Function *global_funcs)
          "-revengebot "
          "-protecthalfops "
          "-autohalfop "
-         "-nodesynch ");
-  module_register(MODULE_NAME, channels_table, 1, 0);
-  if (!module_depend(MODULE_NAME, "eggdrop", 106, 7)) {
+         "-nodesynch "
+         "-static ");
+  module_register(MODULE_NAME, channels_table, 1, 1);
+  if (!module_depend(MODULE_NAME, "eggdrop", 106, 20)) {
     module_undepend(MODULE_NAME);
-    return "This module requires Eggdrop 1.6.7 or later.";
+    return "This module requires Eggdrop 1.6.20 or later.";
   }
   add_hook(HOOK_MINUTELY, (Function) check_expired_bans);
   add_hook(HOOK_MINUTELY, (Function) check_expired_exempts);
@@ -990,7 +1015,6 @@ char *channels_start(Function *global_funcs)
   my_tcl_ints[0].val = &share_greet;
   add_tcl_ints(my_tcl_ints);
   add_tcl_coups(mychan_tcl_coups);
-  read_channels(0);
-  setstatic = 1;
+  read_channels(0, 1);
   return NULL;
 }
