@@ -2,11 +2,11 @@
  * cmdschan.c -- part of channels.mod
  *   commands from a user via dcc that cause server interaction
  *
- * $Id: cmdschan.c,v 1.75 2008-02-16 21:41:07 guppy Exp $
+ * $Id: cmdschan.c,v 1.85 2010/07/10 14:54:49 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -66,7 +66,7 @@ static void cmd_pls_ban(struct userrec *u, int idx, char *par)
       p = newsplit(&par);
       p_expire = p + 1;
       while (*(++p) != 0) {
-        switch (tolower(*p)) {
+        switch (tolower((unsigned) *p)) {
         case 'd':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
@@ -114,7 +114,7 @@ static void cmd_pls_ban(struct userrec *u, int idx, char *par)
                    me->funcs[SERVER_BOTUSERHOST]);
     else
       egg_snprintf(s1, sizeof s1, "%s!%s@%s", origbotname, botuser, hostname);
-    if (wild_match(s, s1)) {
+    if (match_addr(s, s1)) {
       dprintf(idx, "I'm not going to ban myself.\n");
       putlog(LOG_CMDS, "*", "#%s# attempted +ban %s", dcc[idx].nick, s);
     } else {
@@ -204,7 +204,7 @@ static void cmd_pls_exempt(struct userrec *u, int idx, char *par)
       p = newsplit(&par);
       p_expire = p + 1;
       while (*(++p) != 0) {
-        switch (tolower(*p)) {
+        switch (tolower((unsigned) *p)) {
         case 'd':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
@@ -332,7 +332,7 @@ static void cmd_pls_invite(struct userrec *u, int idx, char *par)
       p = newsplit(&par);
       p_expire = p + 1;
       while (*(++p) != 0) {
-        switch (tolower(*p)) {
+        switch (tolower((unsigned) *p)) {
         case 'd':
           *p = 0;
           expire_foo = strtol(p_expire, NULL, 10);
@@ -1173,6 +1173,8 @@ static void cmd_mns_chrec(struct userrec *u, int idx, char *par)
 
 static void cmd_pls_chan(struct userrec *u, int idx, char *par)
 {
+  int i, argc;
+  EGG_CONST char **argv;
   char *chname;
   struct chanset_t *chan;
 
@@ -1196,6 +1198,25 @@ static void cmd_pls_chan(struct userrec *u, int idx, char *par)
     return;
   }
 
+  if (Tcl_SplitList(NULL, par, &argc, &argv ) == TCL_ERROR) {
+    dprintf(idx, "Invalid channel options.\n");
+    return;
+  }
+  for (i = 0; i < argc; i++) {
+    if ((!strncmp(argv[i], "need-", 5) || !strcmp(argv[i] + 1, "static"))
+        && (!(u->flags & USER_OWNER) || (!isowner(dcc[idx].nick)
+        && must_be_owner))) {
+      dprintf(idx, "Due to security concerns, only permanent owners can "
+                   "set the need-* and +/-static modes.\n");
+      Tcl_Free((char *) argv);
+      return;
+    }
+    if (argv[i][0] == '-' || argv[i][0] == '+')
+      continue;
+    i++;
+  }
+  Tcl_Free((char *) argv);
+
   if (tcl_channel_add(0, chname, par) == TCL_ERROR)
     dprintf(idx, "Invalid channel or channel options.\n");
   else
@@ -1223,7 +1244,7 @@ static void cmd_mns_chan(struct userrec *u, int idx, char *par)
     return;
   }
   if (channel_static(chan)) {
-    dprintf(idx, "Cannot remove %s, it is not a dynamic channel!.\n", chname);
+    dprintf(idx, "Cannot remove %s, it is a static channel!\n", chname);
     return;
   }
 
@@ -1282,6 +1303,7 @@ static void cmd_chaninfo(struct userrec *u, int idx, char *par)
       dprintf(idx, "revenge-mode: %d\n", chan->revenge_mode);
     else
       dprintf(idx, "revenge-mode: 0\n");
+    dprintf(idx, "ban-type: %d\n", chan->ban_type);
     if (chan->ban_time)
       dprintf(idx, "ban-time: %d\n", chan->ban_time);
     else
@@ -1339,8 +1361,9 @@ static void cmd_chaninfo(struct userrec *u, int idx, char *par)
             (chan->status & CHAN_DYNAMICBANS) ? '+' : '-',
             (chan->status & CHAN_NOUSERBANS) ? '-' : '+',
             (chan->status & CHAN_AUTOHALFOP) ? '+' : '-');
-    dprintf(idx, "     %cprotecthalfops\n",
-            (chan->status & CHAN_PROTECTHALFOPS) ? '+' : '-');
+    dprintf(idx, "     %cprotecthalfops %cstatic\n",
+            (chan->status & CHAN_PROTECTHALFOPS) ? '+' : '-',
+            (chan->status & CHAN_STATIC) ? '+' : '-');
     dprintf(idx,
             "     %cdynamicexempts %cuserexempts    %cdynamicinvites "
             "%cuserinvites\n",
@@ -1487,6 +1510,12 @@ static void cmd_chanset(struct userrec *u, int idx, char *par)
       while (list[0][0]) {
         if (list[0][0] == '+' || list[0][0] == '-' ||
             (!strcmp(list[0], "dont-idle-kick"))) {
+            if (!strcmp(list[0] + 1, "static") && must_be_owner &&
+                !(isowner(dcc[idx].nick))) {
+              dprintf(idx, "Only permanent owners can modify the static flag.\n");
+              nfree(buf);
+              return;
+            }
           if (tcl_channel_modify(0, chan, 1, list) == TCL_OK) {
             strcat(answers, list[0]);
             strcat(answers, " ");
@@ -1503,8 +1532,7 @@ static void cmd_chanset(struct userrec *u, int idx, char *par)
         if (strncmp(list[0], "need-", 5) || (u->flags & USER_OWNER)) {
           if (!strncmp(list[0], "need-", 5) && !(isowner(dcc[idx].nick)) &&
               must_be_owner) {
-            dprintf(idx, "Due to security concerns, only permanent owners can "
-                    "set these modes.\n");
+            dprintf(idx, "Due to security concerns, only permanent owners can set these modes.\n");
             nfree(buf);
             return;
           }
@@ -1564,8 +1592,7 @@ static void cmd_chanload(struct userrec *u, int idx, char *par)
   else {
     dprintf(idx, "Reloading all dynamic channel settings.\n");
     putlog(LOG_CMDS, "*", "#%s# chanload", dcc[idx].nick);
-    setstatic = 0;
-    read_channels(1);
+    read_channels(1, 1);
   }
 }
 
@@ -1577,26 +1604,26 @@ static void cmd_chanload(struct userrec *u, int idx, char *par)
  * NOTE: As with msg commands, the function is responsible for any logging.
  */
 static cmd_t C_dcc_irc[] = {
-  {"+ban",     "ol|ol", (Function) cmd_pls_ban,    NULL},
-  {"+exempt",  "ol|ol", (Function) cmd_pls_exempt, NULL},
-  {"+invite",  "ol|ol", (Function) cmd_pls_invite, NULL},
-  {"+chan",    "n",     (Function) cmd_pls_chan,   NULL},
-  {"+chrec",   "m|m",   (Function) cmd_pls_chrec,  NULL},
-  {"-ban",     "ol|ol", (Function) cmd_mns_ban,    NULL},
-  {"-chan",    "n",     (Function) cmd_mns_chan,   NULL},
-  {"-chrec",   "m|m",   (Function) cmd_mns_chrec,  NULL},
-  {"bans",     "ol|ol", (Function) cmd_bans,       NULL},
-  {"-exempt",  "ol|ol", (Function) cmd_mns_exempt, NULL},
-  {"-invite",  "ol|ol", (Function) cmd_mns_invite, NULL},
-  {"exempts",  "ol|ol", (Function) cmd_exempts,    NULL},
-  {"invites",  "ol|ol", (Function) cmd_invites,    NULL},
-  {"chaninfo", "m|m",   (Function) cmd_chaninfo,   NULL},
-  {"chanload", "n|n",   (Function) cmd_chanload,   NULL},
-  {"chanset",  "n|n",   (Function) cmd_chanset,    NULL},
-  {"chansave", "n|n",   (Function) cmd_chansave,   NULL},
-  {"chinfo",   "m|m",   (Function) cmd_chinfo,     NULL},
-  {"info",     "",      (Function) cmd_info,       NULL},
-  {"stick",    "ol|ol", (Function) cmd_stick,      NULL},
-  {"unstick",  "ol|ol", (Function) cmd_unstick,    NULL},
+  {"+ban",     "ol|ol", (IntFunc) cmd_pls_ban,    NULL},
+  {"+exempt",  "ol|ol", (IntFunc) cmd_pls_exempt, NULL},
+  {"+invite",  "ol|ol", (IntFunc) cmd_pls_invite, NULL},
+  {"+chan",    "n",     (IntFunc) cmd_pls_chan,   NULL},
+  {"+chrec",   "m|m",   (IntFunc) cmd_pls_chrec,  NULL},
+  {"-ban",     "ol|ol", (IntFunc) cmd_mns_ban,    NULL},
+  {"-chan",    "n",     (IntFunc) cmd_mns_chan,   NULL},
+  {"-chrec",   "m|m",   (IntFunc) cmd_mns_chrec,  NULL},
+  {"bans",     "ol|ol", (IntFunc) cmd_bans,       NULL},
+  {"-exempt",  "ol|ol", (IntFunc) cmd_mns_exempt, NULL},
+  {"-invite",  "ol|ol", (IntFunc) cmd_mns_invite, NULL},
+  {"exempts",  "ol|ol", (IntFunc) cmd_exempts,    NULL},
+  {"invites",  "ol|ol", (IntFunc) cmd_invites,    NULL},
+  {"chaninfo", "m|m",   (IntFunc) cmd_chaninfo,   NULL},
+  {"chanload", "n|n",   (IntFunc) cmd_chanload,   NULL},
+  {"chanset",  "n|n",   (IntFunc) cmd_chanset,    NULL},
+  {"chansave", "n|n",   (IntFunc) cmd_chansave,   NULL},
+  {"chinfo",   "m|m",   (IntFunc) cmd_chinfo,     NULL},
+  {"info",     "",      (IntFunc) cmd_info,       NULL},
+  {"stick",    "ol|ol", (IntFunc) cmd_stick,      NULL},
+  {"unstick",  "ol|ol", (IntFunc) cmd_unstick,    NULL},
   {NULL,       NULL,    NULL,                      NULL}
 };
