@@ -1,11 +1,11 @@
 /*
  * tclchan.c -- part of channels.mod
  *
- * $Id: tclchan.c,v 1.97 2008-02-16 21:41:07 guppy Exp $
+ * $Id: tclchan.c,v 1.106 2010/07/10 14:54:49 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -786,6 +786,8 @@ static int tcl_channel_info(Tcl_Interp *irp, struct chanset_t *chan)
   Tcl_AppendElement(irp, s);
   simple_sprintf(s, "%d:%d", chan->aop_min, chan->aop_max);
   Tcl_AppendElement(irp, s);
+  simple_sprintf(s, "%d", chan->ban_type);
+  Tcl_AppendElement(irp, s);
   simple_sprintf(s, "%d", chan->ban_time);
   Tcl_AppendElement(irp, s);
   simple_sprintf(s, "%d", chan->exempt_time);
@@ -892,6 +894,10 @@ static int tcl_channel_info(Tcl_Interp *irp, struct chanset_t *chan)
     Tcl_AppendElement(irp, "+nodesynch");
   else
     Tcl_AppendElement(irp, "-nodesynch");
+  if (chan->status & CHAN_STATIC)
+    Tcl_AppendElement(irp, "+static");
+  else
+    Tcl_AppendElement(irp, "-static");
   for (ul = udef; ul; ul = ul->next) {
     /* If it's undefined, skip it. */
     if (!ul->defined || !ul->name)
@@ -959,6 +965,8 @@ static int tcl_channel_get(Tcl_Interp *irp, struct chanset_t *chan,
     simple_sprintf(s, "%d", chan->stopnethack_mode);
   else if (!strcmp(setting, "revenge-mode"))
     simple_sprintf(s, "%d", chan->revenge_mode);
+  else if (!strcmp(setting, "ban-type"))
+    simple_sprintf(s, "%d", chan->ban_type);
   else if (!strcmp(setting, "ban-time"))
     simple_sprintf(s, "%d", chan->ban_time);
   else if (!strcmp(setting, "exempt-time"))
@@ -1000,6 +1008,7 @@ static int tcl_channel_get(Tcl_Interp *irp, struct chanset_t *chan,
   else if CHKFLAG_POS(CHAN_CYCLE, "cycle", chan->status)
   else if CHKFLAG_POS(CHAN_SEEN, "seen", chan->status)
   else if CHKFLAG_POS(CHAN_NODESYNCH, "nodesynch", chan->status)
+  else if CHKFLAG_POS(CHAN_STATIC, "static", chan->status)
   else if CHKFLAG_POS(CHAN_DYNAMICEXEMPTS, "dynamicexempts",
                       chan->ircnet_status)
   else if CHKFLAG_NEG(CHAN_NOUSEREXEMPTS, "userexempts",
@@ -1046,7 +1055,7 @@ static int tcl_channel STDVAR
 {
   struct chanset_t *chan;
 
-  BADARGS(2, 999, " command ?options?");
+  BADARGS(2, -1, " command ?options?");
 
   if (!strcmp(argv[1], "add")) {
     BADARGS(3, 4, " add channel-name ?options-list?");
@@ -1056,7 +1065,7 @@ static int tcl_channel STDVAR
     return tcl_channel_add(irp, argv[2], argv[3]);
   }
   if (!strcmp(argv[1], "set")) {
-    BADARGS(3, 999, " set channel-name ?options?");
+    BADARGS(3, -1, " set channel-name ?options?");
 
     chan = findchan_by_dname(argv[2]);
     if (chan == NULL) {
@@ -1199,6 +1208,14 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
         return TCL_ERROR;
       }
       chan->revenge_mode = atoi(item[i]);
+    } else if (!strcmp(item[i], "ban-type")) {
+      i++;
+      if (i >= items) {
+        if (irp)
+          Tcl_AppendResult(irp, "channel ban-type needs argument", NULL);
+        return TCL_ERROR;
+      }
+      chan->ban_type = atoi(item[i]);
     } else if (!strcmp(item[i], "ban-time")) {
       i++;
       if (i >= items) {
@@ -1307,6 +1324,10 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
       chan->status |= CHAN_SEEN;
     else if (!strcmp(item[i], "-seen"))
       chan->status &= ~CHAN_SEEN;
+    else if (!strcmp(item[i], "+static"))
+      chan->status |= CHAN_STATIC;
+    else if (!strcmp(item[i], "-static"))
+      chan->status &= ~CHAN_STATIC;
     else if (!strcmp(item[i], "+dynamicexempts"))
       chan->ircnet_status |= CHAN_DYNAMICEXEMPTS;
     else if (!strcmp(item[i], "-dynamicexempts"))
@@ -1442,7 +1463,7 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
           val = nmalloc(3 * strlen(item[i]) + 10);
           convert_element(item[i], val);
           val = nrealloc(val, strlen(val) + 1);
-          setudef(ul, chan->dname, (int) val);
+          setudef(ul, chan->dname, (intptr_t) val);
           found = 1;
           break;
         }
@@ -1466,10 +1487,17 @@ static int tcl_channel_modify(Tcl_Interp *irp, struct chanset_t *chan,
       if (channel_inactive(chan) && (chan->status & (CHAN_ACTIVE | CHAN_PEND)))
         dprintf(DP_SERVER, "PART %s\n", chan->name);
       if (!channel_inactive(chan) &&
-          !(chan->status & (CHAN_ACTIVE | CHAN_PEND)))
-        dprintf(DP_SERVER, "JOIN %s %s\n", (chan->name[0]) ?
-                chan->name : chan->dname,
-                chan->channel.key[0] ? chan->channel.key : chan->key_prot);
+          !(chan->status & (CHAN_ACTIVE | CHAN_PEND))) {
+        char *key;
+
+        key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
+        if (key[0])
+          dprintf(DP_SERVER, "JOIN %s %s\n",
+                  chan->name[0] ? chan->name : chan->dname, key);
+        else
+          dprintf(DP_SERVER, "JOIN %s\n",
+                  chan->name[0] ? chan->name : chan->dname);
+      }
     }
     if ((old_status ^ chan->status) & (CHAN_ENFORCEBANS | CHAN_OPONJOIN |
                                        CHAN_BITCH | CHAN_AUTOVOICE |
@@ -1600,8 +1628,7 @@ static int tcl_loadchannels STDVAR
     Tcl_AppendResult(irp, "no channel file", NULL);
     return TCL_ERROR;
   }
-  setstatic = 0;
-  read_channels(1);
+  read_channels(1, 1);
   return TCL_OK;
 }
 
@@ -1775,27 +1802,35 @@ static void init_masklist(masklist *m)
  */
 static void init_channel(struct chanset_t *chan, int reset)
 {
-  chan->channel.maxmembers = 0;
-  chan->channel.mode = 0;
-  chan->channel.members = 0;
+  int flags = reset ? reset : CHAN_RESETALL;
+
   if (!reset) {
     chan->channel.key = nmalloc(1);
     chan->channel.key[0] = 0;
+    chan->channel.members = 0;
+    chan->channel.member = nmalloc(sizeof(memberlist));
+    chan->channel.member->nick[0] = 0;
+    chan->channel.member->next = NULL;
   }
-
-  chan->channel.ban = nmalloc(sizeof(masklist));
-  init_masklist(chan->channel.ban);
-
-  chan->channel.exempt = nmalloc(sizeof(masklist));
-  init_masklist(chan->channel.exempt);
-
-  chan->channel.invite = nmalloc(sizeof(masklist));
-  init_masklist(chan->channel.invite);
-
-  chan->channel.member = nmalloc(sizeof(memberlist));
-  chan->channel.member->nick[0] = 0;
-  chan->channel.member->next = NULL;
-  chan->channel.topic = NULL;
+  if (flags & CHAN_RESETMODES) {
+    if (!reset)
+    chan->channel.mode = 0;
+    chan->channel.maxmembers = 0;
+  }
+  if (flags & CHAN_RESETBANS) {
+    chan->channel.ban = nmalloc(sizeof(masklist));
+    init_masklist(chan->channel.ban);
+  }
+  if (flags & CHAN_RESETEXEMPTS) {
+    chan->channel.exempt = nmalloc(sizeof(masklist));
+    init_masklist(chan->channel.exempt);
+  }
+  if (flags & CHAN_RESETINVITED) {
+    chan->channel.invite = nmalloc(sizeof(masklist));
+    init_masklist(chan->channel.invite);
+  }
+  if (flags & CHAN_RESETTOPIC)
+    chan->channel.topic = NULL;
 }
 
 static void clear_masklist(masklist *m)
@@ -1816,24 +1851,34 @@ static void clear_masklist(masklist *m)
  */
 static void clear_channel(struct chanset_t *chan, int reset)
 {
+  int flags = reset ? reset : CHAN_RESETALL;
   memberlist *m, *m1;
 
-  if (chan->channel.topic)
-    nfree(chan->channel.topic);
-  for (m = chan->channel.member; m; m = m1) {
-    m1 = m->next;
-    nfree(m);
+  if (flags & CHAN_RESETWHO) {
+    for (m = chan->channel.member; m; m = m1) {
+      m1 = m->next;
+      if (reset)
+         m->flags &= ~WHO_SYNCED;
+      else
+        nfree(m);
+    }
   }
-
-  clear_masklist(chan->channel.ban);
-  chan->channel.ban = NULL;
-  clear_masklist(chan->channel.exempt);
-  chan->channel.exempt = NULL;
-  clear_masklist(chan->channel.invite);
-  chan->channel.invite = NULL;
-
+  if (flags & CHAN_RESETBANS) {
+    clear_masklist(chan->channel.ban);
+    chan->channel.ban = NULL;
+  }
+  if (flags & CHAN_RESETEXEMPTS) {
+    clear_masklist(chan->channel.exempt);
+    chan->channel.exempt = NULL;
+  }
+  if (flags & CHAN_RESETINVITED) {
+    clear_masklist(chan->channel.invite);
+    chan->channel.invite = NULL;
+  }
+  if ((flags & CHAN_RESETTOPIC) && chan->channel.topic)
+    nfree(chan->channel.topic);
   if (reset)
-    init_channel(chan, 1);
+    init_channel(chan, reset);
 }
 
 /* Create new channel and parse commands.
@@ -1892,6 +1937,7 @@ static int tcl_channel_add(Tcl_Interp *irp, char *newname, char *options)
     chan->flood_nick_time = gfld_nick_time;
     chan->stopnethack_mode = global_stopnethack_mode;
     chan->revenge_mode = global_revenge_mode;
+    chan->ban_type = global_ban_type;
     chan->ban_time = global_ban_time;
     chan->exempt_time = global_exempt_time;
     chan->invite_time = global_invite_time;
@@ -1908,12 +1954,10 @@ static int tcl_channel_add(Tcl_Interp *irp, char *newname, char *options)
 
     /* Initialize chan->channel info */
     init_channel(chan, 0);
-    list_append((struct list_type **) &chanset, (struct list_type *) chan);
+    egg_list_append((struct list_type **) &chanset, (struct list_type *) chan);
     /* Channel name is stored in xtra field for sharebot stuff */
     join = 1;
   }
-  if (setstatic)
-    chan->status |= CHAN_STATIC;
   /* If chan_hack is set, we're loading the userfile. Ignore errors while
    * reading userfile and just return TCL_OK. This is for compatability
    * if a user goes back to an eggdrop that no-longer supports certain
@@ -1924,8 +1968,12 @@ static int tcl_channel_add(Tcl_Interp *irp, char *newname, char *options)
     ret = TCL_ERROR;
   }
   Tcl_Free((char *) item);
-  if (join && !channel_inactive(chan) && module_find("irc", 0, 0))
-    dprintf(DP_SERVER, "JOIN %s %s\n", chan->dname, chan->key_prot);
+  if (join && !channel_inactive(chan) && module_find("irc", 0, 0)) {
+    if (chan->key_prot[0])
+      dprintf(DP_SERVER, "JOIN %s %s\n", chan->dname, chan->key_prot);
+    else
+      dprintf(DP_SERVER, "JOIN %s\n", chan->dname);
+  }
   return ret;
 }
 
