@@ -1,11 +1,11 @@
 /*
  * userchan.c -- part of channels.mod
  *
- * $Id: userchan.c,v 1.50 2008-02-16 21:41:07 guppy Exp $
+ * $Id: userchan.c,v 1.53 2010/01/07 13:48:31 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -228,7 +228,7 @@ static int u_equals_mask(maskrec *u, char *mask)
 static int u_match_mask(maskrec *rec, char *mask)
 {
   for (; rec; rec = rec->next)
-    if (wild_match(rec->mask, mask))
+    if (match_addr(rec->mask, mask))
       return 1;
   return 0;
 }
@@ -428,7 +428,7 @@ static int u_addban(struct chanset_t *chan, char *ban, char *from, char *note,
                    me->funcs[SERVER_BOTUSERHOST]);
   else
     simple_sprintf(s, "%s!%s@%s", origbotname, botuser, hostname);
-  if (wild_match(host, s)) {
+  if (match_addr(host, s)) {
     putlog(LOG_MISC, "*", IRC_IBANNEDME);
     return 0;
   }
@@ -822,7 +822,7 @@ static void tell_bans(int idx, int show_inact, char *match)
     dprintf(idx, "%s:\n", BANS_GLOBAL);
   for (u = global_bans; u; u = u->next) {
     if (match[0]) {
-      if ((wild_match(match, u->mask)) ||
+      if ((match_addr(match, u->mask)) ||
           (wild_match(match, u->desc)) || (wild_match(match, u->user)))
         display_ban(idx, k, u, chan, 1);
       k++;
@@ -838,7 +838,7 @@ static void tell_bans(int idx, int show_inact, char *match)
               BANS_BYCHANNEL, chan->dname, MODES_NOTBYBOT);
     for (u = chan->bans; u; u = u->next) {
       if (match[0]) {
-        if ((wild_match(match, u->mask)) ||
+        if ((match_addr(match, u->mask)) ||
             (wild_match(match, u->desc)) || (wild_match(match, u->user)))
           display_ban(idx, k, u, chan, 1);
         k++;
@@ -866,7 +866,7 @@ static void tell_bans(int idx, int show_inact, char *match)
             sprintf(s, " (active %02d:%02d)", min, sec);
             strcat(fill, s);
           }
-          if ((!match[0]) || (wild_match(match, b->mask)))
+          if ((!match[0]) || (match_addr(match, b->mask)))
             dprintf(idx, "* [%3d] %s\n", k, fill);
           k++;
         }
@@ -911,7 +911,7 @@ static void tell_exempts(int idx, int show_inact, char *match)
     dprintf(idx, "%s:\n", EXEMPTS_GLOBAL);
   for (u = global_exempts; u; u = u->next) {
     if (match[0]) {
-      if ((wild_match(match, u->mask)) ||
+      if ((match_addr(match, u->mask)) ||
           (wild_match(match, u->desc)) || (wild_match(match, u->user)))
         display_exempt(idx, k, u, chan, 1);
       k++;
@@ -927,7 +927,7 @@ static void tell_exempts(int idx, int show_inact, char *match)
               EXEMPTS_BYCHANNEL, chan->dname, MODES_NOTBYBOT);
     for (u = chan->exempts; u; u = u->next) {
       if (match[0]) {
-        if ((wild_match(match, u->mask)) ||
+        if ((match_addr(match, u->mask)) ||
             (wild_match(match, u->desc)) || (wild_match(match, u->user)))
           display_exempt(idx, k, u, chan, 1);
         k++;
@@ -955,7 +955,7 @@ static void tell_exempts(int idx, int show_inact, char *match)
             sprintf(s, " (active %02d:%02d)", min, sec);
             strcat(fill, s);
           }
-          if ((!match[0]) || (wild_match(match, e->mask)))
+          if ((!match[0]) || (match_addr(match, e->mask)))
             dprintf(idx, "* [%3d] %s\n", k, fill);
           k++;
         }
@@ -1000,7 +1000,7 @@ static void tell_invites(int idx, int show_inact, char *match)
     dprintf(idx, "%s:\n", INVITES_GLOBAL);
   for (u = global_invites; u; u = u->next) {
     if (match[0]) {
-      if ((wild_match(match, u->mask)) ||
+      if ((match_addr(match, u->mask)) ||
           (wild_match(match, u->desc)) || (wild_match(match, u->user)))
         display_invite(idx, k, u, chan, 1);
       k++;
@@ -1016,7 +1016,7 @@ static void tell_invites(int idx, int show_inact, char *match)
               INVITES_BYCHANNEL, chan->dname, MODES_NOTBYBOT);
     for (u = chan->invites; u; u = u->next) {
       if (match[0]) {
-        if ((wild_match(match, u->mask)) ||
+        if ((match_addr(match, u->mask)) ||
             (wild_match(match, u->desc)) || (wild_match(match, u->user)))
           display_invite(idx, k, u, chan, 1);
         k++;
@@ -1044,7 +1044,7 @@ static void tell_invites(int idx, int show_inact, char *match)
             sprintf(s, " (active %02d:%02d)", min, sec);
             strcat(fill, s);
           }
-          if ((!match[0]) || (wild_match(match, i->mask)))
+          if ((!match[0]) || (match_addr(match, i->mask)))
             dprintf(idx, "* [%3d] %s\n", k, fill);
           k++;
         }
@@ -1077,7 +1077,7 @@ static int write_bans(FILE *f, int idx)
         fprintf(f, "- %s:%s%lu%s:+%lu:%lu:%s:%s\n", mask,
                 (b->flags & MASKREC_PERM) ? "+" : "", expire,
                 (b->flags & MASKREC_STICKY) ? "*" : "", added,
-                b->lastactive, b->user ? b->user : botnetnick,
+                (long) b->lastactive, b->user ? b->user : botnetnick,
                 b->desc ? b->desc : "requested") == EOF) {
       if (mask)
         nfree(mask);
@@ -1104,7 +1104,7 @@ static int write_bans(FILE *f, int idx)
               fprintf(f, "- %s:%s%lu%s:+%lu:%lu:%s:%s\n", mask,
                       (b->flags & MASKREC_PERM) ? "+" : "", expire,
                       (b->flags & MASKREC_STICKY) ? "*" : "", added,
-                      b->lastactive, b->user ? b->user : botnetnick,
+                      (long) b->lastactive, b->user ? b->user : botnetnick,
                       b->desc ? b->desc : "requested") == EOF) {
             if (mask)
               nfree(mask);
@@ -1137,7 +1137,7 @@ static int write_exempts(FILE *f, int idx)
         fprintf(f, "%s %s:%s%lu%s:+%lu:%lu:%s:%s\n", "%", mask,
                 (e->flags & MASKREC_PERM) ? "+" : "", expire,
                 (e->flags & MASKREC_STICKY) ? "*" : "", added,
-                e->lastactive, e->user ? e->user : botnetnick,
+                (long) e->lastactive, e->user ? e->user : botnetnick,
                 e->desc ? e->desc : "requested") == EOF) {
       if (mask)
         nfree(mask);
@@ -1164,7 +1164,7 @@ static int write_exempts(FILE *f, int idx)
               fprintf(f, "%s %s:%s%lu%s:+%lu:%lu:%s:%s\n", "%", mask,
                       (e->flags & MASKREC_PERM) ? "+" : "", expire,
                       (e->flags & MASKREC_STICKY) ? "*" : "", added,
-                      e->lastactive, e->user ? e->user : botnetnick,
+                      (long) e->lastactive, e->user ? e->user : botnetnick,
                       e->desc ? e->desc : "requested") == EOF) {
             if (mask)
               nfree(mask);
@@ -1197,7 +1197,7 @@ static int write_invites(FILE *f, int idx)
         fprintf(f, "@ %s:%s%lu%s:+%lu:%lu:%s:%s\n", mask,
                 (ir->flags & MASKREC_PERM) ? "+" : "", expire,
                 (ir->flags & MASKREC_STICKY) ? "*" : "", added,
-                ir->lastactive, ir->user ? ir->user : botnetnick,
+                (long) ir->lastactive, ir->user ? ir->user : botnetnick,
                 ir->desc ? ir->desc : "requested") == EOF) {
       if (mask)
         nfree(mask);
@@ -1224,7 +1224,7 @@ static int write_invites(FILE *f, int idx)
               fprintf(f, "@ %s:%s%lu%s:+%lu:%lu:%s:%s\n", mask,
                       (ir->flags & MASKREC_PERM) ? "+" : "", expire,
                       (ir->flags & MASKREC_STICKY) ? "*" : "", added,
-                      ir->lastactive, ir->user ? ir->user : botnetnick,
+                      (long) ir->lastactive, ir->user ? ir->user : botnetnick,
                       ir->desc ? ir->desc : "requested") == EOF) {
             if (mask)
               nfree(mask);
@@ -1372,7 +1372,7 @@ static void check_expired_exempts(void)
         match = 0;
         b = chan->channel.ban;
         while (b->mask[0] && !match) {
-          if (wild_match(b->mask, u->mask) || wild_match(u->mask, b->mask))
+          if (mask_match(b->mask, u->mask))
             match = 1;
           else
             b = b->next;
@@ -1400,7 +1400,7 @@ static void check_expired_exempts(void)
         match = 0;
         b = chan->channel.ban;
         while (b->mask[0] && !match) {
-          if (wild_match(b->mask, u->mask) || wild_match(u->mask, b->mask))
+          if (mask_match(b->mask, u->mask))
             match = 1;
           else
             b = b->next;
