@@ -5,10 +5,10 @@
  *
  * Modified/written by Fabian Knittel <fknittel@gmx.de>
  *
- * $Id: coredns.c,v 1.32 2008-02-16 21:41:08 guppy Exp $
+ * $Id: coredns.c,v 1.35 2010/06/29 15:52:24 thommey Exp $
  */
 /*
- * Portions Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Portions Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -566,8 +566,15 @@ static void dorequest(char *s, int type, u_16bit_t id)
 {
   packetheader *hp;
   int r, i;
-  u_8bit_t buf[(MAX_PACKETSIZE / sizeof(char)) + 1];
+  u_8bit_t *buf;
 
+  /* Use malloc here instead of a static buffer, as per res_mkquery()'s manual
+   * buf should be aligned on an eight byte boundary. malloc() should return a
+   * pointer to an address properly aligned for any data type. Failing to
+   * provide a aligned buffer will result in a SIGBUS crash atleast on SPARC
+   * CPUs.
+   */
+  buf = nmalloc(MAX_PACKETSIZE + 1);
   r = res_mkquery(QUERY, s, C_IN, type, NULL, 0, NULL, buf, MAX_PACKETSIZE);
   if (r == -1) {
     ddebug0(RES_ERR "Query too large.");
@@ -579,6 +586,7 @@ static void dorequest(char *s, int type, u_16bit_t id)
     (void) sendto(resfd, buf, r, 0,
                   (struct sockaddr *) &_res.nsaddr_list[i],
                   sizeof(struct sockaddr));
+  nfree(buf);
 }
 
 /* (Re-)send request with existing id.
@@ -1073,7 +1081,12 @@ static int init_dns_network(void)
            strerror(errno));
     return 0;
   }
-  (void) allocsock(resfd, SOCK_PASS);
+  if (allocsock(resfd, SOCK_PASS) == -1) {
+    putlog(LOG_MISC, "*",
+           "Unable to allocate socket in socklist for nameserver communication");
+    killsock(resfd);
+    return 0;
+  }
   option = 1;
   if (setsockopt(resfd, SOL_SOCKET, SO_BROADCAST, (char *) &option,
                  sizeof(option))) {
