@@ -4,10 +4,10 @@
  *
  * Written by Fabian Knittel <fknittel@gmx.de>
  *
- * $Id: filedb3.h,v 1.22 2008-02-16 21:41:08 guppy Exp $
+ * $Id: filedb3.h,v 1.24 2010/01/03 13:27:42 pseudo Exp $
  */
 /*
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -99,6 +99,12 @@ do {                                                                    \
     my_free(target);                                                    \
 } while (0)
 
+#define malloc_strcpy_nocheck(target, entry)                            \
+do {                                                                    \
+  (target) = nrealloc((target), strlen(entry) + 1);                     \
+  strcpy((target), (entry));                                            \
+} while (0)
+
 /* Macro to calculate the total length of dynamic data. */
 #define filedb_tot_dynspace(fdh) ((fdh).filename_len + (fdh).desc_len + \
         (fdh).chan_len + (fdh).uploader_len + (fdh).flags_req_len + \
