@@ -2,11 +2,11 @@
  * filesys.c -- part of filesys.mod
  *   main file of the filesys eggdrop module
  *
- * $Id: filesys.c,v 1.74 2008-02-16 21:41:09 guppy Exp $
+ * $Id: filesys.c,v 1.78 2010/06/29 15:52:24 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -464,7 +464,7 @@ static int do_dcc_send(int idx, char *dir, char *fn, char *nick, int resend)
   if (at_limit(nick)) {
     char xxx[1024];
 
-    sprintf(xxx, "%d*%s%s", strlen(dccdir), dccdir, dir);
+    sprintf(xxx, "%d*%s%s", (int) strlen(dccdir), dccdir, dir);
     queue_file(xxx, fn, dcc[idx].nick, nick);
     dprintf(idx, "Queued: %s to %s\n", fn, nick);
     my_free(s);
@@ -848,7 +848,7 @@ static int filesys_DCC_CHAT(char *nick, char *from, char *handle,
   strcpy(buf, text + 5);
   get_user_flagrec(u, &fr, 0);
   param = newsplit(&msg);
-  if (dcc_total == max_dcc) {
+  if (dcc_total == max_dcc && increase_socks_max()) {
     putlog(LOG_MISC, "*", DCC_TOOMANYDCCS2, "CHAT(file)", param, nick, from);
   } else if (glob_party(fr) || (!require_p && chan_op(fr)))
     return 0;                   /* Allow ctcp.so to pick up the chat */
@@ -914,7 +914,7 @@ static void init_server_ctcps(char *module)
 }
 
 static cmd_t myload[] = {
-  {"server", "",   (Function) init_server_ctcps, "filesys:server"},
+  {"server", "",   (IntFunc) init_server_ctcps, "filesys:server"},
   {NULL,     NULL, NULL,                                     NULL}
 };
 
