@@ -2,11 +2,11 @@
  * tclfiles.c -- part of filesys.mod
  *   Tcl stubs for file system commands moved here to support modules
  *
- * $Id: tclfiles.c,v 1.28 2008-02-16 21:41:09 guppy Exp $
+ * $Id: tclfiles.c,v 1.30 2010/01/03 13:27:42 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -237,7 +237,7 @@ static int tcl_setflags STDVAR
 
     break_down_flags(argv[2], &fr, NULL);
     build_flags(f, &fr, NULL);
-    malloc_strcpy(fdbe->flags_req, f);
+    malloc_strcpy_nocheck(fdbe->flags_req, f);
   } else
     my_free(fdbe->flags_req);
   if (argc == 4)
@@ -350,7 +350,7 @@ static int tcl_mkdir STDVAR
 
     break_down_flags(argv[2], &fr, NULL);
     build_flags(f, &fr, NULL);
-    malloc_strcpy(fdbe->flags_req, f);
+    malloc_strcpy_nocheck(fdbe->flags_req, f);
   } else if (fdbe->flags_req) {
     my_free(fdbe->flags_req);
   }
