@@ -6,11 +6,11 @@
  *   user kickban, kick, op, deop
  *   idle kicking
  *
- * $Id: chan.c,v 1.125 2008-02-16 21:41:09 guppy Exp $
+ * $Id: chan.c,v 1.135 2010/07/01 16:10:49 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -54,6 +54,25 @@ static memberlist *newmember(struct chanset_t *chan)
   return x;
 }
 
+/* Remove channel members for which no WHO reply was received */
+static inline void sync_members(struct chanset_t *chan)
+{
+  memberlist *m, *next, *prev;
+
+  for (m = chan->channel.member, prev = 0; m && m->nick[0]; m = next) {
+    next = m->next;
+    if (!chan_whosynced(m)) {
+      if (prev)
+        prev->next = next;
+      else
+        chan->channel.member = next;
+      nfree(m);
+      chan->channel.members--;
+    } else
+      prev = m;
+  }
+}
+
 /* Always pass the channel dname (display name) to this function <cybah>
  */
 static void update_idle(char *chname, char *nick)
@@ -134,7 +153,7 @@ static void check_exemptlist(struct chanset_t *chan, char *from)
     return;
 
   for (e = chan->channel.exempt; e->mask[0]; e = e->next)
-    if (wild_match(e->mask, from)) {
+    if (match_addr(e->mask, from)) {
       add_mode(chan, '-', 'e', e->mask);
       ok = 1;
     }
@@ -150,7 +169,7 @@ static void check_exemptlist(struct chanset_t *chan, char *from)
 static void do_mask(struct chanset_t *chan, masklist *m, char *mask, char mode)
 {
   for (; m && m->mask[0]; m = m->next)
-    if (wild_match(mask, m->mask) && rfc_casecmp(mask, m->mask))
+    if (cmp_masks(mask, m->mask) && rfc_casecmp(mask, m->mask))
       add_mode(chan, '-', mode, m->mask);
   add_mode(chan, '+', mode, mask);
   flush_mode(chan, QUICK);
@@ -354,7 +373,7 @@ static char *quickban(struct chanset_t *chan, char *uhost)
 {
   static char s1[512];
 
-  maskban(uhost, s1);
+  maskaddr(uhost, s1, chan->ban_type);
   do_mask(chan, chan->channel.ban, s1, 'b');
   return s1;
 }
@@ -380,7 +399,7 @@ static void kick_all(struct chanset_t *chan, char *hostmask, char *comment,
     sprintf(s, "%s!%s", m->nick, m->userhost);
     get_user_flagrec(m->user ? m->user : get_user_by_host(s), &fr, chan->dname);
     if ((me_op(chan) || (me_halfop(chan) && !chan_hasop(m))) &&
-        wild_match(hostmask, s) && !chan_sentkick(m) &&
+        match_addr(hostmask, s) && !chan_sentkick(m) &&
         !match_my_nick(m->nick) && !chan_issplit(m) &&
         !glob_friend(fr) && !chan_friend(fr) && !(use_exempts && ((bantype &&
         isexempted(chan, s)) || (u_match_mask(global_exempts, s) ||
@@ -423,7 +442,7 @@ static void refresh_ban_kick(struct chanset_t *chan, char *user, char *nick)
   /* Check global bans in first cycle and channel bans in second cycle. */
   for (cycle = 0; cycle < 2; cycle++) {
     for (b = cycle ? chan->bans : global_bans; b; b = b->next) {
-      if (wild_match(b->mask, user)) {
+      if (match_addr(b->mask, user)) {
         struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
         char c[512];            /* The ban comment.     */
         char s[UHOSTLEN];
@@ -460,9 +479,9 @@ static void refresh_exempt(struct chanset_t *chan, char *user)
   /* Check global exempts in first cycle and channel exempts in second cycle. */
   for (cycle = 0; cycle < 2; cycle++) {
     for (e = cycle ? chan->exempts : global_exempts; e; e = e->next) {
-      if (wild_match(user, e->mask) || wild_match(e->mask, user)) {
+      if (mask_match(user, e->mask)) {
         for (b = chan->channel.ban; b && b->mask[0]; b = b->next) {
-          if (wild_match(b->mask, user) || wild_match(user, b->mask)) {
+          if (mask_match(b->mask, user)) {
             if (e->lastactive < now - 60 && !isexempted(chan, e->mask)) {
               do_mask(chan, chan->channel.exempt, e->mask, 'e');
               e->lastactive = now;
@@ -482,7 +501,7 @@ static void refresh_invite(struct chanset_t *chan, char *user)
   /* Check global invites in first cycle and channel invites in second cycle. */
   for (cycle = 0; cycle < 2; cycle++) {
     for (i = cycle ? chan->invites : global_invites; i; i = i->next) {
-      if (wild_match(i->mask, user) &&
+      if (match_addr(i->mask, user) &&
           ((i->flags & MASKREC_STICKY) || (chan->channel.mode & CHANINV))) {
         if (i->lastactive < now - 60 && !isinvited(chan, i->mask)) {
           do_mask(chan, chan->channel.invite, i->mask, 'I');
@@ -507,7 +526,7 @@ static void enforce_bans(struct chanset_t *chan)
   simple_sprintf(me, "%s!%s", botname, botuserhost);
   /* Go through all bans, kicking the users. */
   for (b = chan->channel.ban; b && b->mask[0]; b = b->next) {
-    if (!wild_match(b->mask, me))
+    if (!match_addr(b->mask, me))
       if (!isexempted(chan, b->mask))
         kick_all(chan, b->mask, IRC_YOUREBANNED, 1);
   }
@@ -550,7 +569,7 @@ static void recheck_exempts(struct chanset_t *chan)
           (!channel_dynamicexempts(chan) || (e->flags & MASKREC_STICKY)))
         add_mode(chan, '+', 'e', e->mask);
       for (b = chan->channel.ban; b && b->mask[0]; b = b->next) {
-        if ((wild_match(b->mask, e->mask) || wild_match(e->mask, b->mask)) &&
+        if (mask_match(b->mask, e->mask) &&
             !isexempted(chan, e->mask))
           add_mode(chan, '+', 'e', e->mask);
         /* do_mask(chan, chan->channel.exempt, e->mask, 'e'); */
@@ -626,7 +645,7 @@ static void check_this_ban(struct chanset_t *chan, char *banmask, int sticky)
 
   for (m = chan->channel.member; m && m->nick[0]; m = m->next) {
     sprintf(user, "%s!%s", m->nick, m->userhost);
-    if (wild_match(banmask, user) &&
+    if (match_addr(banmask, user) &&
         !(use_exempts &&
           (u_match_mask(global_exempts, user) ||
            u_match_mask(chan->exempts, user))))
@@ -693,7 +712,7 @@ static void recheck_channel_modes(struct chanset_t *chan)
     if (pls & CHANNOAMSG && !(cur & CHANNOAMSG))
       add_mode(chan, '+', 'T', "");
     else if (mns & CHANNOAMSG && cur & CHANNOAMSG)
-      add_mode(chan, '-', 'T', "");    
+      add_mode(chan, '-', 'T', "");
     if (pls & CHANTOPIC && !(cur & CHANTOPIC))
       add_mode(chan, '+', 't', "");
     else if (mns & CHANTOPIC && cur & CHANTOPIC)
@@ -836,7 +855,7 @@ static void check_this_user(char *hand, int delete, char *host)
       sprintf(s, "%s!%s", m->nick, m->userhost);
       u = m->user ? m->user : get_user_by_host(s);
       if ((u && !egg_strcasecmp(u->handle, hand) && delete < 2) ||
-          (!u && delete == 2 && wild_match(host, fixfrom(s)))) {
+          (!u && delete == 2 && match_addr(host, fixfrom(s)))) {
         u = delete ? NULL : u;
         get_user_flagrec(u, &fr, chan->dname);
         check_this_member(chan, m->nick, &fr);
@@ -873,6 +892,13 @@ static void recheck_channel(struct chanset_t *chan, int dobans)
     m->flags &= ~(SENTHALFOP | SENTKICK);
     check_this_member(chan, m->nick, &fr);
   }
+  /* Most IRCDs nowadays require +h/+o for getting e/I lists,
+   * so if we're still waiting for these, we'll request them here.
+   * In case we got them on join, nothing will be done */
+  if (chan->ircnet_status & (CHAN_ASKED_EXEMPTS | CHAN_ASKED_INVITED)) {
+    chan->ircnet_status &= ~(CHAN_ASKED_EXEMPTS | CHAN_ASKED_INVITED);
+    reset_chan_info(chan, CHAN_RESETEXEMPTS | CHAN_RESETINVITED);
+  }
   if (dobans) {
     if (channel_nouserbans(chan) && !stop_reset)
       resetbans(chan);
@@ -1022,10 +1048,9 @@ static int got352or4(struct chanset_t *chan, char *user, char *host,
   simple_sprintf(userhost, "%s!%s", nick, m->userhost);
   /* Combine n!u@h */
   m->user = NULL;               /* No handle match (yet) */
-  if (match_my_nick(nick)) {    /* Is it me? */
+  if (match_my_nick(nick))      /* Is it me? */
     strcpy(botuserhost, m->userhost);   /* Yes, save my own userhost */
-    m->joined = now;            /* set this to keep the whining masses happy */
-  }
+  m->flags |= WHO_SYNCED;
   if (strpbrk(flags, opchars) != NULL)
     m->flags |= (CHANOP | WASOP);
   else
@@ -1094,12 +1119,13 @@ static int got354(char *from, char *msg)
   return 0;
 }
 
+
 /* got 315: end of who
  * <server> 315 <to> <chan> :End of /who
  */
 static int got315(char *from, char *msg)
 {
-  char *chname;
+  char *chname, *key;
   struct chanset_t *chan;
 
   newsplit(&msg);
@@ -1108,6 +1134,7 @@ static int got315(char *from, char *msg)
   if (!chan || !channel_pending(chan)) /* Left channel before we got a 315? */
     return 0;
 
+  sync_members(chan);
   chan->status |= CHAN_ACTIVE;
   chan->status &= ~CHAN_PEND;
   if (!ismember(chan, botname)) {      /* Am I on the channel now?          */
@@ -1115,9 +1142,14 @@ static int got315(char *from, char *msg)
            chan->dname);
     clear_channel(chan, 1);
     chan->status &= ~CHAN_ACTIVE;
-    dprintf(DP_MODE, "JOIN %s %s\n",
-            (chan->name[0]) ? chan->name : chan->dname,
-            chan->channel.key[0] ? chan->channel.key : chan->key_prot);
+
+    key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
+    if (key[0])
+      dprintf(DP_SERVER, "JOIN %s %s\n",
+              chan->name[0] ? chan->name : chan->dname, key);
+    else
+      dprintf(DP_SERVER, "JOIN %s\n",
+              chan->name[0] ? chan->name : chan->dname);
   } else if (me_op(chan))
     recheck_channel(chan, 1);
   else if (chan->channel.members == 1)
@@ -1420,7 +1452,7 @@ static int got474(char *from, char *msg)
   return 0;
 }
 
-/* got 475: can't goin channel, bad key
+/* got 475: can't join channel, bad key
  */
 static int got475(char *from, char *msg)
 {
@@ -1446,7 +1478,11 @@ static int got475(char *from, char *msg)
       nfree(chan->channel.key);
       chan->channel.key = (char *) channel_malloc(1);
       chan->channel.key[0] = 0;
-      dprintf(DP_MODE, "JOIN %s %s\n", chan->dname, chan->key_prot);
+
+      if (chan->key_prot[0])
+        dprintf(DP_SERVER, "JOIN %s %s\n", chan->dname, chan->key_prot);
+      else
+        dprintf(DP_SERVER, "JOIN %s\n", chan->dname);
     } else {
       check_tcl_need(chan->dname, "key");
 
@@ -1466,7 +1502,7 @@ static int got475(char *from, char *msg)
  */
 static int gotinvite(char *from, char *msg)
 {
-  char *nick;
+  char *nick, *key;
   struct chanset_t *chan;
 
   newsplit(&msg);
@@ -1486,9 +1522,16 @@ static int gotinvite(char *from, char *msg)
 
   if (chan && (channel_pending(chan) || channel_active(chan)))
     dprintf(DP_HELP, "NOTICE %s :I'm already here.\n", nick);
-  else if (chan && !channel_inactive(chan))
-    dprintf(DP_MODE, "JOIN %s %s\n", (chan->name[0]) ? chan->name : chan->dname,
-            chan->channel.key[0] ? chan->channel.key : chan->key_prot);
+  else if (chan && !channel_inactive(chan)) {
+
+    key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
+    if (key[0])
+      dprintf(DP_SERVER, "JOIN %s %s\n",
+              chan->name[0] ? chan->name : chan->dname, key);
+    else
+      dprintf(DP_SERVER, "JOIN %s\n",
+              chan->name[0] ? chan->name : chan->dname);
+  }
   return 0;
 }
 
@@ -1568,31 +1611,57 @@ static int got332(char *from, char *msg)
   return 0;
 }
 
+/* Set delay for +o, +h, or +v channel modes
+ */
 static void set_delay(struct chanset_t *chan, char *nick)
 {
   time_t a_delay;
-  int aop_min = chan->aop_min, aop_max = chan->aop_max, count = 0;
+  int aop_min, aop_max, aop_diff, count = 0;
   memberlist *m, *m2;
 
   m = ismember(chan, nick);
   if (!m)
     return;
-  if (aop_min >= aop_max)
+
+  /* aop-delay 5:30 -- aop_min:aop_max */
+  aop_min = chan->aop_min;
+  aop_max = chan->aop_max;
+  aop_diff = aop_max - aop_min;
+
+  /* If either min or max is less than or equal to 0 we don't delay. */
+  if ((aop_min <= 0) || (aop_max <= 0)) {
+    a_delay = now + 1;
+
+  /* Use min value for delay if min greater then or equal to max or if the
+   * difference of max and min is greater than RANDOM_MAX (sanity check).
+   */
+  } else if ((aop_min >= aop_max) || (aop_diff > RANDOM_MAX)) {
     a_delay = now + aop_min;
-  else
-    a_delay = now + randint(aop_max - aop_min) + aop_min + 1;
+
+  /* Set a random delay based on the difference of max and min */
+  } else {
+    a_delay = now + randint(aop_diff) + aop_min + 1;
+  }
+
   for (m2 = chan->channel.member; m2 && m2->nick[0]; m2 = m2->next)
     if (m2->delay && !(m2->flags & FULL_DELAY))
       count++;
-  if (count)
-    for (m2 = chan->channel.member; m2 && m2->nick[0]; m2 = m2->next)
+
+  if (count) {
+    for (m2 = chan->channel.member; m2 && m2->nick[0]; m2 = m2->next) {
       if (m2->delay && !(m2->flags & FULL_DELAY)) {
         m2->delay = a_delay;
+
         if (count + 1 >= modesperline)
           m2->flags |= FULL_DELAY;
+
       }
+    }
+  }
+
   if (count + 1 >= modesperline)
     m->flags |= FULL_DELAY;
+
   m->delay = a_delay;
 }
 
@@ -1684,7 +1753,7 @@ static int gotjoin(char *from, char *chname)
              chan->dname);
       chan->status |= CHAN_ACTIVE;
       chan->status &= ~CHAN_PEND;
-      reset_chan_info(chan);
+      reset_chan_info(chan, CHAN_RESETALL);
     } else {
       m = ismember(chan, nick);
       if (m && m->split && !egg_strcasecmp(m->userhost, uhost)) {
@@ -1765,7 +1834,7 @@ static int gotjoin(char *from, char *chname)
           else
             putlog(LOG_JOIN | LOG_MISC, chan->dname, "%s joined %s.", nick,
                    chname);
-          reset_chan_info(chan);
+          reset_chan_info(chan, (CHAN_RESETALL & ~CHAN_RESETTOPIC));
         } else {
           struct chanuserrec *cr;
 
@@ -1822,7 +1891,7 @@ static int gotjoin(char *from, char *chname)
               (!use_exempts || !isexempted(chan, from)) && (me_op(chan) ||
               (me_halfop(chan) && !chan_hasop(m)))) {
             for (b = chan->channel.ban; b->mask[0]; b = b->next) {
-              if (wild_match(b->mask, from)) {
+              if (match_addr(b->mask, from)) {
                 dprintf(DP_SERVER, "KICK %s %s :%s\n", chname, m->nick,
                         IRC_YOUREBANNED);
                 m->flags |= SENTKICK;
@@ -1894,7 +1963,7 @@ exit:
  */
 static int gotpart(char *from, char *msg)
 {
-  char *nick, *chname;
+  char *nick, *chname, *key;
   struct chanset_t *chan;
   struct userrec *u;
 
@@ -1917,7 +1986,7 @@ static int gotpart(char *from, char *msg)
              chan->dname);
       chan->status |= CHAN_ACTIVE;
       chan->status &= ~CHAN_PEND;
-      reset_chan_info(chan);
+      reset_chan_info(chan, CHAN_RESETALL);
     }
     set_handle_laston(chan->dname, u, now);
     /* This must be directly above the killmember, in case we're doing anything
@@ -1939,10 +2008,16 @@ static int gotpart(char *from, char *msg)
     if (match_my_nick(nick)) {
       clear_channel(chan, 1);
       chan->status &= ~(CHAN_ACTIVE | CHAN_PEND);
-      if (!channel_inactive(chan))
-        dprintf(DP_MODE, "JOIN %s %s\n",
-                (chan->name[0]) ? chan->name : chan->dname,
-                chan->channel.key[0] ? chan->channel.key : chan->key_prot);
+      if (!channel_inactive(chan)) {
+
+        key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
+        if (key[0])
+          dprintf(DP_SERVER, "JOIN %s %s\n",
+                  chan->name[0] ? chan->name : chan->dname, key);
+        else
+          dprintf(DP_SERVER, "JOIN %s\n",
+                  chan->name[0] ? chan->name : chan->dname);
+      }
     } else
       check_lonely_channel(chan);
   }
@@ -1954,7 +2029,7 @@ static int gotpart(char *from, char *msg)
 static int gotkick(char *from, char *origmsg)
 {
   char *nick, *whodid, *chname, s1[UHOSTLEN], buf[UHOSTLEN], *uhost = buf;
-  char buf2[511], *msg;
+  char buf2[511], *msg, *key;
   memberlist *m;
   struct chanset_t *chan;
   struct userrec *u;
@@ -1971,9 +2046,14 @@ static int gotkick(char *from, char *origmsg)
   if (match_my_nick(nick) && channel_pending(chan) &&
       !channel_inactive(chan)) {
     chan->status &= ~(CHAN_ACTIVE | CHAN_PEND);
-    dprintf(DP_MODE, "JOIN %s %s\n",
-            (chan->name[0]) ? chan->name : chan->dname,
-            chan->channel.key[0] ? chan->channel.key : chan->key_prot);
+
+    key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
+    if (key[0])
+      dprintf(DP_SERVER, "JOIN %s %s\n",
+              chan->name[0] ? chan->name : chan->dname, key);
+    else
+      dprintf(DP_SERVER, "JOIN %s\n",
+              chan->name[0] ? chan->name : chan->dname);
     clear_channel(chan, 1);
     return 0;                   /* rejoin if kicked before getting needed info <Wcc[08/08/02]> */
   }
@@ -2014,9 +2094,14 @@ static int gotkick(char *from, char *origmsg)
     /* Kicked ME?!? the sods! */
     if (match_my_nick(nick) && !channel_inactive(chan)) {
       chan->status &= ~(CHAN_ACTIVE | CHAN_PEND);
-      dprintf(DP_MODE, "JOIN %s %s\n",
-              (chan->name[0]) ? chan->name : chan->dname,
-              chan->channel.key[0] ? chan->channel.key : chan->key_prot);
+
+      key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
+      if (key[0])
+        dprintf(DP_SERVER, "JOIN %s %s\n",
+                chan->name[0] ? chan->name : chan->dname, key);
+      else
+        dprintf(DP_SERVER, "JOIN %s\n",
+                chan->name[0] ? chan->name : chan->dname);
       clear_channel(chan, 1);
     } else {
       killmember(chan, nick);
@@ -2465,33 +2550,33 @@ static int gotnotice(char *from, char *msg)
 }
 
 static cmd_t irc_raw[] = {
-  {"324",     "",   (Function) got324,       "irc:324"},
-  {"352",     "",   (Function) got352,       "irc:352"},
-  {"354",     "",   (Function) got354,       "irc:354"},
-  {"315",     "",   (Function) got315,       "irc:315"},
-  {"367",     "",   (Function) got367,       "irc:367"},
-  {"368",     "",   (Function) got368,       "irc:368"},
-  {"403",     "",   (Function) got403,       "irc:403"},
-  {"405",     "",   (Function) got405,       "irc:405"},
-  {"471",     "",   (Function) got471,       "irc:471"},
-  {"473",     "",   (Function) got473,       "irc:473"},
-  {"474",     "",   (Function) got474,       "irc:474"},
-  {"475",     "",   (Function) got475,       "irc:475"},
-  {"INVITE",  "",   (Function) gotinvite, "irc:invite"},
-  {"TOPIC",   "",   (Function) gottopic,   "irc:topic"},
-  {"331",     "",   (Function) got331,       "irc:331"},
-  {"332",     "",   (Function) got332,       "irc:332"},
-  {"JOIN",    "",   (Function) gotjoin,     "irc:join"},
-  {"PART",    "",   (Function) gotpart,     "irc:part"},
-  {"KICK",    "",   (Function) gotkick,     "irc:kick"},
-  {"NICK",    "",   (Function) gotnick,     "irc:nick"},
-  {"QUIT",    "",   (Function) gotquit,     "irc:quit"},
-  {"PRIVMSG", "",   (Function) gotmsg,       "irc:msg"},
-  {"NOTICE",  "",   (Function) gotnotice, "irc:notice"},
-  {"MODE",    "",   (Function) gotmode,     "irc:mode"},
-  {"346",     "",   (Function) got346,       "irc:346"},
-  {"347",     "",   (Function) got347,       "irc:347"},
-  {"348",     "",   (Function) got348,       "irc:348"},
-  {"349",     "",   (Function) got349,       "irc:349"},
+  {"324",     "",   (IntFunc) got324,       "irc:324"},
+  {"352",     "",   (IntFunc) got352,       "irc:352"},
+  {"354",     "",   (IntFunc) got354,       "irc:354"},
+  {"315",     "",   (IntFunc) got315,       "irc:315"},
+  {"367",     "",   (IntFunc) got367,       "irc:367"},
+  {"368",     "",   (IntFunc) got368,       "irc:368"},
+  {"403",     "",   (IntFunc) got403,       "irc:403"},
+  {"405",     "",   (IntFunc) got405,       "irc:405"},
+  {"471",     "",   (IntFunc) got471,       "irc:471"},
+  {"473",     "",   (IntFunc) got473,       "irc:473"},
+  {"474",     "",   (IntFunc) got474,       "irc:474"},
+  {"475",     "",   (IntFunc) got475,       "irc:475"},
+  {"INVITE",  "",   (IntFunc) gotinvite, "irc:invite"},
+  {"TOPIC",   "",   (IntFunc) gottopic,   "irc:topic"},
+  {"331",     "",   (IntFunc) got331,       "irc:331"},
+  {"332",     "",   (IntFunc) got332,       "irc:332"},
+  {"JOIN",    "",   (IntFunc) gotjoin,     "irc:join"},
+  {"PART",    "",   (IntFunc) gotpart,     "irc:part"},
+  {"KICK",    "",   (IntFunc) gotkick,     "irc:kick"},
+  {"NICK",    "",   (IntFunc) gotnick,     "irc:nick"},
+  {"QUIT",    "",   (IntFunc) gotquit,     "irc:quit"},
+  {"PRIVMSG", "",   (IntFunc) gotmsg,       "irc:msg"},
+  {"NOTICE",  "",   (IntFunc) gotnotice, "irc:notice"},
+  {"MODE",    "",   (IntFunc) gotmode,     "irc:mode"},
+  {"346",     "",   (IntFunc) got346,       "irc:346"},
+  {"347",     "",   (IntFunc) got347,       "irc:347"},
+  {"348",     "",   (IntFunc) got348,       "irc:348"},
+  {"349",     "",   (IntFunc) got349,       "irc:349"},
   {NULL,      NULL, NULL,                         NULL}
 };
