@@ -2,11 +2,11 @@
  * chancmds.c -- part of irc.mod
  *   handles commands directly relating to channel interaction
  *
- * $Id: cmdsirc.c,v 1.62 2008-02-16 21:41:09 guppy Exp $
+ * $Id: cmdsirc.c,v 1.66 2010/02/18 09:52:29 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -1001,12 +1001,12 @@ static void cmd_adduser(struct userrec *u, int idx, char *par)
     int ok = 1;
 
     for (p = par; *p; p++)
-      if ((*p <= 32) || (*p >= 127))
+      if ((unsigned char) *p <= 32)
         ok = 0;
     if (!ok) {
       dprintf(idx, "You can't have strange characters in a nick.\n");
       return;
-    } else if (strchr("-,+*=:!.@#;$", par[0]) != NULL) {
+    } else if (strchr(BADHANDCHARS, par[0]) != NULL) {
       dprintf(idx, "You can't start a nick with '%c'.\n", par[0]);
       return;
     }
@@ -1143,7 +1143,7 @@ static void cmd_reset(struct userrec *u, int idx, char *par)
       else {
         putlog(LOG_CMDS, "*", "#%s# reset %s", dcc[idx].nick, par);
         dprintf(idx, "Resetting channel info for %s...\n", chan->dname);
-        reset_chan_info(chan);
+        reset_chan_info(chan, CHAN_RESETALL);
       }
     }
   } else if (!(u->flags & USER_MASTER))
@@ -1153,31 +1153,31 @@ static void cmd_reset(struct userrec *u, int idx, char *par)
     dprintf(idx, "Resetting channel info for all channels...\n");
     for (chan = chanset; chan; chan = chan->next) {
       if (channel_active(chan))
-        reset_chan_info(chan);
+        reset_chan_info(chan, CHAN_RESETALL);
     }
   }
 }
 
 static cmd_t irc_dcc[] = {
-  {"adduser",      "m|m",   (Function) cmd_adduser,      NULL},
-  {"deluser",      "m|m",   (Function) cmd_deluser,      NULL},
-  {"reset",        "m|m",   (Function) cmd_reset,        NULL},
-  {"resetbans",    "o|o",   (Function) cmd_resetbans,    NULL},
-  {"resetexempts", "o|o",   (Function) cmd_resetexempts, NULL},
-  {"resetinvites", "o|o",   (Function) cmd_resetinvites, NULL},
-  {"act",          "o|o",   (Function) cmd_act,          NULL},
-  {"channel",      "o|o",   (Function) cmd_channel,      NULL},
-  {"op",           "o|o",   (Function) cmd_op,           NULL},
-  {"deop",         "o|o",   (Function) cmd_deop,         NULL},
-  {"halfop",       "ol|ol", (Function) cmd_halfop,       NULL},
-  {"dehalfop",     "ol|ol", (Function) cmd_dehalfop,     NULL},
-  {"voice",        "ov|ov", (Function) cmd_voice,        NULL},
-  {"devoice",      "ov|ov", (Function) cmd_devoice,      NULL},
-  {"invite",       "o|o",   (Function) cmd_invite,       NULL},
-  {"kick",         "lo|lo", (Function) cmd_kick,         NULL},
-  {"kickban",      "lo|lo", (Function) cmd_kickban,      NULL},
-  {"msg",          "o",     (Function) cmd_msg,          NULL},
-  {"say",          "o|o",   (Function) cmd_say,          NULL},
-  {"topic",        "lo|lo", (Function) cmd_topic,        NULL},
+  {"adduser",      "m|m",   (IntFunc) cmd_adduser,      NULL},
+  {"deluser",      "m|m",   (IntFunc) cmd_deluser,      NULL},
+  {"reset",        "m|m",   (IntFunc) cmd_reset,        NULL},
+  {"resetbans",    "o|o",   (IntFunc) cmd_resetbans,    NULL},
+  {"resetexempts", "o|o",   (IntFunc) cmd_resetexempts, NULL},
+  {"resetinvites", "o|o",   (IntFunc) cmd_resetinvites, NULL},
+  {"act",          "o|o",   (IntFunc) cmd_act,          NULL},
+  {"channel",      "o|o",   (IntFunc) cmd_channel,      NULL},
+  {"op",           "o|o",   (IntFunc) cmd_op,           NULL},
+  {"deop",         "o|o",   (IntFunc) cmd_deop,         NULL},
+  {"halfop",       "ol|ol", (IntFunc) cmd_halfop,       NULL},
+  {"dehalfop",     "ol|ol", (IntFunc) cmd_dehalfop,     NULL},
+  {"voice",        "ov|ov", (IntFunc) cmd_voice,        NULL},
+  {"devoice",      "ov|ov", (IntFunc) cmd_devoice,      NULL},
+  {"invite",       "o|o",   (IntFunc) cmd_invite,       NULL},
+  {"kick",         "lo|lo", (IntFunc) cmd_kick,         NULL},
+  {"kickban",      "lo|lo", (IntFunc) cmd_kickban,      NULL},
+  {"msg",          "o",     (IntFunc) cmd_msg,          NULL},
+  {"say",          "o|o",   (IntFunc) cmd_say,          NULL},
+  {"topic",        "lo|lo", (IntFunc) cmd_topic,        NULL},
   {NULL,           NULL,    NULL,                        NULL}
 };
