@@ -2,11 +2,11 @@
  * irc.c -- part of irc.mod
  *   support for channels within the bot
  *
- * $Id: irc.c,v 1.106 2008-02-16 21:41:09 guppy Exp $
+ * $Id: irc.c,v 1.119 2010/02/18 09:52:29 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -173,7 +173,7 @@ static void punish_badguy(struct chanset_t *chan, char *whobad,
     /* ... or creating new user and setting that to deop */
     else {
       strcpy(s1, whobad);
-      maskban(s1, s);
+      maskaddr(s1, s, chan->ban_type);
       strcpy(s1, badnick);
       /* If that handle exists use "badX" (where X is an increasing number)
        * instead.
@@ -209,7 +209,7 @@ static void punish_badguy(struct chanset_t *chan, char *whobad,
     char s[UHOSTLEN], s1[UHOSTLEN];
 
     splitnick(&whobad);
-    maskban(whobad, s1);
+    maskaddr(whobad, s1, chan->ban_type);
     simple_sprintf(s, "(%s) %s", ct, reason);
     u_addban(chan, s1, botnetnick, s, now + (60 * chan->ban_time), 0);
     if (!mevictim && HALFOP_CANDOMODE('b')) {
@@ -281,6 +281,15 @@ static int hand_on_chan(struct chanset_t *chan, struct userrec *u)
   return 0;
 }
 
+static void refresh_who_chan(char *channame)
+{
+  if (use_354)
+    dprintf(DP_MODE, "WHO %s c%%chnuf\n", channame);
+  else
+    dprintf(DP_MODE, "WHO %s\n", channame);
+  return;
+}
+
 /* Adds a ban, exempt or invite mask to the list
  * m should be chan->channel.(exempt|invite|ban)
  */
@@ -401,43 +410,54 @@ static int any_ops(struct chanset_t *chan)
   return 1;
 }
 
-/* Reset the channel information.
+/* Reset channel information.
  */
-static void reset_chan_info(struct chanset_t *chan)
+static void reset_chan_info(struct chanset_t *chan, int reset)
 {
-  /* Don't reset the channel if we're already resetting it */
+  char beI[4] = "\0";
+  /* Leave the channel if we aren't supposed to be there */
   if (channel_inactive(chan)) {
     dprintf(DP_MODE, "PART %s\n", chan->name);
     return;
   }
-  if (!channel_pending(chan)) {
-    nfree(chan->channel.key);
-    chan->channel.key = (char *) channel_malloc(1);
-    chan->channel.key[0] = 0;
-    clear_channel(chan, 1);
-    chan->status |= CHAN_PEND;
-    chan->status &= ~(CHAN_ACTIVE | CHAN_ASKEDMODES);
-    if (!(chan->status & CHAN_ASKEDBANS)) {
-      chan->status |= CHAN_ASKEDBANS;
-      dprintf(DP_MODE, "MODE %s +b\n", chan->name);
-    }
-    if (!(chan->ircnet_status & CHAN_ASKED_EXEMPTS) && use_exempts == 1) {
-      chan->ircnet_status |= CHAN_ASKED_EXEMPTS;
-      dprintf(DP_MODE, "MODE %s +e\n", chan->name);
-    }
-    if (!(chan->ircnet_status & CHAN_ASKED_INVITED) && use_invites == 1) {
-      chan->ircnet_status |= CHAN_ASKED_INVITED;
-      dprintf(DP_MODE, "MODE %s +I\n", chan->name);
-    }
-    /* These 2 need to get out asap, so into the mode queue */
-    dprintf(DP_MODE, "MODE %s\n", chan->name);
-    if (use_354)
-      dprintf(DP_MODE, "WHO %s %%c%%h%%n%%u%%f\n", chan->name);
-    else
-      dprintf(DP_MODE, "WHO %s\n", chan->name);
-    /* clear_channel nuked the data...so */
-    dprintf(DP_MODE, "TOPIC %s\r\n", chan->name);
+
+  /* Don't reset the channel if we're already resetting it */
+  if (channel_pending(chan))
+    return;
+
+  clear_channel(chan, reset);
+  if ((reset & CHAN_RESETBANS) && !(chan->status & CHAN_ASKEDBANS)) {
+    chan->status |= CHAN_ASKEDBANS;
+    strcat(beI, "b");
   }
+  if ((reset & CHAN_RESETEXEMPTS) &&
+      !(chan->ircnet_status & CHAN_ASKED_EXEMPTS) && (use_exempts == 1)) {
+    chan->ircnet_status |= CHAN_ASKED_EXEMPTS;
+    strcat(beI, "e");
+  }
+  if ((reset & CHAN_RESETINVITED) &&
+      !(chan->ircnet_status & CHAN_ASKED_INVITED) && (use_invites == 1)) {
+    chan->ircnet_status |= CHAN_ASKED_INVITED;
+    strcat(beI, "I");
+  }
+  if (beI[0])
+    dprintf(DP_MODE, "MODE %s +%s\n", chan->name, beI);
+  if (reset & CHAN_RESETMODES) {
+    /* done here to keep expmem happy, as this is accounted in
+       irc.mod, not channels.mod where clear_channel() resides */
+    nfree(chan->channel.key);
+    chan->channel.key = (char *) channel_malloc (1);
+    chan->channel.key[0] = 0;
+    chan->status &= ~CHAN_ASKEDMODES;
+    dprintf(DP_MODE, "MODE %s\n", chan->name);
+  }
+  if (reset & CHAN_RESETWHO) {
+    chan->status |= CHAN_PEND;
+    chan->status &= ~CHAN_ACTIVE;
+    refresh_who_chan(chan->name);
+  }
+  if (reset & CHAN_RESETTOPIC)
+    dprintf(DP_MODE, "TOPIC %s\n", chan->name);
 }
 
 /* Leave the specified channel and notify registered Tcl procs. This
@@ -514,7 +534,6 @@ static void check_lonely_channel(struct chanset_t *chan)
   memberlist *m;
   char s[UHOSTLEN];
   int i = 0;
-  static int whined = 0;
 
   if (channel_pending(chan) || !channel_active(chan) || me_op(chan) ||
       channel_inactive(chan) || (chan->channel.mode & CHANANON))
@@ -527,13 +546,18 @@ static void check_lonely_channel(struct chanset_t *chan)
     if (chan->name[0] != '+') { /* Its pointless to cycle + chans for ops */
       putlog(LOG_MISC, "*", "Trying to cycle %s to regain ops.", chan->dname);
       dprintf(DP_MODE, "PART %s\n", chan->name);
+
       /* If it's a !chan, we need to recreate the channel with !!chan <cybah> */
-      dprintf(DP_MODE, "JOIN %s%s %s\n", (chan->dname[0] == '!') ? "!" : "",
-              chan->dname, chan->key_prot);
-      whined = 0;
+      if (chan->key_prot[0])
+        dprintf(DP_MODE, "JOIN %s%s %s\n", (chan->dname[0] == '!') ? "!" : "",
+                chan->dname, chan->key_prot);
+      else
+        dprintf(DP_MODE, "JOIN %s%s\n", (chan->dname[0] == '!') ? "!" : "",
+                chan->dname);
+      chan->status &= ~CHAN_WHINED;
     }
   } else if (any_ops(chan)) {
-    whined = 0;
+    chan->status &= ~CHAN_WHINED;
     check_tcl_need(chan->dname, "op");
     if (chan->need_op[0])
       do_tcl("need-op", chan->need_op);
@@ -544,13 +568,13 @@ static void check_lonely_channel(struct chanset_t *chan)
     int ok = 1;
     struct userrec *u;
 
-    if (!whined) {
+    if (!channel_whined(chan)) {
       /* + is opless. Complaining about no ops when without special
        * help(services), we cant get them - Raist
        */
       if (chan->name[0] != '+' && channel_logstatus(chan))
         putlog(LOG_MISC, "*", "%s is active but has no ops :(", chan->dname);
-      whined = 1;
+      chan->status |= CHAN_WHINED;
     }
     for (m = chan->channel.member; m && m->nick[0]; m = m->next) {
       sprintf(s, "%s!%s", m->nick, m->userhost);
@@ -578,7 +602,7 @@ static void check_expired_chanstuff()
 {
   masklist *b, *e;
   memberlist *m, *n;
-  char s[UHOSTLEN];
+  char *key, s[UHOSTLEN];
   struct chanset_t *chan;
   struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
 
@@ -609,8 +633,7 @@ static void check_expired_chanstuff()
               int match = 0;
 
               for (b = chan->channel.ban; b->mask[0]; b = b->next)
-                if (wild_match(b->mask, e->mask) ||
-                    wild_match(e->mask, b->mask)) {
+                if (mask_match(b->mask, e->mask)) {
                   match = 1;
                   break;
                 }
@@ -674,10 +697,16 @@ static void check_expired_chanstuff()
         m = n;
       }
       check_lonely_channel(chan);
-    } else if (!channel_inactive(chan) && !channel_pending(chan))
-      dprintf(DP_MODE, "JOIN %s %s\n",
-              (chan->name[0]) ? chan->name : chan->dname,
-              chan->channel.key[0] ? chan->channel.key : chan->key_prot);
+    } else if (!channel_inactive(chan) && !channel_pending(chan)) {
+
+      key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
+      if (key[0])
+        dprintf(DP_SERVER, "JOIN %s %s\n",
+                chan->name[0] ? chan->name : chan->dname, key);
+      else
+        dprintf(DP_SERVER, "JOIN %s\n",
+                chan->name[0] ? chan->name : chan->dname);
+    }
   }
 }
 
@@ -689,7 +718,7 @@ static int channels_6char STDVAR
   BADARGS(7, 7, " nick user@host handle desto/chan keyword/nick text");
 
   CHECKVALIDITY(channels_6char);
-  sprintf(x, "%d", F(argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]));
+  sprintf(x, "%d", (int) F(argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]));
   Tcl_AppendResult(irp, x, NULL);
   return TCL_OK;
 }
@@ -783,25 +812,40 @@ static void check_tcl_signtopcnick(char *nick, char *uhost, struct userrec *u,
                  MATCH_MASK | BIND_USE_ATTR | BIND_STACKABLE);
 }
 
-static void check_tcl_kickmode(char *nick, char *uhost, struct userrec *u,
-                               char *chname, char *dest, char *reason,
-                               p_tcl_bind_list table)
+static void check_tcl_mode(char *nick, char *uhost, struct userrec *u,
+                           char *chname, char *mode, char *target)
 {
   struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
   char args[512];
 
   get_user_flagrec(u, &fr, chname);
-  if (table == H_mode)
-    simple_sprintf(args, "%s %s", chname, dest);
-  else
-    simple_sprintf(args, "%s %s %s", chname, dest, reason);
+  simple_sprintf(args, "%s %s", chname, mode);
+  Tcl_SetVar(interp, "_mode1", nick, 0);
+  Tcl_SetVar(interp, "_mode2", uhost, 0);
+  Tcl_SetVar(interp, "_mode3", u ? u->handle : "*", 0);
+  Tcl_SetVar(interp, "_mode4", chname, 0);
+  Tcl_SetVar(interp, "_mode5", mode, 0);
+  Tcl_SetVar(interp, "_mode6", target, 0);
+  check_tcl_bind(H_mode, args, &fr,
+                 " $_mode1 $_mode2 $_mode3 $_mode4 $_mode5 $_mode6",
+                 MATCH_MODE | BIND_USE_ATTR | BIND_STACKABLE);
+}
+
+static void check_tcl_kick(char *nick, char *uhost, struct userrec *u,
+                           char *chname, char *dest, char *reason)
+{
+  struct flag_record fr = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
+  char args[512];
+
+  get_user_flagrec(u, &fr, chname);
+  simple_sprintf(args, "%s %s %s", chname, dest, reason);
   Tcl_SetVar(interp, "_kick1", nick, 0);
   Tcl_SetVar(interp, "_kick2", uhost, 0);
   Tcl_SetVar(interp, "_kick3", u ? u->handle : "*", 0);
   Tcl_SetVar(interp, "_kick4", chname, 0);
   Tcl_SetVar(interp, "_kick5", dest, 0);
   Tcl_SetVar(interp, "_kick6", reason, 0);
-  check_tcl_bind(table, args, &fr,
+  check_tcl_bind(H_kick, args, &fr,
                  " $_kick1 $_kick2 $_kick3 $_kick4 $_kick5 $_kick6",
                  MATCH_MASK | BIND_USE_ATTR | BIND_STACKABLE);
 }
@@ -1056,7 +1100,7 @@ static void do_nettype()
     break;
   }
   /* Update all rfc_ function pointers */
-  add_hook(HOOK_RFC_CASECMP, (Function) rfc_compliant);
+  add_hook(HOOK_RFC_CASECMP, (Function) (intptr_t) rfc_compliant);
 }
 
 static char *traced_nettype(ClientData cdata, Tcl_Interp *irp,
@@ -1075,7 +1119,7 @@ static char *traced_rfccompliant(ClientData cdata, Tcl_Interp *irp,
    * links to point to the rfc compliant versions if rfc_compliant
    * is 1, or to the normal version if it's 0.
    */
-  add_hook(HOOK_RFC_CASECMP, (Function) rfc_compliant);
+  add_hook(HOOK_RFC_CASECMP, (Function) (intptr_t) rfc_compliant);
   return NULL;
 }
 
@@ -1088,7 +1132,9 @@ static char *irc_close()
 {
   struct chanset_t *chan;
 
+  /* Force bot to part all channels */
   dprintf(DP_MODE, "JOIN 0\n");
+
   for (chan = chanset; chan; chan = chan->next)
     clear_channel(chan, 1);
   del_bind_table(H_topc);
@@ -1167,23 +1213,28 @@ char *irc_start(Function *global_funcs)
 
   global = global_funcs;
 
-  module_register(MODULE_NAME, irc_table, 1, 3);
-  if (!module_depend(MODULE_NAME, "eggdrop", 106, 0)) {
+  module_register(MODULE_NAME, irc_table, 1, 4);
+  if (!module_depend(MODULE_NAME, "eggdrop", 106, 20)) {
     module_undepend(MODULE_NAME);
-    return "This module requires Eggdrop 1.6.0 or later.";
+    return "This module requires Eggdrop 1.6.20 or later.";
   }
   if (!(server_funcs = module_depend(MODULE_NAME, "server", 1, 0))) {
     module_undepend(MODULE_NAME);
     return "This module requires server module 1.0 or later.";
   }
-  if (!(channels_funcs = module_depend(MODULE_NAME, "channels", 1, 0))) {
+  if (!(channels_funcs = module_depend(MODULE_NAME, "channels", 1, 1))) {
     module_undepend(MODULE_NAME);
-    return "This module requires channels module 1.0 or later.";
+    return "This module requires channels module 1.1 or later.";
   }
   for (chan = chanset; chan; chan = chan->next) {
-    if (!channel_inactive(chan))
-      dprintf(DP_MODE, "JOIN %s %s\n",
-              (chan->name[0]) ? chan->name : chan->dname, chan->key_prot);
+    if (!channel_inactive(chan)) {
+      if (chan->key_prot[0])
+        dprintf(DP_SERVER, "JOIN %s %s\n",
+                chan->name[0] ? chan->name : chan->dname, chan->key_prot);
+      else
+        dprintf(DP_SERVER, "JOIN %s\n",
+                chan->name[0] ? chan->name : chan->dname);
+    }
     chan->status &= ~(CHAN_ACTIVE | CHAN_PEND | CHAN_ASKEDBANS);
     chan->ircnet_status &= ~(CHAN_ASKED_INVITED | CHAN_ASKED_EXEMPTS);
   }
