@@ -1,11 +1,11 @@
 /*
  * irc.h -- part of irc.mod
  *
- * $Id: irc.h,v 1.30 2008-02-16 21:41:09 guppy Exp $
+ * $Id: irc.h,v 1.35 2010/02/06 09:20:12 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,16 +31,14 @@
 #define check_tcl_sign(a,b,c,d,e) check_tcl_signtopcnick(a,b,c,d,e,H_sign)
 #define check_tcl_topc(a,b,c,d,e) check_tcl_signtopcnick(a,b,c,d,e,H_topc)
 #define check_tcl_nick(a,b,c,d,e) check_tcl_signtopcnick(a,b,c,d,e,H_nick)
-#define check_tcl_mode(a,b,c,d,e,f) check_tcl_kickmode(a,b,c,d,e,f,H_mode)
-#define check_tcl_kick(a,b,c,d,e,f) check_tcl_kickmode(a,b,c,d,e,f,H_kick)
 
 #define REVENGE_KICK 1          /* Kicked victim        */
 #define REVENGE_DEOP 2          /* Took op              */
 
 #ifdef MAKING_IRC
 static void check_tcl_need(char *, char *);
-static void check_tcl_kickmode(char *, char *, struct userrec *, char *,
-                               char *, char *, p_tcl_bind_list);
+static void check_tcl_kick(char *, char *, struct userrec *, char *, char *, char *);
+static void check_tcl_mode(char *, char *, struct userrec *, char *, char *, char *);
 static void check_tcl_joinspltrejn(char *, char *, struct userrec *, char *,
                                    p_tcl_bind_list);
 static void check_tcl_part(char *, char *, struct userrec *, char *, char *);
@@ -56,6 +54,7 @@ static int hand_on_chan(struct chanset_t *, struct userrec *);
 static char *getchanmode(struct chanset_t *);
 static void flush_mode(struct chanset_t *, int);
 static void set_delay(struct chanset_t *, char *);
+static void refresh_who_chan(char *);
 
 /* reset(bans|exempts|invites) are now just macros that call resetmasks
  * in order to reduce the code duplication. <cybah>
@@ -67,7 +66,7 @@ static void set_delay(struct chanset_t *, char *);
 #define resetinvites(chan)  resetmasks((chan), (chan)->channel.invite,       \
                                        (chan)->invites, global_invites, 'I')
 
-static void reset_chan_info(struct chanset_t *);
+static void reset_chan_info(struct chanset_t *, int);
 static void recheck_channel(struct chanset_t *, int);
 static void set_key(struct chanset_t *, char *);
 static void maybe_revenge(struct chanset_t *, char *, char *, int);
@@ -89,7 +88,6 @@ static int gotmode(char *, char *);
                                                 who)
 #define newinvite(chan, mask, who)      newmask((chan)->channel.invite, mask, \
                                                 who)
-
 #else
 /* 4 - 7 */
 #define H_splt (*(p_tcl_bind_list*)(irc_funcs[4]))
