@@ -4,11 +4,11 @@
  *   channel mode changes and the bot's reaction to them
  *   setting and getting the current wanted channel modes
  *
- * $Id: mode.c,v 1.83 2008-02-16 21:41:09 guppy Exp $
+ * $Id: mode.c,v 1.90 2010/07/01 16:10:49 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,6 +40,30 @@ static int reversing = 0;
 static struct flag_record user = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
 static struct flag_record victim = { FR_GLOBAL | FR_CHAN, 0, 0, 0, 0, 0 };
 
+/* This is called after a mode bind is processed. It will check if the
+ * channel still exists and will refresh the user and victim flag records,
+ * in case users were also modified.
+ */
+struct chanset_t *modebind_refresh(char *chname,
+                                   char *usrhost, struct flag_record *usr,
+                                   char *vcrhost, struct flag_record *vcr)
+{
+  struct userrec *u;
+  struct chanset_t *chan;
+
+  if (!chname || !(chan = findchan(chname)))
+    return NULL;
+  if (usrhost) {
+    u = get_user_by_host(usrhost);
+    get_user_flagrec(u, usr, chan->dname);
+  }
+  if (vcrhost) {
+    u = get_user_by_host(vcrhost);
+    get_user_flagrec(u, vcr, chan->dname);
+  }
+  return chan;
+}
+
 static void flush_mode(struct chanset_t *chan, int pri)
 {
   char *p, out[512], post[512];
@@ -182,7 +206,7 @@ static void real_add_mode(struct chanset_t *chan,
   memberlist *mx;
   char s[21];
 
-  /* Some IRCds do not allow halfops to set certian modes. The modes halfops
+  /* Some IRCds do not allow halfops to set certain modes. The modes halfops
    * are not allowed to set can be changed in chan.h. */
 #ifdef NO_HALFOP_CHANMODES
   if (!me_op(chan))
@@ -379,6 +403,7 @@ static void got_op(struct chanset_t *chan, char *nick, char *from,
                    char *who, struct userrec *opu, struct flag_record *opper)
 {
   memberlist *m;
+  char ch[sizeof chan->name];
   char s[UHOSTLEN];
   struct userrec *u;
   int check_chan = 0, snm = chan->stopnethack_mode;
@@ -388,17 +413,20 @@ static void got_op(struct chanset_t *chan, char *nick, char *from,
     if (channel_pending(chan))
       return;
     putlog(LOG_MISC, chan->dname, CHAN_BADCHANMODE, chan->dname, who);
-    dprintf(DP_MODE, "WHO %s\n", who);
+    chan->status |= CHAN_PEND;
+    refresh_who_chan(chan->name);
     return;
   }
+
   /* Did *I* just get opped? */
   if (!me_op(chan) && match_my_nick(who))
     check_chan = 1;
 
-  if (!m->user) {
-    simple_sprintf(s, "%s!%s", m->nick, m->userhost);
+  strcpy(ch, chan->name);
+  simple_sprintf(s, "%s!%s", m->nick, m->userhost);
+  if (!m->user)
     u = get_user_by_host(s);
-  } else
+  else
     u = m->user;
 
   get_user_flagrec(u, &victim, chan->dname);
@@ -407,6 +435,9 @@ static void got_op(struct chanset_t *chan, char *nick, char *from,
    */
   m->flags |= CHANOP;
   check_tcl_mode(nick, from, opu, chan->dname, "+o", who);
+  if (!(chan = modebind_refresh(ch, from, opper, s, &victim)) ||
+      !(m = ismember(chan, who)))
+    return;
   /* Added new meaning of WASOP:
    * In mode binds it means: was the user an op before got (de)opped. A script
    * now can use [wasop nick chan] to check if user was op or wasnt.
@@ -470,6 +501,7 @@ static void got_halfop(struct chanset_t *chan, char *nick, char *from,
 {
   memberlist *m;
   char s[UHOSTLEN];
+  char ch[sizeof chan->name];
   struct userrec *u;
   int check_chan = 0;
   int snm = chan->stopnethack_mode;
@@ -479,16 +511,20 @@ static void got_halfop(struct chanset_t *chan, char *nick, char *from,
     if (channel_pending(chan))
       return;
     putlog(LOG_MISC, chan->dname, CHAN_BADCHANMODE, chan->dname, who);
-    dprintf(DP_MODE, "WHO %s\n", who);
+    chan->status |= CHAN_PEND;
+    refresh_who_chan(chan->name);
     return;
   }
-  if (!me_op(chan) && match_my_nick(who))
+
+  /* Did *I* just get halfopped? */
+  if (!me_op(chan) && !me_halfop(chan) && match_my_nick(who))
     check_chan = 1;
 
-  if (!m->user) {
-    simple_sprintf(s, "%s!%s", m->nick, m->userhost);
+  strcpy(ch, chan->name);
+  simple_sprintf(s, "%s!%s", m->nick, m->userhost);
+  if (!m->user)
     u = get_user_by_host(s);
-  } else
+  else
     u = m->user;
 
   get_user_flagrec(u, &victim, chan->dname);
@@ -497,6 +533,9 @@ static void got_halfop(struct chanset_t *chan, char *nick, char *from,
    */
   m->flags |= CHANHALFOP;
   check_tcl_mode(nick, from, opu, chan->dname, "+h", who);
+  if (!(chan = modebind_refresh(ch, from, opper, s, &victim)) ||
+      !(m = ismember(chan, who)))
+    return;
   m->flags &= ~SENTHALFOP;
 
   if (channel_pending(chan))
@@ -557,6 +596,7 @@ static void got_deop(struct chanset_t *chan, char *nick, char *from,
                      char *who, struct userrec *opu)
 {
   memberlist *m;
+  char ch[sizeof chan->name];
   char s[UHOSTLEN], s1[UHOSTLEN];
   struct userrec *u;
   int had_halfop;
@@ -566,10 +606,12 @@ static void got_deop(struct chanset_t *chan, char *nick, char *from,
     if (channel_pending(chan))
       return;
     putlog(LOG_MISC, chan->dname, CHAN_BADCHANMODE, chan->dname, who);
-    dprintf(DP_MODE, "WHO %s\n", who);
+    chan->status |= CHAN_PEND;
+    refresh_who_chan(chan->name);
     return;
   }
 
+  strcpy(ch, chan->name);
   simple_sprintf(s, "%s!%s", m->nick, m->userhost);
   simple_sprintf(s1, "%s!%s", nick, from);
   u = get_user_by_host(s);
@@ -581,6 +623,9 @@ static void got_deop(struct chanset_t *chan, char *nick, char *from,
    */
   m->flags &= ~(CHANOP | SENTDEOP | FAKEOP);
   check_tcl_mode(nick, from, opu, chan->dname, "-o", who);
+  if (!(chan = modebind_refresh(ch, from, &user, s, &victim)) ||
+      !(m = ismember(chan, who)))
+    return;
   m->flags &= ~WASOP;
 
   if (channel_pending(chan))
@@ -607,20 +652,20 @@ static void got_deop(struct chanset_t *chan, char *nick, char *from,
   if (!nick[0])
     putlog(LOG_MODES, chan->dname, "TS resync (%s): %s deopped by %s",
            chan->dname, who, from);
+
   /* Check for mass deop */
   if (nick[0])
     detect_chan_flood(nick, from, s1, chan, FLOOD_DEOP, who);
-  /* Having op hides your +v and +h  status -- so now that someone's lost ops,
+
+  /* Having op hides your +v and +h status -- so now that someone's lost ops,
    * check to see if they have +v or +h
    */
-  if (!(m->flags & (CHANVOICE | STOPWHO))) {
-    dprintf(DP_HELP, "WHO %s\n", m->nick);
-    m->flags |= STOPWHO;
-  }
-  if (!(m->flags & (CHANHALFOP | STOPWHO))) {
-    dprintf(DP_HELP, "WHO %s\n", m->nick);
+  if (!(m->flags & (CHANVOICE | CHANHALFOP | STOPWHO))) {
+    chan->status |= CHAN_PEND;
+    refresh_who_chan(chan->name);
     m->flags |= STOPWHO;
   }
+
   /* Was the bot deopped? */
   if (match_my_nick(who)) {
     /* Cancel any pending kicks and modes */
@@ -644,6 +689,7 @@ static void got_dehalfop(struct chanset_t *chan, char *nick, char *from,
                          char *who, struct userrec *opu)
 {
   memberlist *m;
+  char ch[sizeof chan->name];
   char s[UHOSTLEN], s1[UHOSTLEN];
   struct userrec *u;
   int had_halfop;
@@ -653,10 +699,12 @@ static void got_dehalfop(struct chanset_t *chan, char *nick, char *from,
     if (channel_pending(chan))
       return;
     putlog(LOG_MISC, chan->dname, CHAN_BADCHANMODE, chan->dname, who);
-    dprintf(DP_MODE, "WHO %s\n", who);
+    chan->status |= CHAN_PEND;
+    refresh_who_chan(chan->name);
     return;
   }
 
+  strcpy(ch, chan->name);
   simple_sprintf(s, "%s!%s", m->nick, m->userhost);
   simple_sprintf(s1, "%s!%s", nick, from);
   u = get_user_by_host(s);
@@ -668,6 +716,9 @@ static void got_dehalfop(struct chanset_t *chan, char *nick, char *from,
    */
   m->flags &= ~(CHANHALFOP | SENTDEHALFOP | FAKEHALFOP);
   check_tcl_mode(nick, from, opu, chan->dname, "-h", who);
+  if (!(chan = modebind_refresh(ch, from, &user, s, &victim)) ||
+      !(m = ismember(chan, who)))
+    return;
   /* Check comments in got_op()  (drummer) */
   m->flags &= ~WASHALFOP;
 
@@ -698,25 +749,30 @@ static void got_dehalfop(struct chanset_t *chan, char *nick, char *from,
     putlog(LOG_MODES, chan->dname, "TS resync (%s): %s deopped by %s",
            chan->dname, who, from);
   if (!(m->flags & (CHANVOICE | STOPWHO))) {
-    dprintf(DP_HELP, "WHO %s\n", m->nick);
+    chan->status |= CHAN_PEND;
+    refresh_who_chan(chan->name);
     m->flags |= STOPWHO;
   }
 }
 
-static void got_ban(struct chanset_t *chan, char *nick, char *from, char *who)
+static void got_ban(struct chanset_t *chan, char *nick, char *from, char *who,
+                    char *ch, struct userrec *u)
 {
   char me[UHOSTLEN], s[UHOSTLEN], s1[UHOSTLEN];
   memberlist *m;
-  struct userrec *u;
+  struct userrec *targ;
 
   egg_snprintf(me, sizeof me, "%s!%s", botname, botuserhost);
   egg_snprintf(s, sizeof s, "%s!%s", nick, from);
   newban(chan, who, s);
+  check_tcl_mode(nick, from, u, chan->dname, "+b", who);
+  if (!(chan = modebind_refresh(ch, from, &user, NULL, NULL)))
+    return;
 
   if (channel_pending(chan) || HALFOP_CANTDOMODE('b'))
     return;
 
-  if (wild_match(who, me) && !isexempted(chan, me)) {
+  if (match_addr(who, me) && !isexempted(chan, me)) {
     add_mode(chan, '-', 'b', who);
     reversing = 1;
     return;
@@ -729,10 +785,10 @@ static void got_ban(struct chanset_t *chan, char *nick, char *from, char *who)
     }
     for (m = chan->channel.member; m && m->nick[0]; m = m->next) {
       egg_snprintf(s1, sizeof s1, "%s!%s", m->nick, m->userhost);
-      if (wild_match(who, s1)) {
-        u = get_user_by_host(s1);
-        if (u) {
-          get_user_flagrec(u, &victim, chan->dname);
+      if (match_addr(who, s1)) {
+        targ = get_user_by_host(s1);
+        if (targ) {
+          get_user_flagrec(targ, &victim, chan->dname);
           if ((glob_friend(victim) || (glob_op(victim) && !chan_deop(victim)) ||
                chan_friend(victim) || chan_op(victim)) && !glob_master(user) &&
               !glob_bot(user) && !chan_master(user) && !isexempted(chan, s1)) {
@@ -751,7 +807,7 @@ static void got_ban(struct chanset_t *chan, char *nick, char *from, char *who)
 
     for (cycle = 0; cycle < 2; cycle++) {
       for (b = cycle ? chan->bans : global_bans; b; b = b->next) {
-        if (wild_match(b->mask, who)) {
+        if (match_addr(b->mask, who)) {
           if (b->desc && b->desc[0] != '@')
             egg_snprintf(resn, sizeof resn, "%s %s", IRC_PREBANNED, b->desc);
           else
@@ -768,7 +824,7 @@ static void got_ban(struct chanset_t *chan, char *nick, char *from, char *who)
 }
 
 static void got_unban(struct chanset_t *chan, char *nick, char *from,
-                      char *who, struct userrec *u)
+                      char *who, char *ch, struct userrec *u)
 {
   masklist *b, *old;
 
@@ -784,6 +840,9 @@ static void got_unban(struct chanset_t *chan, char *nick, char *from,
     nfree(b->who);
     nfree(b);
   }
+  check_tcl_mode(nick, from, u, chan->dname, "-b", who);
+  if (!(chan = modebind_refresh(ch, from, &user, NULL, NULL)))
+    return;
 
   if (channel_pending(chan))
     return;
@@ -798,12 +857,15 @@ static void got_unban(struct chanset_t *chan, char *nick, char *from,
 }
 
 static void got_exempt(struct chanset_t *chan, char *nick, char *from,
-                       char *who)
+                       char *who, char *ch, struct userrec *u)
 {
   char s[UHOSTLEN];
 
   simple_sprintf(s, "%s!%s", nick, from);
   newexempt(chan, who, s);
+  check_tcl_mode(nick, from, u, chan->dname, "+e", who);
+  if (!(chan = modebind_refresh(ch, from, &user, NULL, NULL)))
+    return;
 
   if (channel_pending(chan) || HALFOP_CANTDOMODE('e'))
     return;
@@ -824,7 +886,7 @@ static void got_exempt(struct chanset_t *chan, char *nick, char *from,
 }
 
 static void got_unexempt(struct chanset_t *chan, char *nick, char *from,
-                         char *who, struct userrec *u)
+                         char *who, char *ch, struct userrec *u)
 {
   masklist *e = chan->channel.exempt, *old = NULL;
   masklist *b;
@@ -843,6 +905,9 @@ static void got_unexempt(struct chanset_t *chan, char *nick, char *from,
     nfree(e->who);
     nfree(e);
   }
+  check_tcl_mode(nick, from, u, chan->dname, "-e", who);
+  if (!(chan = modebind_refresh(ch, from, &user, NULL, NULL)))
+    return;
 
   if (channel_pending(chan))
     return;
@@ -854,7 +919,7 @@ static void got_unexempt(struct chanset_t *chan, char *nick, char *from,
   if (!nick[0] && glob_bot(user) && !glob_master(user) && !chan_master(user)) {
     b = chan->channel.ban;
     while (b->mask[0] && !match) {
-      if (wild_match(b->mask, who) || wild_match(who, b->mask)) {
+      if (mask_match(b->mask, who)) {
         add_mode(chan, '+', 'e', who);
         match = 1;
       } else
@@ -869,12 +934,15 @@ static void got_unexempt(struct chanset_t *chan, char *nick, char *from,
 }
 
 static void got_invite(struct chanset_t *chan, char *nick, char *from,
-                       char *who)
+                       char *who, char *ch, struct userrec *u)
 {
   char s[UHOSTLEN];
 
   simple_sprintf(s, "%s!%s", nick, from);
   newinvite(chan, who, s);
+  check_tcl_mode(nick, from, u, chan->dname, "+I", who);
+  if (!(chan = modebind_refresh(ch, from, &user, NULL, NULL)))
+    return;
 
   if (channel_pending(chan) || HALFOP_CANTDOMODE('I'))
     return;
@@ -895,7 +963,7 @@ static void got_invite(struct chanset_t *chan, char *nick, char *from,
 }
 
 static void got_uninvite(struct chanset_t *chan, char *nick, char *from,
-                         char *who, struct userrec *u)
+                         char *who, char *ch, struct userrec *u)
 {
   masklist *inv = chan->channel.invite, *old = NULL;
 
@@ -912,6 +980,9 @@ static void got_uninvite(struct chanset_t *chan, char *nick, char *from,
     nfree(inv->who);
     nfree(inv);
   }
+  check_tcl_mode(nick, from, u, chan->dname, "-I", who);
+  if (!(chan = modebind_refresh(ch, from, &user, NULL, NULL)))
+    return;
 
   if (channel_pending(chan))
     return;
@@ -1084,6 +1155,9 @@ static int gotmode(char *from, char *origmsg)
             reversing = 1;
           if (ms2[0] == '-') {
             check_tcl_mode(nick, from, u, chan->dname, ms2, "");
+            /* The Tcl proc might have modified/removed the chan or user */
+            if (!(chan = modebind_refresh(ch, from, &user, NULL, NULL)))
+              return 0;
             if (channel_active(chan)) {
               if (reversing && (chan->channel.maxmembers != 0)) {
                 simple_sprintf(s, "%d", chan->channel.maxmembers);
@@ -1103,6 +1177,9 @@ static int gotmode(char *from, char *origmsg)
             chan->channel.maxmembers = atoi(op);
             check_tcl_mode(nick, from, u, chan->dname, ms2,
                            int_to_base10(chan->channel.maxmembers));
+            /* The Tcl proc might have modified/removed the chan or user */
+            if (!(chan = modebind_refresh(ch, from, &user, NULL, NULL)))
+              return 0;
             if (channel_pending(chan))
               break;
             if ((reversing && !(chan->mode_pls_prot & CHANLIMIT)) ||
@@ -1127,6 +1204,10 @@ static int gotmode(char *from, char *origmsg)
           if (op == '\0') {
             break;
           }
+          check_tcl_mode(nick, from, u, chan->dname, ms2, op);
+          /* The Tcl proc might have modified/removed the chan or user */
+          if (!(chan = modebind_refresh(ch, from, &user, NULL, NULL)))
+            return 0;
           if (ms2[0] == '+') {
             set_key(chan, op);
             if (channel_active(chan))
@@ -1141,7 +1222,6 @@ static int gotmode(char *from, char *origmsg)
             }
             set_key(chan, NULL);
           }
-          check_tcl_mode(nick, from, u, chan->dname, ms2, op);
           break;
         case 'o':
           op = newsplit(&msg);
@@ -1167,7 +1247,8 @@ static int gotmode(char *from, char *origmsg)
             if (channel_pending(chan))
               break;
             putlog(LOG_MISC, chan->dname, CHAN_BADCHANMODE, chan->dname, op);
-            dprintf(DP_MODE, "WHO %s\n", op);
+            chan->status |= CHAN_PEND;
+            refresh_who_chan(chan->name);
           } else {
             simple_sprintf(s, "%s!%s", m->nick, m->userhost);
             get_user_flagrec(m->user ? m->user : get_user_by_host(s),
@@ -1176,6 +1257,8 @@ static int gotmode(char *from, char *origmsg)
               m->flags &= ~SENTVOICE;
               m->flags |= CHANVOICE;
               check_tcl_mode(nick, from, u, chan->dname, ms2, op);
+              if (!(chan = modebind_refresh(ch, from, &user, s, &victim)))
+                return 0;
               if (channel_active(chan) && !glob_master(user) &&
                   !chan_master(user) && !match_my_nick(nick)) {
                 if (chan_quiet(victim) ||
@@ -1188,6 +1271,8 @@ static int gotmode(char *from, char *origmsg)
               m->flags &= ~SENTDEVOICE;
               m->flags &= ~CHANVOICE;
               check_tcl_mode(nick, from, u, chan->dname, ms2, op);
+              if (!(chan = modebind_refresh(ch, from, &user, s, &victim)))
+                return 0;
               if (channel_active(chan) && !glob_master(user) &&
                   !chan_master(user) && !match_my_nick(nick)) {
                 if ((channel_autovoice(chan) && !chan_quiet(victim) &&
@@ -1205,32 +1290,31 @@ static int gotmode(char *from, char *origmsg)
           op = newsplit(&msg);
           fixcolon(op);
           if (ms2[0] == '+')
-            got_ban(chan, nick, from, op);
+            got_ban(chan, nick, from, op, ch, u);
           else
-            got_unban(chan, nick, from, op, u);
-          check_tcl_mode(nick, from, u, chan->dname, ms2, op);
+            got_unban(chan, nick, from, op, ch, u);
           break;
         case 'e':
           op = newsplit(&msg);
           fixcolon(op);
           if (ms2[0] == '+')
-            got_exempt(chan, nick, from, op);
+            got_exempt(chan, nick, from, op, ch, u);
           else
-            got_unexempt(chan, nick, from, op, u);
-          check_tcl_mode(nick, from, u, chan->dname, ms2, op);
+            got_unexempt(chan, nick, from, op, ch, u);
           break;
         case 'I':
           op = newsplit(&msg);
           fixcolon(op);
           if (ms2[0] == '+')
-            got_invite(chan, nick, from, op);
+            got_invite(chan, nick, from, op, ch, u);
           else
-            got_uninvite(chan, nick, from, op, u);
-          check_tcl_mode(nick, from, u, chan->dname, ms2, op);
+            got_uninvite(chan, nick, from, op, ch, u);
           break;
         }
         if (todo) {
           check_tcl_mode(nick, from, u, chan->dname, ms2, "");
+          if (!(chan = modebind_refresh(ch, from, &user, NULL, NULL)))
+            return 0;
           if (ms2[0] == '+')
             chan->channel.mode |= todo;
           else
