@@ -2,11 +2,11 @@
  * msgcmds.c -- part of irc.mod
  *   all commands entered via /MSG
  *
- * $Id: msgcmds.c,v 1.53 2008-02-16 21:41:09 guppy Exp $
+ * $Id: msgcmds.c,v 1.58 2010/02/18 09:52:29 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -674,10 +674,10 @@ static int msg_key(char *nick, char *host, struct userrec *u, char *par)
           if (chan->channel.key[0]) {
             dprintf(DP_SERVER, "NOTICE %s :%s: key is %s\n", nick, par,
                     chan->channel.key);
+            putlog(LOG_CMDS, "*", "(%s!%s) !%s! KEY %s", nick, host, u->handle,
+                   par);
             if (invite_key && (chan->channel.mode & CHANINV)) {
               dprintf(DP_SERVER, "INVITE %s %s\n", nick, chan->name);
-              putlog(LOG_CMDS, "*", "(%s!%s) !%s! KEY %s",
-                     nick, host, u->handle, par);
             }
           } else {
             dprintf(DP_HELP, "NOTICE %s :%s: no key set for this channel\n",
@@ -1001,13 +1001,13 @@ static int msg_reset(char *nick, char *host, struct userrec *u, char *par)
     }
     putlog(LOG_CMDS, "*", "(%s!%s) !%s! RESET %s", nick, host, u->handle, par);
     dprintf(DP_HELP, "NOTICE %s :%s: %s\n", nick, par, IRC_RESETCHAN);
-    reset_chan_info(chan);
+    reset_chan_info(chan, CHAN_RESETALL);
     return 1;
   }
   putlog(LOG_CMDS, "*", "(%s!%s) !%s! RESET ALL", nick, host, u->handle);
   dprintf(DP_HELP, "NOTICE %s :%s\n", nick, IRC_RESETCHAN);
   for (chan = chanset; chan; chan = chan->next)
-    reset_chan_info(chan);
+    reset_chan_info(chan, CHAN_RESETALL);
   return 1;
 }
 
@@ -1028,14 +1028,14 @@ static int msg_go(char *nick, char *host, struct userrec *u, char *par)
     if (!chan)
       return 0;
     if (!(chan->status & CHAN_ACTIVE)) {
-      putlog(LOG_CMDS, "*", "(%s!%s) !%s! failed GO (i'm blind)", nick, host,
-             u->handle);
+      putlog(LOG_CMDS, "*", "(%s!%s) !%s! failed GO %s (i'm blind)", nick,
+             host, u->handle, par);
       return 1;
     }
     get_user_flagrec(u, &fr, par);
     if (!chan_op(fr) && !(glob_op(fr) && !chan_deop(fr))) {
-      putlog(LOG_CMDS, "*", "(%s!%s) !%s! failed GO (not op)", nick, host,
-             u->handle);
+      putlog(LOG_CMDS, "*", "(%s!%s) !%s! failed GO %s (not op)", nick, host,
+             u->handle, par);
       return 1;
     }
     if (!me_op(chan)) {
@@ -1061,13 +1061,13 @@ static int msg_go(char *nick, char *host, struct userrec *u, char *par)
     }
   }
   if (ok) {
-    putlog(LOG_CMDS, "*", "(%s!%s) !%s! GO", nick, host, u->handle);
+    putlog(LOG_CMDS, "*", "(%s!%s) !%s! GO %s", nick, host, u->handle, par);
   } else if (ok2) {
-    putlog(LOG_CMDS, "*", "(%s!%s) !%s! failed GO (i'm chop)", nick, host,
-           u->handle);
+    putlog(LOG_CMDS, "*", "(%s!%s) !%s! failed GO %s (i'm chop)", nick, host,
+           u->handle, par);
   } else {
-    putlog(LOG_CMDS, "*", "(%s!%s) !%s! failed GO (not op)", nick, host,
-           u->handle);
+    putlog(LOG_CMDS, "*", "(%s!%s) !%s! failed GO %s (not op)", nick, host,
+           u->handle, par);
   }
   return 1;
 }
@@ -1117,26 +1117,26 @@ static int msg_jump(char *nick, char *host, struct userrec *u, char *par)
  * 0 if not.
  */
 static cmd_t C_msg[] = {
-  {"addhost", "",    (Function) msg_addhost, NULL},
-  {"die",     "n",   (Function) msg_die,     NULL},
-  {"go",      "",    (Function) msg_go,      NULL},
-  {"hello",   "",    (Function) msg_hello,   NULL},
-  {"help",    "",    (Function) msg_help,    NULL},
-  {"ident",   "",    (Function) msg_ident,   NULL},
-  {"info",    "",    (Function) msg_info,    NULL},
-  {"invite",  "o|o", (Function) msg_invite,  NULL},
-  {"jump",    "m",   (Function) msg_jump,    NULL},
-  {"key",     "o|o", (Function) msg_key,     NULL},
-  {"memory",  "m",   (Function) msg_memory,  NULL},
-  {"op",      "",    (Function) msg_op,      NULL},
-  {"halfop",  "",    (Function) msg_halfop,  NULL},
-  {"pass",    "",    (Function) msg_pass,    NULL},
-  {"rehash",  "m",   (Function) msg_rehash,  NULL},
-  {"reset",   "m",   (Function) msg_reset,   NULL},
-  {"save",    "m",   (Function) msg_save,    NULL},
-  {"status",  "m|m", (Function) msg_status,  NULL},
-  {"voice",   "",    (Function) msg_voice,   NULL},
-  {"who",     "",    (Function) msg_who,     NULL},
-  {"whois",   "",    (Function) msg_whois,   NULL},
+  {"addhost", "",    (IntFunc) msg_addhost, NULL},
+  {"die",     "n",   (IntFunc) msg_die,     NULL},
+  {"go",      "",    (IntFunc) msg_go,      NULL},
+  {"hello",   "",    (IntFunc) msg_hello,   NULL},
+  {"help",    "",    (IntFunc) msg_help,    NULL},
+  {"ident",   "",    (IntFunc) msg_ident,   NULL},
+  {"info",    "",    (IntFunc) msg_info,    NULL},
+  {"invite",  "o|o", (IntFunc) msg_invite,  NULL},
+  {"jump",    "m",   (IntFunc) msg_jump,    NULL},
+  {"key",     "o|o", (IntFunc) msg_key,     NULL},
+  {"memory",  "m",   (IntFunc) msg_memory,  NULL},
+  {"op",      "",    (IntFunc) msg_op,      NULL},
+  {"halfop",  "",    (IntFunc) msg_halfop,  NULL},
+  {"pass",    "",    (IntFunc) msg_pass,    NULL},
+  {"rehash",  "m",   (IntFunc) msg_rehash,  NULL},
+  {"reset",   "m",   (IntFunc) msg_reset,   NULL},
+  {"save",    "m",   (IntFunc) msg_save,    NULL},
+  {"status",  "m|m", (IntFunc) msg_status,  NULL},
+  {"voice",   "",    (IntFunc) msg_voice,   NULL},
+  {"who",     "",    (IntFunc) msg_who,     NULL},
+  {"whois",   "",    (IntFunc) msg_whois,   NULL},
   {NULL,      NULL,  NULL,                   NULL}
 };
