@@ -1,11 +1,11 @@
 /*
  * module.h
  *
- * $Id: module.h,v 1.85 2008-02-16 21:41:06 guppy Exp $
+ * $Id: module.h,v 1.95 2010/07/08 19:23:06 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -52,6 +52,7 @@
 #undef get_data_ptr
 #undef wild_match
 #undef wild_match_per
+#undef wild_match_partial_case
 #undef maskhost
 #undef maskban
 #undef user_realloc
@@ -97,7 +98,7 @@
 #ifdef DEBUG_CONTEXT
 #  define Context (global[2](__FILE__, __LINE__, MODULE_NAME))
 #else
-#  define Context {}
+#  define Context do {} while (0)
 #endif
 #define module_rename ((int (*)(char *, char *))global[3])
 /* 4 - 7 */
@@ -106,7 +107,7 @@
 #define module_depend ((Function *(*)(char *,char *,int,int))global[6])
 #define module_undepend ((int(*)(char *))global[7])
 /* 8 - 11 */
-#define add_bind_table ((p_tcl_bind_list(*)(const char *,int,Function))global[8])
+#define add_bind_table ((p_tcl_bind_list(*)(const char *,int,IntFunc))global[8])
 #define del_bind_table ((void (*) (p_tcl_bind_list))global[9])
 #define find_bind_table ((p_tcl_bind_list(*)(const char *))global[10])
 #define check_tcl_bind ((int (*) (p_tcl_bind_list,const char *,struct flag_record *,const char *, int))global[11])
@@ -186,10 +187,10 @@
 #define chatout (global[70])
 #define chanout_but ((void(*)())global[71])
 /* 72 - 75 */
-#define check_validity ((int (*) (char *,Function))global[72])
-#define list_delete ((int (*)( struct list_type **, struct list_type *))global[73])
-#define list_append ((int (*) ( struct list_type **, struct list_type *))global[74])
-#define list_contains ((int (*) (struct list_type *, struct list_type *))global[75])
+#define check_validity ((int (*) (char *,IntFunc))global[72])
+#define egg_list_delete ((int (*)( struct list_type **, struct list_type *))global[73])
+#define egg_list_append ((int (*) ( struct list_type **, struct list_type *))global[74])
+#define egg_list_contains ((int (*) (struct list_type *, struct list_type *))global[75])
 /* 76 - 79 */
 #define answer ((int (*) (int,char *,unsigned long *,unsigned short *,int))global[76])
 #define getmyip ((IP (*) (void))global[77])
@@ -288,7 +289,7 @@
 /* 152 - 155 */
 #define makepass ((void (*) (char *))global[152])
 #define wild_match ((int (*)(const char *, const char *))global[153])
-#define _maskhost ((void (*)(const char *, char *, int))global[154])
+#define maskaddr ((void (*)(const char *, char *, int))global[154])
 #define show_motd ((void(*)(int))global[155])
 /* 156 - 159 */
 #define tellhelp ((void(*)(int, char *, struct flag_record *, int))global[156])
@@ -389,15 +390,15 @@
 #ifdef DEBUG_CONTEXT
 #  define ContextNote(note) (global[232](__FILE__, __LINE__, MODULE_NAME, note))
 #else
-#  define ContextNote(note)     do {    } while (0)
+#  define ContextNote(note) do {} while (0)
 #endif
 #ifdef DEBUG_ASSERT
-#  define Assert(expr)          do {                                    \
-        if (!(expr))                                                    \
-                (global[233](__FILE__, __LINE__, MODULE_NAME));         \
+#  define Assert(expr) do {                                             \
+          if (!(expr))                                                  \
+            (global[233](__FILE__, __LINE__, MODULE_NAME));             \
 } while (0)
 #else
-#  define Assert(expr)  do {    } while (0)
+#  define Assert(expr) do {} while (0)
 #endif
 #define allocsock ((int(*)(int sock,int options))global[234])
 #define call_hostbyip ((void(*)(IP, char *, int))global[235])
@@ -477,11 +478,21 @@
 #define oatoi ((int (*) (const char *))global[291])
 /* 292 - 295 */
 #define str_isdigit ((int (*) (const char *))global[292])
-#define remove_crlf ((void (*)(char **))global[293])
+#define remove_crlf ((void (*) (char **))global[293])
+#define addr_match ((int (*)(const char *, const char *, int, int))global[294])
+#define mask_match ((int (*)(const char *, const char *))global[295])
+/* 296 - 299 */
+#define check_conflags ((int (*) (struct flag_record *, int))global[296])
+#define increase_socks_max ((int (*)(void))global[297])
+#define LOG_TS ((char *)(global[298]))
 
 /* hostmasking */
-#define maskhost(a,b) _maskhost((a),(b),1)
-#define maskban(a,b) _maskhost((a),(b),0)
+#define maskhost(a,b) maskaddr((a),(b),3)
+#define maskban(a,b) maskaddr((a),(b),3)
+
+/* host matching */
+#define match_addr(a,b) addr_match((a),(b),0,0)
+#define cmp_masks(a,b) addr_match((a),(b),0,1)
 
 /* This is for blowfish module, couldnt be bothered making a whole new .h
  * file for it ;)
