@@ -5,11 +5,11 @@
  *   note cmds
  *   note ignores
  *
- * $Id: notes.c,v 1.57 2008-02-16 21:41:09 guppy Exp $
+ * $Id: notes.c,v 1.62 2010/02/18 09:52:29 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -313,7 +313,8 @@ static int tcl_storenote STDVAR
           else if (argv[1][0] == '@')
             from = p + 1;
         }
-        fprintf(f, "%s %s %li %s%s\n", to, from, now, l ? work : "", blah);
+        fprintf(f, "%s %s %li %s%s\n", to, from, (long) now,
+                l ? work : "", blah);
         fclose(f);
         if (idx >= 0)
           dprintf(idx, "%s.\n", NOTES_STORED_MESSAGE);
@@ -813,7 +814,7 @@ static int msg_notes(char *nick, char *host, struct userrec *u, char *par)
       return 1;
     }
     chmod(notefile, userfile_perm); /* Use userfile permissions. */
-    fprintf(f, "%s %s %li %s\n", to, u->handle, now, par);
+    fprintf(f, "%s %s %li %s\n", to, u->handle, (long) now, par);
     fclose(f);
     dprintf(DP_HELP, "NOTICE %s :%s\n", nick, NOTES_DELIVERED);
     return 1;
@@ -861,7 +862,7 @@ static void notes_hourly()
       k = num_notes(dcc[l].nick);
       if ((k > 0) && (dcc[l].type->flags & DCT_CHAT)) {
         dprintf(l, NOTES_WAITING2, k, (k == 1) ? "" : "s");
-        dprintf(l, "### %s", (k != 1) ? NOTES_DCC_USAGE_READ : NOTES_DCC_USAGE_READ2);
+        dprintf(l, "### %s\n", (k != 1) ? NOTES_DCC_USAGE_READ : NOTES_DCC_USAGE_READ2);
       }
     }
   }
@@ -897,7 +898,7 @@ static void join_notes(char *nick, char *uhost, char *handle, char *par)
           (!egg_strcasecmp(dcc[j].nick, handle)))
         return;                 /* They already know they have notes */
 
-    while (!chan) {
+    while (chan) {
       if (ismember(chan, nick))
         return;                 /* They already know they have notes */
       chan = chan->next;
@@ -1085,27 +1086,27 @@ int match_note_ignore(struct userrec *u, char *from)
 
 
 static cmd_t notes_join[] = {
-  {"*",  "",   (Function) join_notes, "notes"},
+  {"*",  "",   (IntFunc) join_notes, "notes"},
   {NULL, NULL, NULL,                     NULL}
 };
 
 static cmd_t notes_nkch[] = {
-  {"*",  "",   (Function) notes_change, "notes"},
+  {"*",  "",   (IntFunc) notes_change, "notes"},
   {NULL, NULL, NULL,                       NULL}
 };
 
 static cmd_t notes_away[] = {
-  {"*",  "",   (Function) away_notes, "notes"},
+  {"*",  "",   (IntFunc) away_notes, "notes"},
   {NULL, NULL, NULL,                     NULL}
 };
 
 static cmd_t notes_chon[] = {
-  {"*",  "",   (Function) chon_notes, "notes"},
+  {"*",  "",   (IntFunc) chon_notes, "notes"},
   {NULL, NULL, NULL,                     NULL}
 };
 
 static cmd_t notes_msgs[] = {
-  {"notes", "",   (Function) msg_notes, NULL},
+  {"notes", "",   (IntFunc) msg_notes, NULL},
   {NULL,    NULL, NULL,                 NULL}
 };
 
