@@ -2,11 +2,11 @@
  * cmdsserv.c -- part of server.mod
  *   handles commands from a user via dcc that cause server interaction
  *
- * $Id: cmdsserv.c,v 1.24 2008-02-16 21:41:10 guppy Exp $
+ * $Id: cmdsserv.c,v 1.26 2010/02/18 09:52:29 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -132,9 +132,9 @@ static void cmd_clearqueue(struct userrec *u, int idx, char *par)
 }
 
 static cmd_t C_dcc_serv[] = {
-  {"dump",       "m",  (Function) cmd_dump,       NULL},
-  {"jump",       "m",  (Function) cmd_jump,       NULL},
-  {"servers",    "o",  (Function) cmd_servers,    NULL},
-  {"clearqueue", "m",  (Function) cmd_clearqueue, NULL},
+  {"dump",       "m",  (IntFunc) cmd_dump,       NULL},
+  {"jump",       "m",  (IntFunc) cmd_jump,       NULL},
+  {"servers",    "o",  (IntFunc) cmd_servers,    NULL},
+  {"clearqueue", "m",  (IntFunc) cmd_clearqueue, NULL},
   {NULL,         NULL, NULL,                      NULL}
 };
