@@ -2,11 +2,11 @@
  * server.c -- part of server.mod
  *   basic irc server support
  *
- * $Id: server.c,v 1.128 2008-02-16 21:41:10 guppy Exp $
+ * $Id: server.c,v 1.138 2010/07/12 16:18:09 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -57,7 +57,7 @@ static int nick_juped = 0;      /* True if origbotname is juped(RPL437) (dw) */
 static int check_stoned;        /* Check for a stoned server? */
 static int serverror_quit;      /* Disconnect from server if ERROR
                                  * messages received? */
-static int waiting_for_awake;   /* set when i unidle myself, cleared when
+static time_t lastpingcheck;    /* set when i unidle myself, cleared when
                                  * i get the response */
 static time_t server_online;    /* server connection time */
 static time_t server_cycle_wait;        /* seconds to wait before
@@ -95,10 +95,11 @@ static int nick_len;            /* Maximal nick length allowed on the
                                  * network. */
 static int kick_method;
 static int optimize_kicks;
-
+static int msgrate;             /* Number of seconds between sending
+                                 * queued lines to server. */
 
 static p_tcl_bind_list H_wall, H_raw, H_notc, H_msgm, H_msg, H_flud, H_ctcr,
-                       H_ctcp;
+                       H_ctcp, H_out;
 
 static void empty_msgq(void);
 static void next_server(int *, char *, unsigned int *, char *);
@@ -119,12 +120,6 @@ static char *realservername;
 
 #define MAXPENALTY 10
 
-/* Number of seconds to wait between transmitting queued lines to the server
- * lower this value at your own risk.  ircd is known to start flood control
- * at 512 bytes/2 seconds.
- */
-#define msgrate 2
-
 /* Maximum messages to store in each queue. */
 static int maxqmsg;
 static struct msgq_head mq, hq, modeq;
@@ -177,6 +172,7 @@ static void deq_msg()
         burst++;
         continue;
       }
+      check_tcl_out(DP_MODE, modeq.head->msg, 1);
       write_to_server(modeq.head->msg, modeq.head->len);
       if (raw_log)
         putlog(LOG_SRVOUT, "*", "[m->] %s", modeq.head->msg);
@@ -206,6 +202,7 @@ static void deq_msg()
     if (fast_deq(DP_SERVER))
       return;
 
+    check_tcl_out(DP_SERVER, mq.head->msg, 1);
     write_to_server(mq.head->msg, mq.head->len);
     if (raw_log)
       putlog(LOG_SRVOUT, "*", "[s->] %s", mq.head->msg);
@@ -232,6 +229,7 @@ static void deq_msg()
   if (fast_deq(DP_HELP))
     return;
 
+  check_tcl_out(DP_HELP, hq.head->msg, 1);
   write_to_server(hq.head->msg, hq.head->len);
   if (raw_log)
     putlog(LOG_SRVOUT, "*", "[h->] %s", hq.head->msg);
@@ -482,6 +480,7 @@ static int fast_deq(int which)
   if (doit) {
     simple_sprintf(tosend, "%s %s %s", cmd, victims, msg);
     len = strlen(tosend);
+    check_tcl_out(which, tosend, 1);
     write_to_server(tosend, len);
     if (raw_log) {
       switch (which) {
@@ -745,6 +744,7 @@ static int deq_kick(int which)
   }
   egg_snprintf(newmsg, sizeof newmsg, "KICK %s %s %s", chan, newnicks + 1,
                reason);
+  check_tcl_out(which, newmsg, 1);
   write_to_server(newmsg, strlen(newmsg));
   if (raw_log) {
     switch (which) {
@@ -783,11 +783,12 @@ static void empty_msgq()
 
 /* Queues outgoing messages so there's no flooding.
  */
-static void queue_server(int which, char *buf, int len)
+static void queue_server(int which, char *msg, int len)
 {
   struct msgq_head *h = NULL, tempq;
   struct msgq *q, *tq, *tqq;
   int doublemsg = 0, qnext = 0;
+  char buf[511];
 
   /* Don't even BOTHER if there's no server online. */
   if (serv < 0)
@@ -796,7 +797,9 @@ static void queue_server(int which, char *buf, int len)
   /* Remove \r\n. We will add these back when we send the text to the server.
    * - Wcc [01/09/2004]
    */
-  remove_crlf(&buf);
+  strncpy(buf, msg, sizeof buf);
+  msg = buf;
+  remove_crlf(&msg);
   buf[510] = 0;
   len = strlen(buf);
 
@@ -804,6 +807,7 @@ static void queue_server(int which, char *buf, int len)
   if (!egg_strncasecmp(buf, "PING", 4) || !egg_strncasecmp(buf, "PONG", 4)) {
     if (buf[1] == 'I' || buf[1] == 'i')
       lastpingtime = now;
+    check_tcl_out(which, buf, 1);
     write_to_server(buf, len);
     if (raw_log)
       putlog(LOG_SRVOUT, "*", "[m->] %s", buf);
@@ -864,6 +868,9 @@ static void queue_server(int which, char *buf, int len)
       }
     }
 
+    if (check_tcl_out(which, buf, 0))
+      return; /* a Tcl proc requested not to send the message */
+
     q = nmalloc(sizeof(struct msgq));
 
     /* Insert into queue. */
@@ -943,7 +950,7 @@ static void queue_server(int which, char *buf, int len)
 
 /* Add a new server to the server_list.
  */
-static void add_server(char *ss)
+static void add_server(const char *ss)
 {
   struct server_list *x, *z;
   char *p, *q;
@@ -1017,8 +1024,6 @@ static void next_server(int *ptr, char *serv, unsigned int *port, char *pass)
   struct server_list *x = serverlist;
   int i = 0;
 
-  if (x == NULL)
-    return;
   /* -1  -->  Go to specified server */
   if (*ptr == -1) {
     for (; x; x = x->next) {
@@ -1047,11 +1052,14 @@ static void next_server(int *ptr, char *serv, unsigned int *port, char *pass)
       strcpy(x->pass, pass);
     } else
       x->pass = NULL;
-    list_append((struct list_type **) (&serverlist), (struct list_type *) x);
+    egg_list_append((struct list_type **) (&serverlist),
+                    (struct list_type *) x);
     *ptr = i;
     return;
   }
   /* Find where i am and boogie */
+  if (x == NULL)
+    return;
   i = (*ptr);
   while (i > 0 && x != NULL) {
     x = x->next;
@@ -1131,6 +1139,17 @@ static int server_raw STDVAR
   return TCL_OK;
 }
 
+static int server_out STDVAR
+{
+  Function F = (Function) cd;
+
+  BADARGS(4, 4, " queue message sent");
+
+  CHECKVALIDITY(server_out);
+  F(argv[1], argv[2], argv[3]);
+  return TCL_OK;
+}
+
 /* Read/write normal string variable.
  */
 static char *nick_change(ClientData cdata, Tcl_Interp *irp,
@@ -1365,6 +1384,7 @@ static tcl_ints my_tcl_ints[] = {
   {"isjuped",           &nick_juped,                0},
   {"stack-limit",       &stack_limit,               0},
   {"exclusive-binds",   &exclusive_binds,           0},
+  {"msg-rate",          &msgrate,                   0},
   {NULL,                NULL,                       0}
 };
 
@@ -1407,7 +1427,7 @@ static char *tcl_eggserver(ClientData cdata, Tcl_Interp *irp,
     if (slist != NULL) {
       code = Tcl_SplitList(interp, slist, &lc, &list);
       if (code == TCL_ERROR)
-        return interp->result;
+        return "variable must be a list";
       for (i = 0; i < lc && i < 50; i++)
         add_server((char *) list[i]);
 
@@ -1418,7 +1438,8 @@ static char *tcl_eggserver(ClientData cdata, Tcl_Interp *irp,
         int servidx = findanyidx(serv);
 
         curserv = -1;
-        next_server(&curserv, dcc[servidx].host, &dcc[servidx].port, "");
+        if (serverlist)
+          next_server(&curserv, dcc[servidx].host, &dcc[servidx].port, "");
       }
       Tcl_Free((char *) list);
     }
@@ -1460,7 +1481,7 @@ static int ctcp_DCC_CHAT(char *nick, char *from, char *handle,
   if (egg_strcasecmp(action, "CHAT") || egg_strcasecmp(object, botname) || !u)
     return 0;
   get_user_flagrec(u, &fr, 0);
-  if (dcc_total == max_dcc) {
+  if (dcc_total == max_dcc && increase_socks_max()) {
     if (!quiet_reject)
       dprintf(DP_HELP, "NOTICE %s :%s\n", nick, DCC_TOOMANYDCCS1);
     putlog(LOG_MISC, "*", DCC_TOOMANYDCCS2, "CHAT", param, nick, from);
@@ -1561,8 +1582,8 @@ static void server_secondly()
 static void server_5minutely()
 {
   if (check_stoned && server_online) {
-    if (waiting_for_awake) {
-      /* Uh oh!  Never got pong from last time, five minutes ago!
+    if (lastpingcheck && (now - lastpingcheck >= 240)) {
+      /* Still waiting for activity, requested longer than 4 minutes ago.
        * Server is probably stoned.
        */
       int servidx = findanyidx(serv);
@@ -1573,7 +1594,7 @@ static void server_5minutely()
     } else if (!trying_server) {
       /* Check for server being stoned. */
       dprintf(DP_MODE, "PING :%li\n", now);
-      waiting_for_awake = 1;
+      lastpingcheck = now;
     }
   }
 }
@@ -1672,7 +1693,7 @@ static void server_report(int idx, int details)
               keepnick ? " (trying)" : "");
     daysdur(now, server_online, s1);
     egg_snprintf(s, sizeof s, "(connected %s)", s1);
-    if (server_lag && !waiting_for_awake) {
+    if (server_lag && !lastpingcheck) {
       if (server_lag == -1)
         egg_snprintf(s1, sizeof s1, " (bad pong replies)");
       else
@@ -1742,6 +1763,7 @@ static char *server_close()
   del_bind_table(H_flud);
   del_bind_table(H_ctcr);
   del_bind_table(H_ctcp);
+  del_bind_table(H_out);
   rem_tcl_coups(my_tcl_coups);
   rem_tcl_strings(my_tcl_strings);
   rem_tcl_ints(my_tcl_ints);
@@ -1831,7 +1853,9 @@ static Function server_table[] = {
   (Function) get_altbotnick,    /* char *                               */
   (Function) & nick_len,        /* int                                  */
   (Function) check_tcl_notc,
-  (Function) & exclusive_binds  /* int                                  */
+  (Function) & exclusive_binds, /* int                                  */
+  /* 40 - 43 */
+  (Function) & H_out            /* p_tcl_bind_list                      */
 };
 
 char *server_start(Function *global_funcs)
@@ -1861,7 +1885,7 @@ char *server_start(Function *global_funcs)
   keepnick = 1;
   check_stoned = 1;
   serverror_quit = 1;
-  waiting_for_awake = 0;
+  lastpingcheck = 0;
   server_online = 0;
   server_cycle_wait = 60;
   strcpy(botrealname, "A deranged product of evil coders");
@@ -1895,9 +1919,10 @@ char *server_start(Function *global_funcs)
   optimize_kicks = 0;
   stack_limit = 4;
   realservername = 0;
+  msgrate = 2;
 
   server_table[4] = (Function) botname;
-  module_register(MODULE_NAME, server_table, 1, 2);
+  module_register(MODULE_NAME, server_table, 1, 3);
   if (!module_depend(MODULE_NAME, "eggdrop", 106, 7)) {
     module_undepend(MODULE_NAME);
     return "This module requires Eggdrop 1.6.7 or later.";
@@ -1938,6 +1963,7 @@ char *server_start(Function *global_funcs)
   H_flud = add_bind_table("flud", HT_STACKABLE, server_5char);
   H_ctcr = add_bind_table("ctcr", HT_STACKABLE, server_6char);
   H_ctcp = add_bind_table("ctcp", HT_STACKABLE, server_6char);
+  H_out = add_bind_table("out", HT_STACKABLE, server_out);
   add_builtins(H_raw, my_raw_binds);
   add_builtins(H_dcc, C_dcc_serv);
   add_builtins(H_ctcp, my_ctcps);
