@@ -1,11 +1,11 @@
 /*
  * server.h -- part of server.mod
  *
- * $Id: server.h,v 1.26 2008-02-16 21:41:10 guppy Exp $
+ * $Id: server.h,v 1.28 2010/01/03 13:27:55 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -74,6 +74,8 @@
 #define nick_len (*(int *)(server_funcs[37]))
 #define check_tcl_notc ((int (*)(char *,char *,struct userrec *,char *,char *))server_funcs[38])
 #define exclusive_binds (*(int *)(server_funcs[39]))
+/* 40 - 43 */
+#define H_out (*(p_tcl_bind_list *)(server_funcs[40]))
 #else /* MAKING_SERVER */
 
 /* Macros for commonly used commands. */
