@@ -1,11 +1,11 @@
 /*
  * servmsg.c -- part of server.mod
  *
- * $Id: servmsg.c,v 1.95 2008-02-16 21:41:10 guppy Exp $
+ * $Id: servmsg.c,v 1.106 2010/07/12 16:18:09 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -245,6 +245,37 @@ static int check_tcl_flud(char *nick, char *uhost, struct userrec *u,
   return (x == BIND_EXEC_LOG);
 }
 
+static int check_tcl_out(int which, char *msg, int sent)
+{
+  int x;
+  char args[32], *queue;
+
+  switch (which) {
+  case DP_MODE:
+  case DP_MODE_NEXT:
+    queue = "mode";
+    break;
+  case DP_SERVER:
+  case DP_SERVER_NEXT:
+    queue = "server";
+    break;
+  case DP_HELP:
+  case DP_HELP_NEXT:
+    queue = "help";
+    break;
+  default:
+    queue = "noqueue";
+  }
+  snprintf(args, sizeof args, "%s %s", queue, sent ? "sent" : "queued");
+  Tcl_SetVar(interp, "_out1", queue, 0);
+  Tcl_SetVar(interp, "_out2", msg, 0);
+  Tcl_SetVar(interp, "_out3", sent ? "sent" : "queued", 0);
+  x = check_tcl_bind(H_out, args, 0, " $_out1 $_out2 $_out3",
+                     MATCH_MASK | BIND_STACKABLE | BIND_WANTRET);
+
+  return (x == BIND_EXEC_LOG);
+}
+
 static int match_my_nick(char *nick)
 {
   if (!rfc_casecmp(nick, botname))
@@ -255,9 +286,10 @@ static int match_my_nick(char *nick)
 /* 001: welcome to IRC (use it to fix the server name) */
 static int got001(char *from, char *msg)
 {
-  struct server_list *x = serverlist;
   int i;
+  char *key;
   struct chanset_t *chan;
+  struct server_list *x = serverlist;
 
   /* FIXME - x should never be NULL anywhere in this function, but
    * apparently it sometimes is. */
@@ -295,9 +327,14 @@ static int got001(char *from, char *msg)
     for (chan = chanset; chan; chan = chan->next) {
       chan->status &= ~(CHAN_ACTIVE | CHAN_PEND);
       if (!channel_inactive(chan)) {
-        dprintf(DP_SERVER, "JOIN %s %s\n", (chan->name[0]) ? chan->name :
-                chan->dname, chan->channel.key[0] ? chan->channel.key :
-                chan->key_prot);
+
+        key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
+        if (key[0])
+          dprintf(DP_SERVER, "JOIN %s %s\n",
+                  chan->name[0] ? chan->name : chan->dname, key);
+        else
+          dprintf(DP_SERVER, "JOIN %s\n",
+                  chan->name[0] ? chan->name : chan->dname);
       }
     }
   }
@@ -309,7 +346,7 @@ static int got001(char *from, char *msg)
  */
 static int got442(char *from, char *msg)
 {
-  char *chname;
+  char *chname, *key;
   struct chanset_t *chan;
 
   if (!realservername || egg_strcasecmp(from, realservername))
@@ -317,18 +354,20 @@ static int got442(char *from, char *msg)
   newsplit(&msg);
   chname = newsplit(&msg);
   chan = findchan(chname);
-  if (chan)
-    if (!channel_inactive(chan)) {
-      module_entry *me = module_find("channels", 0, 0);
+  if (chan && !channel_inactive(chan)) {
+    module_entry *me = module_find("channels", 0, 0);
 
-      putlog(LOG_MISC, chname, IRC_SERVNOTONCHAN, chname);
-      if (me && me->funcs)
-        (me->funcs[CHANNEL_CLEAR]) (chan, 1);
-      chan->status &= ~CHAN_ACTIVE;
-      dprintf(DP_MODE, "JOIN %s %s\n", chan->name,
-              chan->channel.key[0] ? chan->channel.key : chan->key_prot);
-    }
+    putlog(LOG_MISC, chname, IRC_SERVNOTONCHAN, chname);
+    if (me && me->funcs)
+      (me->funcs[CHANNEL_CLEAR]) (chan, 1);
+    chan->status &= ~CHAN_ACTIVE;
 
+    key = chan->channel.key[0] ? chan->channel.key : chan->key_prot;
+    if (key[0])
+      dprintf(DP_SERVER, "JOIN %s %s\n", chan->name, key);
+    else
+      dprintf(DP_SERVER, "JOIN %s\n", chan->name);
+  }
   return 0;
 }
 
@@ -488,9 +527,9 @@ static int gotmsg(char *from, char *msg)
       *p = 0;
       strncpyz(ctcpbuf, p1, sizeof(ctcpbuf));
       ctcp = ctcpbuf;
-      /* copy the part after the second : in front of it after
-       * the first :, this is temporary copied to ctcpbuf */
-      strncpy(p1 - 1, p + 1, strlen(ctcpbuf) - 1);
+
+      /* remove the ctcp in msg */
+      memmove(p1 - 1, p + 1, strlen(p + 1) + 1);
 
       if (!ignoring)
         detect_flood(nick, uhost, from,
@@ -1080,7 +1119,7 @@ static void server_activity(int idx, char *msg, int len)
     trying_server = 0;
     SERVER_SOCKET.timeout_val = 0;
   }
-  waiting_for_awake = 0;
+  lastpingcheck = 0;
   from = "";
   if (msg[0] == ':') {
     msg++;
@@ -1088,8 +1127,12 @@ static void server_activity(int idx, char *msg, int len)
   }
   code = newsplit(&msg);
   if (raw_log && ((strcmp(code, "PRIVMSG") && strcmp(code, "NOTICE")) ||
-      !match_ignore(from)))
-    putlog(LOG_RAW, "*", "[@] %s %s %s", from, code, msg);
+      !match_ignore(from))) {
+    if (!strcmp(from, ""))
+      putlog(LOG_RAW, "*", "[@] %s %s", code, msg);
+    else
+      putlog(LOG_RAW, "*", "[@] %s %s %s", from, code, msg);
+  }
   /* This has GOT to go into the raw binding table, * merely because this
    * is less effecient.
    */
@@ -1123,13 +1166,15 @@ static int gotkick(char *from, char *msg)
  */
 static int whoispenalty(char *from, char *msg)
 {
-  if (realservername && use_penalties) {
-    if (strcmp(realservername, from)) {
-      last_time += 1;
-      if (raw_log)
-        putlog(LOG_SRVOUT, "*", "adding 1sec penalty (remote whois)");
-    }
+  if (realservername && use_penalties &&
+      egg_strcasecmp(from, realservername)) {
+
+    last_time += 1;
+
+    if (raw_log)
+      putlog(LOG_SRVOUT, "*", "adding 1sec penalty (remote whois)");
   }
+
   return 0;
 }
 
@@ -1151,29 +1196,48 @@ static int got311(char *from, char *msg)
   return 0;
 }
 
+
+/*
+ * 465     ERR_YOUREBANNEDCREEP :You are banned from this server
+ */
+static int got465(char *from, char *msg)
+{
+  newsplit(&msg); /* 465 */
+  newsplit(&msg); /* nick */
+
+  fixcolon(msg);
+
+  putlog(LOG_SERV, "*", "Server (%s) says I'm banned: %s", from, msg);
+  putlog(LOG_SERV, "*", "Disconnecting from server.");
+  nuke_server("Banned from server.");
+  return 1;
+}
+
+
 static cmd_t my_raw_binds[] = {
-  {"PRIVMSG", "",   (Function) gotmsg,       NULL},
-  {"NOTICE",  "",   (Function) gotnotice,    NULL},
-  {"MODE",    "",   (Function) gotmode,      NULL},
-  {"PING",    "",   (Function) gotping,      NULL},
-  {"PONG",    "",   (Function) gotpong,      NULL},
-  {"WALLOPS", "",   (Function) gotwall,      NULL},
-  {"001",     "",   (Function) got001,       NULL},
-  {"251",     "",   (Function) got251,       NULL},
-  {"303",     "",   (Function) got303,       NULL},
-  {"432",     "",   (Function) got432,       NULL},
-  {"433",     "",   (Function) got433,       NULL},
-  {"437",     "",   (Function) got437,       NULL},
-  {"438",     "",   (Function) got438,       NULL},
-  {"451",     "",   (Function) got451,       NULL},
-  {"442",     "",   (Function) got442,       NULL},
-  {"NICK",    "",   (Function) gotnick,      NULL},
-  {"ERROR",   "",   (Function) goterror,     NULL},
+  {"PRIVMSG", "",   (IntFunc) gotmsg,       NULL},
+  {"NOTICE",  "",   (IntFunc) gotnotice,    NULL},
+  {"MODE",    "",   (IntFunc) gotmode,      NULL},
+  {"PING",    "",   (IntFunc) gotping,      NULL},
+  {"PONG",    "",   (IntFunc) gotpong,      NULL},
+  {"WALLOPS", "",   (IntFunc) gotwall,      NULL},
+  {"001",     "",   (IntFunc) got001,       NULL},
+  {"251",     "",   (IntFunc) got251,       NULL},
+  {"303",     "",   (IntFunc) got303,       NULL},
+  {"432",     "",   (IntFunc) got432,       NULL},
+  {"433",     "",   (IntFunc) got433,       NULL},
+  {"437",     "",   (IntFunc) got437,       NULL},
+  {"438",     "",   (IntFunc) got438,       NULL},
+  {"451",     "",   (IntFunc) got451,       NULL},
+  {"442",     "",   (IntFunc) got442,       NULL},
+  {"465",     "",   (IntFunc) got465,       NULL},
+  {"NICK",    "",   (IntFunc) gotnick,      NULL},
+  {"ERROR",   "",   (IntFunc) goterror,     NULL},
 /* ircu2.10.10 has a bug when a client is throttled ERROR is sent wrong */
-  {"ERROR:",  "",   (Function) goterror,     NULL},
-  {"KICK",    "",   (Function) gotkick,      NULL},
-  {"318",     "",   (Function) whoispenalty, NULL},
-  {"311",     "",   (Function) got311,       NULL},
+  {"ERROR:",  "",   (IntFunc) goterror,     NULL},
+  {"KICK",    "",   (IntFunc) gotkick,      NULL},
+  {"318",     "",   (IntFunc) whoispenalty, NULL},
+  {"311",     "",   (IntFunc) got311,       NULL},
   {NULL,      NULL, NULL,                    NULL}
 };
 
@@ -1189,7 +1253,7 @@ static void connect_server(void)
   int servidx;
   unsigned int botserverport = 0;
 
-  waiting_for_awake = 0;
+  lastpingcheck = 0;
   trying_server = now;
   empty_msgq();
   /* Start up the counter (always reset it if "never-give-up" is on) */
@@ -1204,13 +1268,16 @@ static void connect_server(void)
     newserverport = 0;
     newserverpass[0] = 0;
     oldserv = -1;
-  } else
+  } else {
+    if (curserv == -1)
+      curserv = 999;
     pass[0] = 0;
+  }
   if (!cycle_time) {
     struct chanset_t *chan;
     struct server_list *x = serverlist;
 
-    if (!x) {
+    if (!x && !botserverport) {
       putlog(LOG_SERV, "*", "No servers in server list");
       cycle_time = 300;
       return;
