@@ -1,11 +1,11 @@
 /*
  * tclserv.c -- part of server.mod
  *
- * $Id: tclserv.c,v 1.21 2008-02-16 21:41:10 guppy Exp $
+ * $Id: tclserv.c,v 1.25 2010/01/03 13:27:55 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,6 +33,57 @@ static int tcl_isbotnick STDVAR
   return TCL_OK;
 }
 
+static int tcl_putnow STDVAR
+{
+  int len;
+  char buf[512], *p, *q, *r;
+
+  BADARGS(2, 3, " text ?options?");
+
+  if ((argc == 3) && egg_strcasecmp(argv[2], "-oneline")) {
+    Tcl_AppendResult(irp, "unknown putnow option: should be ",
+                     "-oneline", NULL);
+    return TCL_ERROR;
+  }
+  if (!serv) /* no server - no output */
+    return TCL_OK;
+
+  for (p = r = argv[1], q = buf; ; p++) {
+    if (*p && *p != '\r' && *p != '\n')
+      continue; /* look for message delimiters */
+    if (p == r) { /* empty message */
+      if (*p) {
+        r++;
+        continue;
+      } else
+        break;
+    }
+    if ((p - r) > (sizeof(buf) - 2 - (q - buf)))
+      break; /* That's all folks, no space left */
+    len = p - r + 1; /* leave space for '\0' */
+    strncpyz(q, r, len);
+    if (check_tcl_out(0, q, 0)) {
+      if (!*p || ((argc == 3) && !egg_strcasecmp(argv[2], "-oneline")))
+        break;
+      r = p + 1;
+      continue;
+    }
+    check_tcl_out(0, q, 1);
+    if (q == buf)
+      putlog(LOG_SRVOUT, "*", "[r->] %s", q);
+    else
+      putlog(LOG_SRVOUT, "*", "[+r->] %s", q);
+    q += len - 1; /* the '\0' must be overwritten */
+    *q++ = '\r';
+    *q++ = '\n'; /* comply with the RFC */
+    if (!*p || ((argc == 3) && !egg_strcasecmp(argv[2], "-oneline")))
+      break; /* cut on newline requested or message ended */
+    r = p + 1;
+  }
+  tputs(serv, buf, q - buf); /* q points after the last '\n' */
+  return TCL_OK;
+}
+
 static int tcl_putquick STDVAR
 {
   char s[511], *p;
@@ -261,5 +312,6 @@ static tcl_cmds my_tcl_cmds[] = {
   {"puthelp",    tcl_puthelp},
   {"putserv",    tcl_putserv},
   {"putquick",   tcl_putquick},
+  {"putnow",     tcl_putnow},
   {NULL,         NULL}
 };
