@@ -4,11 +4,11 @@
  *
  * by Darrin Smith (beldin@light.iinet.net.au)
  *
- * $Id: modules.c,v 1.99 2008-02-16 21:41:04 guppy Exp $
+ * $Id: modules.c,v 1.108 2010/07/08 19:23:06 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -86,7 +86,7 @@ extern struct chanset_t *chanset;
 
 extern char tempdir[], botnetnick[], botname[], natip[], hostname[],
             origbotname[], botuser[], admin[], userfile[], ver[], notify_new[],
-            helpdir[], version[], quit_msg[];
+            helpdir[], version[], quit_msg[], log_ts[];
 
 extern int parties, noshare, dcc_total, egg_numver, userfile_perm, do_restart,
            ignore_time, must_be_owner, raw_log, max_dcc, make_userfile,
@@ -287,9 +287,9 @@ Function global_table[] = {
   (Function) chanout_but,
   /* 72 - 75 */
   (Function) check_validity,
-  (Function) list_delete,
-  (Function) list_append,
-  (Function) list_contains,
+  (Function) egg_list_delete,
+  (Function) egg_list_append,
+  (Function) egg_list_contains,
   /* 76 - 79 */
   (Function) answer,
   (Function) getmyip,
@@ -388,7 +388,7 @@ Function global_table[] = {
   /* 152 - 155 */
   (Function) makepass,
   (Function) _wild_match,
-  (Function) _maskhost,
+  (Function) maskaddr,
   (Function) show_motd,
   /* 156 - 159 */
   (Function) tellhelp,
@@ -570,7 +570,13 @@ Function global_table[] = {
   (Function) oatoi,
   /* 292 - 295 */
   (Function) str_isdigit,
-  (Function) remove_crlf
+  (Function) remove_crlf,
+  (Function) addr_match,
+  (Function) mask_match,
+  /* 296 - 299 */
+  (Function) check_conflags,
+  (Function) increase_socks_max,
+  (Function) log_ts
 };
 
 void init_modules(void)
@@ -974,7 +980,7 @@ void mod_free(void *ptr, const char *modname, const char *filename, int line)
   n_free(ptr, x, line);
 }
 
-/* Hooks, various tables of functions to call on ceratin events
+/* Hooks, various tables of functions to call on certain events
  */
 void add_hook(int hook_num, Function func)
 {
@@ -1030,8 +1036,8 @@ void add_hook(int hook_num, Function func)
       }
       break;
     case HOOK_MATCH_NOTEREJ:
-      if (match_noterej == (int (*)(struct userrec *, char *)) false_func)
-        match_noterej = func;
+      if (match_noterej == false_func)
+        match_noterej = (int (*)(struct userrec *, char *)) func;
       break;
     case HOOK_DNS_HOSTBYIP:
       if (dns_hostbyip == block_dns_hostbyip)
