@@ -4,11 +4,11 @@
  *
  * by Darrin Smith (beldin@light.iinet.net.au)
  *
- * $Id: modules.h,v 1.19 2008-02-16 21:41:04 guppy Exp $
+ * $Id: modules.h,v 1.21 2010/02/18 09:52:29 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -51,7 +51,7 @@ void *get_next_hook(int, void *);
 
 extern struct hook_entry {
   struct hook_entry *next;
-  int (*func) ();
+  Function func;
 } *hook_list[REAL_HOOKS];
 
 #define call_hook(x) do {                                       \
