@@ -7,11 +7,11 @@
  * because they use structures in those
  * (saves including those .h files EVERY time) - Beldin
  *
- * $Id: proto.h,v 1.72 2008-02-16 21:41:04 guppy Exp $
+ * $Id: proto.h,v 1.82 2010/07/12 15:40:52 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -128,6 +128,7 @@ int check_ansi(char *);
 void dupwait_notify(char *);
 
 /* dccutil.c */
+int increase_socks_max();
 int findidx(int);
 int findanyidx(int);
 char *add_cr(char *);
@@ -176,18 +177,35 @@ int exist_lang_section(char *);
 /* main.c */
 void fatal(const char *, int);
 int expected_memory(void);
-void patch(const char *);
 void eggContext(const char *, int, const char *);
 void eggContextNote(const char *, int, const char *, const char *);
 void eggAssert(const char *, int, const char *);
 void backup_userfile(void);
+int mainloop(int);
 
 /* match.c */
+int casecharcmp(unsigned char, unsigned char);
+int charcmp(unsigned char, unsigned char);
 int _wild_match(register unsigned char *, register unsigned char *);
-int _wild_match_per(register unsigned char *, register unsigned char *);
+int _wild_match_per(register unsigned char *, register unsigned char *,
+                    int (*)(unsigned char, unsigned char),
+                    int (*)(unsigned char, unsigned char),
+                    unsigned char *);
+int addr_match(char *, char *, int, int);
+int mask_match(char *, char *);
+int cidr_match(char *, char *, int);
+int cron_match(const char *, const char *);
 
 #define wild_match(a,b) _wild_match((unsigned char *)(a),(unsigned char *)(b))
-#define wild_match_per(a,b) _wild_match_per((unsigned char *)(a),(unsigned char *)(b))
+#define wild_match_per(a,b) _wild_match_per((unsigned char *)(a),              \
+                            (unsigned char *)(b),casecharcmp,NULL,NULL)
+#define wild_match_partial_case(a,b) _wild_match_per((unsigned char *)(a),     \
+                            (unsigned char *)(b),casecharcmp,charcmp,          \
+                            (unsigned char *)strchr((b),' '))
+#define match_addr(a,b) addr_match((char *)(a),(char *)(b),0,0)
+#define match_useraddr(a,b) addr_match((char *)(a),(char *)(b),1,0)
+#define cmp_masks(a,b) addr_match((char *)(a),(char *)(b),0,1)
+#define cmp_usermasks(a,b) addr_match((char *)(a),(char *)(b),1,1)
 
 /* mem.c */
 void *n_malloc(int, const char *, int);
@@ -209,7 +227,7 @@ void remove_crlf(char **);
 char *newsplit(char **);
 char *splitnick(char **);
 void stridx(char *, char *, int);
-void dumplots(int, const char *, char *);
+void dumplots(int, const char *, const char *);
 void daysago(time_t, time_t, char *);
 void days(time_t, time_t, char *);
 void daysdur(time_t, time_t, char *);
@@ -236,9 +254,9 @@ void str_unescape(char *, register const char);
 int str_isdigit(const char *);
 void kill_bot(char *, char *);
 
-void _maskhost(const char *, char *, int);
-#define maskhost(a,b) _maskhost((a),(b),1)
-#define maskban(a,b)  _maskhost((a),(b),0)
+void maskaddr(const char *, char *, int);
+#define maskhost(a,b) maskaddr((a),(b),3)
+#define maskban(a,b)  maskaddr((a),(b),3)
 
 /* net.c */
 IP my_atoul(char *);
@@ -247,8 +265,10 @@ IP getmyip();
 void neterror(char *);
 void setsock(int, int);
 int allocsock(int, int);
+int alloctclsock(int, int, Tcl_FileProc *, ClientData);
 int getsock(int);
 void killsock(int);
+void killtclsock(int);
 int answer(int, char *, unsigned long *, unsigned short *, int);
 inline int open_listen(int *);
 int open_address_listen(IP addr, int *);
@@ -257,6 +277,8 @@ int open_telnet_dcc(int, char *, char *);
 int open_telnet_raw(int, char *, int);
 void tputs(int, char *, unsigned int);
 void dequeue_sockets();
+int preparefdset(fd_set *, sock_list *, int, int, int);
+int sockread(char *, int *, sock_list *, int, int);
 int sockgets(char *, int *);
 void tell_netdebug(int);
 int sanitycheck_dcc(char *, char *, char *, char *);
@@ -267,6 +289,8 @@ int sockoptions(int sock, int operation, int sock_options);
 int flush_inbuf(int idx);
 
 /* tcl.c */
+struct threaddata *threaddata();
+int init_threaddata(int);
 void protect_tcl();
 void unprotect_tcl();
 void do_tcl(char *, char *);
@@ -297,6 +321,7 @@ struct userrec *check_dcclist_hand(char *);
 void touch_laston(struct userrec *, char *, time_t);
 void user_del_chan(char *);
 char *fixfrom(char *);
+int check_conflags(struct flag_record *fr, int md);
 
 /* users.c */
 void addignore(char *, char *, char *, time_t);
