@@ -4,11 +4,11 @@
  *   Tcl initialization
  *   getting and setting Tcl/eggdrop variables
  *
- * $Id: tcl.c,v 1.88 2008-02-16 21:41:04 guppy Exp $
+ * $Id: tcl.c,v 1.98 2010/07/12 15:40:52 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -48,8 +48,8 @@ extern time_t online_since;
 extern char origbotname[], botuser[], motdfile[], admin[], userfile[],
             firewall[], helpdir[], notify_new[], hostname[], myip[], moddir[],
             tempdir[], owner[], network[], botnetnick[], bannerfile[],
-            egg_version[], natip[], configfile[], logfile_suffix[], textdir[],
-            pid_file[];
+            egg_version[], natip[], configfile[], logfile_suffix[], log_ts[],
+            textdir[], pid_file[];
 
 extern int flood_telnet_thr, flood_telnet_time, shtime, share_greet,
            require_p, keep_all_logs, allow_new_telnets, stealth_telnets,
@@ -59,7 +59,7 @@ extern int flood_telnet_thr, flood_telnet_time, shtime, share_greet,
            die_on_sigterm, max_logs, max_logsize, dcc_total, raw_log,
            identtimeout, dcc_sanitycheck, dupwait_timeout, egg_numver,
            share_unlinks, protect_telnet, sort_users, strict_host,
-           resolve_timeout, default_uflags, userfile_perm;
+           resolve_timeout, default_uflags, userfile_perm, cidr_support;
 
 extern struct dcc_t *dcc;
 extern tcl_timer_t *timer, *utimer;
@@ -78,7 +78,7 @@ int allow_dk_cmds = 1;
 int must_be_owner = 1;
 int quiet_reject = 1;
 int copy_to_tmp = 1;
-int max_dcc = 20;
+int max_socks = 100;
 int quick_logs = 0;
 int par_telnet_flood = 1;
 int quiet_save = 0;
@@ -91,7 +91,6 @@ int clientdata_stuff = 0;
 int strict_servernames = 0, enable_simul = 1, use_console_r = 0,
     debug_output = 0;
 
-
 /* Prototypes for Tcl */
 Tcl_Interp *Tcl_CreateInterp();
 
@@ -122,7 +121,7 @@ static void botnet_change(char *new)
  *     Vars, traces, misc
  */
 
-int init_dcc_max(), init_misc();
+int init_misc();
 
 /* Used for read/write to integer couplets */
 typedef struct {
@@ -222,12 +221,11 @@ static char *tcl_eggint(ClientData cdata, Tcl_Interp *irp,
         return "read-only variable";
       else {
         if (Tcl_ExprLong(interp, s, &l) == TCL_ERROR)
-          return interp->result;
-        if ((int *) ii->var == &max_dcc) {
-          if (l < max_dcc)
-            return "you can't DECREASE max-dcc";
-          max_dcc = l;
-          init_dcc_max();
+          return "variable must have integer value";
+        if ((int *) ii->var == &max_socks) {
+          if (l < threaddata()->MAXSOCKS)
+            return "you can't DECREASE max-socks below current usage";
+          max_socks = l;
         } else if ((int *) ii->var == &max_logs) {
           if (l < max_logs)
             return "you can't DECREASE max-logs";
@@ -446,6 +444,37 @@ void add_tcl_objcommands(tcl_cmds *table)
 }
 #endif
 
+/* Get the current tcl result string. */
+const char *tcl_resultstring()
+{
+  const char *result;
+#ifdef USE_TCL_OBJ
+  result = Tcl_GetStringResult(interp);
+#else
+  result = interp->result;
+#endif
+  return result;
+}
+
+int tcl_resultempty() {
+  const char *result;
+  result = tcl_resultstring();
+  return (result && result[0]) ? 0 : 1;
+}
+
+/* Get the current tcl result as int. replaces atoi(interp->result) */
+int tcl_resultint()
+{
+  int result;
+#ifdef USE_TCL_OBJ
+  if (Tcl_GetIntFromObj(NULL, Tcl_GetObjResult(interp), &result) != TCL_OK)
+#else
+  if (Tcl_GetInt(NULL, interp->result, &result) != TCL_OK)
+#endif
+    result = 0;
+  return result;
+}
+
 static tcl_strings def_tcl_strings[] = {
   {"botnet-nick",     botnetnick,     HANDLEN,                 0},
   {"userfile",        userfile,       120,           STR_PROTECT},
@@ -470,6 +499,7 @@ static tcl_strings def_tcl_strings[] = {
   {"config",          configfile,     0,                       0},
   {"telnet-banner",   bannerfile,     120,           STR_PROTECT},
   {"logfile-suffix",  logfile_suffix, 20,                      0},
+  {"timestamp-format",log_ts,         32,                      0},
   {"pidfile",         pid_file,       120,           STR_PROTECT},
   {NULL,              NULL,           0,                       0}
 };
@@ -494,7 +524,7 @@ static tcl_ints def_tcl_ints[] = {
   {"die-on-sighup",         &die_on_sighup,        1},
   {"die-on-sigterm",        &die_on_sigterm,       1},
   {"remote-boots",          &remote_boots,         1},
-  {"max-dcc",               &max_dcc,              0},
+  {"max-socks",             &max_socks,            0},
   {"max-logs",              &max_logs,             0},
   {"max-logsize",           &max_logsize,          0},
   {"quick-logs",            &quick_logs,           0},
@@ -518,6 +548,7 @@ static tcl_ints def_tcl_ints[] = {
   {"userfile-perm",         &userfile_perm,        0},
   {"copy-to-tmp",           &copy_to_tmp,          0},
   {"quiet-reject",          &quiet_reject,         0},
+  {"cidr-support",          &cidr_support,         0},
   {"strict-servernames",    &strict_servernames,   0}, /* compat */
   {"enable-simul",          &enable_simul,         0}, /* compat */
   {"debug-output",          &debug_output,         0}, /* compat */
@@ -553,11 +584,105 @@ void kill_tcl()
 extern tcl_cmds tcluser_cmds[], tcldcc_cmds[], tclmisc_cmds[],
        tclmisc_objcmds[], tcldns_cmds[];
 
+#ifdef REPLACE_NOTIFIER
+/* The tickle_*() functions replace the Tcl Notifier
+ * The tickle_*() functions can be called by Tcl threads
+ */
+void tickle_SetTimer (TCL_CONST86 Tcl_Time *timePtr)
+{
+  struct threaddata *td = threaddata();
+  /* we can block 1 second maximum, because we have SECONDLY events */
+  if (!timePtr || timePtr->sec > 1 || (timePtr->sec == 1 && timePtr->usec > 0)) {
+    td->blocktime.tv_sec = 1;
+    td->blocktime.tv_usec = 0;
+  } else {
+    td->blocktime.tv_sec = timePtr->sec;
+    td->blocktime.tv_usec = timePtr->usec;
+  }
+}
+
+int tickle_WaitForEvent (TCL_CONST86 Tcl_Time *timePtr)
+{
+  struct threaddata *td = threaddata();
+
+  tickle_SetTimer(timePtr);
+  return (*td->mainloopfunc)(0);
+}
+
+void tickle_CreateFileHandler(int fd, int mask, Tcl_FileProc *proc, ClientData cd)
+{
+  alloctclsock(fd, mask, proc, cd);
+}
+
+void tickle_DeleteFileHandler(int fd)
+{
+  killtclsock(fd);
+}
+
+void tickle_FinalizeNotifier(ClientData cd)
+{
+  struct threaddata *td = threaddata();
+  if (td->socklist)
+    nfree(td->socklist);
+}
+
+ClientData tickle_InitNotifier()
+{
+  static int ismainthread = 1;
+  init_threaddata(ismainthread);
+  if (ismainthread)
+    ismainthread = 0;
+  return NULL;
+}
+
+int tclthreadmainloop(int zero)
+{
+  int i;
+  i = sockread(NULL, NULL, threaddata()->socklist, threaddata()->MAXSOCKS, 1);
+  return (i == -4);
+}
+
+struct threaddata *threaddata()
+{
+  static Tcl_ThreadDataKey tdkey;
+  struct threaddata *td = Tcl_GetThreadData(&tdkey, sizeof(struct threaddata));
+  return td;
+}
+
+#else /* REPLACE_NOTIFIER */
+
+int tclthreadmainloop() { return 0; }
+
+struct threaddata *threaddata()
+{
+  static struct threaddata tsd;
+  return &tsd;
+}
+
+#endif /* REPLACE_NOTIFIER */
+
+int init_threaddata(int mainthread)
+{
+  struct threaddata *td = threaddata();
+  td->mainloopfunc = mainthread ? mainloop : tclthreadmainloop;
+  td->socklist = NULL;
+  td->mainthread = mainthread;
+  td->blocktime.tv_sec = 1;
+  td->blocktime.tv_usec = 0;
+  td->MAXSOCKS = 0;
+  increase_socks_max();
+  return 0;
+}
+
 /* Not going through Tcl's crazy main() system (what on earth was he
  * smoking?!) so we gotta initialize the Tcl interpreter
  */
 void init_tcl(int argc, char **argv)
 {
+#ifdef REPLACE_NOTIFIER
+  Tcl_NotifierProcs notifierprocs;
+#endif /* REPLACE_NOTIFIER */
+
 #ifdef USE_TCL_ENCODING
   const char *encoding;
   int i;
@@ -568,6 +693,18 @@ void init_tcl(int argc, char **argv)
   char pver[1024] = "";
 #endif /* USE_TCL_PACKAGE */
 
+#ifdef REPLACE_NOTIFIER
+  egg_bzero(&notifierprocs, sizeof(notifierprocs));
+  notifierprocs.initNotifierProc = tickle_InitNotifier;
+  notifierprocs.createFileHandlerProc = tickle_CreateFileHandler;
+  notifierprocs.deleteFileHandlerProc = tickle_DeleteFileHandler;
+  notifierprocs.setTimerProc = tickle_SetTimer;
+  notifierprocs.waitForEventProc = tickle_WaitForEvent;
+  notifierprocs.finalizeNotifierProc = tickle_FinalizeNotifier;
+
+  Tcl_SetNotifier(&notifierprocs);
+#endif /* REPLACE_NOTIFIER */
+
 /* This must be done *BEFORE* Tcl_SetSystemEncoding(),
  * or Tcl_SetSystemEncoding() will cause a segfault.
  */
@@ -592,6 +729,7 @@ void init_tcl(int argc, char **argv)
 
   /* Setup script library facility */
   Tcl_Init(interp);
+  Tcl_SetServiceMode(TCL_SERVICE_ALL);
 
 /* Code based on Tcl's TclpSetInitialEncodings() */
 #ifdef USE_TCL_ENCODING
@@ -704,11 +842,11 @@ void do_tcl(char *whatzit, char *script)
 #ifdef USE_TCL_ENCODING
   /* properly convert string to system encoding. */
   Tcl_DStringInit(&dstr);
-  Tcl_UtfToExternalDString(NULL, interp->result, -1, &dstr);
+  Tcl_UtfToExternalDString(NULL, tcl_resultstring(), -1, &dstr);
   result = Tcl_DStringValue(&dstr);
 #else
   /* use old pre-Tcl 8.1 way. */
-  result = interp->result;
+  result = tcl_resultstring();
 #endif
 
   if (code != TCL_OK) {
@@ -728,7 +866,7 @@ void do_tcl(char *whatzit, char *script)
 int readtclprog(char *fname)
 {
   int code;
-  char *result;
+  EGG_CONST char *result;
 #ifdef USE_TCL_ENCODING
   Tcl_DString dstr;
 #endif
@@ -788,16 +926,14 @@ void add_tcl_strings(tcl_strings *list)
 
 void rem_tcl_strings(tcl_strings *list)
 {
-  int i;
+  int i, f;
   strinfo *st;
 
+  f = TCL_GLOBAL_ONLY | TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS;
   for (i = 0; list[i].name; i++) {
-    st = (strinfo *) Tcl_VarTraceInfo(interp, list[i].name,
-         TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS, tcl_eggstr,
-         NULL);
-    Tcl_UntraceVar(interp, list[i].name,
-                   TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
-                   tcl_eggstr, st);
+    st = (strinfo *) Tcl_VarTraceInfo(interp, list[i].name, f, tcl_eggstr,
+                                      NULL);
+    Tcl_UntraceVar(interp, list[i].name, f, tcl_eggstr, st);
     if (st != NULL) {
       strtot -= sizeof(strinfo);
       nfree(st);
@@ -829,16 +965,14 @@ void add_tcl_ints(tcl_ints *list)
 
 void rem_tcl_ints(tcl_ints *list)
 {
-  int i;
+  int i, f;
   intinfo *ii;
 
+  f = TCL_GLOBAL_ONLY | TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS;
   for (i = 0; list[i].name; i++) {
-    ii = (intinfo *) Tcl_VarTraceInfo(interp, list[i].name,
-         TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS, tcl_eggint,
-         NULL);
-    Tcl_UntraceVar(interp, list[i].name,
-                   TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
-                   tcl_eggint, (ClientData) ii);
+    ii = (intinfo *) Tcl_VarTraceInfo(interp, list[i].name, f, tcl_eggint,
+                                      NULL);
+    Tcl_UntraceVar(interp, list[i].name, f, tcl_eggint, (ClientData) ii);
     if (ii) {
       strtot -= sizeof(intinfo);
       nfree(ii);
@@ -870,17 +1004,37 @@ void add_tcl_coups(tcl_coups *list)
 
 void rem_tcl_coups(tcl_coups *list)
 {
+  int i, f;
   coupletinfo *cp;
-  int i;
 
+  f = TCL_GLOBAL_ONLY | TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS;
   for (i = 0; list[i].name; i++) {
-    cp = (coupletinfo *) Tcl_VarTraceInfo(interp, list[i].name,
-         TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS, tcl_eggcouplet,
-         NULL);
+    cp = (coupletinfo *) Tcl_VarTraceInfo(interp, list[i].name, f,
+                                          tcl_eggcouplet, NULL);
     strtot -= sizeof(coupletinfo);
-    Tcl_UntraceVar(interp, list[i].name,
-                   TCL_TRACE_READS | TCL_TRACE_WRITES | TCL_TRACE_UNSETS,
-                   tcl_eggcouplet, (ClientData) cp);
+    Tcl_UntraceVar(interp, list[i].name, f, tcl_eggcouplet, (ClientData) cp);
     nfree(cp);
   }
 }
+
+/* Check if the Tcl library supports threads
+*/
+int tcl_threaded()
+{
+#ifdef HAVE_TCL_GETCURRENTTHREAD
+  if (Tcl_GetCurrentThread() != (Tcl_ThreadId)0)
+    return 1;
+#endif
+
+  return 0;
+}
+
+/* Check if we need to fork before initializing Tcl
+*/
+int fork_before_tcl()
+{
+#ifndef REPLACE_NOTIFIER
+  return tcl_threaded();
+#endif
+  return 0;
+}
