@@ -2,11 +2,11 @@
  * tcldcc.c -- handles:
  *   Tcl stubs for the dcc commands
  *
- * $Id: tcldcc.c,v 1.61 2008-02-16 21:41:04 guppy Exp $
+ * $Id: tcldcc.c,v 1.69 2010/07/02 21:56:44 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,8 +31,7 @@ extern Tcl_Interp *interp;
 extern tcl_timer_t *timer, *utimer;
 extern struct dcc_t *dcc;
 extern char botnetnick[];
-extern int dcc_total, backgrd, parties, make_userfile, do_restart, max_dcc,
-           remote_boots;
+extern int dcc_total, backgrd, parties, make_userfile, do_restart, remote_boots, max_dcc;
 extern party_t *party;
 extern tand_t *tandbot;
 extern time_t now;
@@ -59,17 +58,25 @@ int expmem_tcldcc(void)
 
 static int tcl_putdcc STDVAR
 {
-  int i, j;
+  int j;
 
-  BADARGS(3, 3, " idx text");
+  BADARGS(3, 4, " idx text ?options?");
 
-  i = atoi(argv[1]);
-  j = findidx(i);
+  if ((argc == 4) && egg_strcasecmp(argv[3], "-raw")) {
+    Tcl_AppendResult(irp, "unknown putdcc option: should be ",
+                     "-raw", NULL);
+    return TCL_ERROR;
+  }
+
+  j = findidx(atoi(argv[1]));
   if (j < 0) {
     Tcl_AppendResult(irp, "invalid idx", NULL);
     return TCL_ERROR;
   }
-  dumplots(-i, "", argv[2]);
+  if (argc == 4)
+    tputs(dcc[j].sock, argv[2], strlen(argv[2]));
+  else
+    dumplots(j, "", argv[2]);
 
   return TCL_OK;
 }
@@ -85,6 +92,7 @@ static int tcl_putdcc STDVAR
  */
 static int tcl_putdccraw STDVAR
 {
+#if 0
   int i, j = 0, z;
 
   BADARGS(4, 4, " idx size text");
@@ -105,6 +113,11 @@ static int tcl_putdccraw STDVAR
   }
   tputs(j, argv[3], atoi(argv[2]));
   return TCL_OK;
+#endif
+
+  Tcl_AppendResult(irp, "putdccraw is deprecated. "
+                   "Please use putdcc/putnow instead.", NULL);
+  return TCL_ERROR;
 }
 
 static int tcl_dccsimul STDVAR
@@ -203,13 +216,13 @@ static int tcl_setchan STDVAR
     if (!strcmp(argv[2], "-1") || !egg_strcasecmp(argv[2], "off"))
       chan = -1;
     else {
-      Tcl_SetVar(irp, "chan", argv[2], 0);
-      if (Tcl_VarEval(irp, "assoc ", "$chan", NULL) != TCL_OK ||
-          !interp->result[0]) {
+      Tcl_SetVar(irp, "_chan", argv[2], 0);
+      if (Tcl_VarEval(irp, "assoc ", "$_chan", NULL) != TCL_OK ||
+          tcl_resultempty()) {
         Tcl_AppendResult(irp, "channel name is invalid", NULL);
         return TCL_ERROR;
       }
-      chan = atoi(interp->result);
+      chan = tcl_resultint();
     }
   } else
     chan = atoi(argv[2]);
@@ -658,13 +671,13 @@ static int tcl_whom STDVAR
     chan = -1;
   else {
     if ((argv[1][0] < '0') || (argv[1][0] > '9')) {
-      Tcl_SetVar(interp, "chan", argv[1], 0);
-      if ((Tcl_VarEval(interp, "assoc ", "$chan", NULL) != TCL_OK) ||
-          !interp->result[0]) {
+      Tcl_SetVar(interp, "_chan", argv[1], 0);
+      if ((Tcl_VarEval(interp, "assoc ", "$_chan", NULL) != TCL_OK) ||
+          tcl_resultempty()) {
         Tcl_AppendResult(irp, "channel name is invalid", NULL);
         return TCL_ERROR;
       }
-      chan = atoi(interp->result);
+      chan = tcl_resultint();
     } else
       chan = atoi(argv[1]);
     if ((chan < 0) || (chan > 199999)) {
@@ -846,7 +859,7 @@ static int tcl_connect STDVAR
 
   BADARGS(3, 3, " hostname port");
 
-  if (dcc_total == max_dcc) {
+  if (dcc_total == max_dcc && increase_socks_max()) {
     Tcl_AppendResult(irp, "out of dcc table space", NULL);
     return TCL_ERROR;
   }
@@ -918,7 +931,7 @@ static int tcl_listen STDVAR
   }
   if (idx < 0) {
     /* Make new one */
-    if (dcc_total >= max_dcc) {
+    if (dcc_total >= max_dcc && increase_socks_max()) {
       Tcl_AppendResult(irp, "No more DCC slots available.", NULL);
       return TCL_ERROR;
     }
@@ -1040,7 +1053,7 @@ static int tcl_boot STDVAR
 
 static int tcl_rehash STDVAR
 {
-  BADARGS(1, 1, " ");
+  BADARGS(1, 1, "");
 
   if (make_userfile) {
     putlog(LOG_MISC, "*", USERF_NONEEDNEW);
@@ -1055,7 +1068,7 @@ static int tcl_rehash STDVAR
 
 static int tcl_restart STDVAR
 {
-  BADARGS(1, 1, " ");
+  BADARGS(1, 1, "");
 
   if (!backgrd) {
     Tcl_AppendResult(interp, "You can't restart a -n bot", NULL);
