@@ -2,11 +2,11 @@
  * tclegg.h
  *   stuff used by tcl.c and tclhash.c
  *
- * $Id: tclegg.h,v 1.32 2008-02-16 21:41:04 guppy Exp $
+ * $Id: tclegg.h,v 1.40 2010/06/29 15:52:24 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -45,16 +45,18 @@
 #define MATCH_EXACT     1
 #define MATCH_MASK      2
 #define MATCH_CASE      3
+#define MATCH_MODE      4
+#define MATCH_CRON      5
 
 /*
  * Bitwise 'or' these:
  */
 
 /* Check flags; make sure the user has the flags required. */
-#define BIND_USE_ATTR       0x04
+#define BIND_USE_ATTR       0x010
 
 /* Bind is stackable; more than one bind can have the same name. */
-#define BIND_STACKABLE      0x08
+#define BIND_STACKABLE      0x020
 
 /* Additional flag checking; check for +d, +k, etc.
  * Currently used for dcc, fil, msg, and pub bind types.
@@ -69,21 +71,21 @@
 /* FIXME: This type actually seems to be obsolete. This was originally
  *        used to check built-in types in Eggdrop version 1.0.
  */
-#define BIND_HAS_BUILTINS   0x10
+#define BIND_HAS_BUILTINS   0x040
 
 /* Want return; we want to know if the proc returns 1.
  * Side effect: immediate return; don't do any further
  * processing of stacked binds.
  */
-#define BIND_WANTRET        0x20
+#define BIND_WANTRET        0x080
 
 /* Alternate args; replace args with the return result from the Tcl proc. */
-#define BIND_ALTER_ARGS     0x40
+#define BIND_ALTER_ARGS     0x100
 
 /* Stacked return; we want to know if any proc returns 1,
  * and also want to process all stacked binds.
  */
-#define BIND_STACKRET       0x80
+#define BIND_STACKRET       0x200
 
 
 /* Return values. */
@@ -113,7 +115,7 @@ typedef struct timer_str {
         char *argv[];
 
 #define BADARGS(nl, nh, example) do {                                   \
-        if ((argc < (nl)) || (argc > (nh))) {                           \
+        if ((argc < (nl)) || ((argc > (nh)) && ((nh) != -1))) {         \
                 Tcl_AppendResult(irp, "wrong # args: should be \"",     \
                                  argv[0], (example), "\"", NULL);       \
                 return TCL_ERROR;                                       \
@@ -155,12 +157,12 @@ typedef struct _tcl_coups {
 
 typedef struct _tcl_cmds {
   char *name;
-  Function func;
+  IntFunc func;
 } tcl_cmds;
 
 typedef struct _cd_tcl_cmd {
   char *name;
-  Function callback;
+  IntFunc callback;
   void *cdata;
 } cd_tcl_cmd;
 
@@ -174,6 +176,11 @@ void add_tcl_coups(tcl_coups *);
 void rem_tcl_coups(tcl_coups *);
 void add_tcl_ints(tcl_ints *);
 void rem_tcl_ints(tcl_ints *);
+const char *tcl_resultstring();
+int tcl_resultint();
+int tcl_resultempty();
+int tcl_threaded();
+int fork_before_tcl();
 
 /* From Tcl's tclUnixInit.c */
 /* The following table is used to map from Unix locale strings to
