@@ -7,11 +7,11 @@
  *   (non-Tcl) procedure lookups for msg/dcc/file commands
  *   (Tcl) binding internal procedures to msg/dcc/file commands
  *
- * $Id: tclhash.c,v 1.61 2008-02-16 21:41:04 guppy Exp $
+ * $Id: tclhash.c,v 1.70 2010/06/29 15:52:24 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,11 +41,12 @@ extern time_t now;
 p_tcl_bind_list bind_table_list;
 p_tcl_bind_list H_chat, H_act, H_bcst, H_chon, H_chof, H_load, H_unld, H_link,
                 H_disc, H_dcc, H_chjn, H_chpt, H_bot, H_time, H_nkch, H_away,
-                H_note, H_filt, H_event;
+                H_note, H_filt, H_event, H_cron, H_log = NULL;
 
 static int builtin_2char();
 static int builtin_3char();
 static int builtin_5int();
+static int builtin_cron();
 static int builtin_char();
 static int builtin_chpt();
 static int builtin_chjn();
@@ -53,6 +54,7 @@ static int builtin_idxchar();
 static int builtin_charidx();
 static int builtin_chat();
 static int builtin_dcc();
+static int builtin_log();
 
 
 /* Allocate and initialise a chunk of memory.
@@ -217,6 +219,7 @@ void init_bind(void)
   add_cd_tcl_cmds(cd_cmd_table);
   H_unld = add_bind_table("unld", HT_STACKABLE, builtin_char);
   H_time = add_bind_table("time", HT_STACKABLE, builtin_5int);
+  H_cron = add_bind_table("cron", HT_STACKABLE, builtin_cron);
   H_note = add_bind_table("note", 0, builtin_3char);
   H_nkch = add_bind_table("nkch", HT_STACKABLE, builtin_2char);
   H_load = add_bind_table("load", HT_STACKABLE, builtin_char);
@@ -234,6 +237,7 @@ void init_bind(void)
   H_away = add_bind_table("away", HT_STACKABLE, builtin_chat);
   H_act = add_bind_table("act", HT_STACKABLE, builtin_chat);
   H_event = add_bind_table("evnt", HT_STACKABLE, builtin_char);
+  H_log = add_bind_table("log", HT_STACKABLE, builtin_log);
   add_builtins(H_dcc, C_dcc);
   Context;
 }
@@ -250,10 +254,11 @@ void kill_bind(void)
       putlog(LOG_DEBUG, "*", "De-Allocated bind table %s", tl->name);
     tcl_bind_list_delete(tl);
   }
+  H_log = NULL;
   bind_table_list = NULL;
 }
 
-tcl_bind_list_t *add_bind_table(const char *nme, int flg, Function func)
+tcl_bind_list_t *add_bind_table(const char *nme, int flg, IntFunc func)
 {
   tcl_bind_list_t *tl, *tl_prev;
   int v;
@@ -487,7 +492,7 @@ static int tcl_bind STDVAR
   return TCL_OK;
 }
 
-int check_validity(char *nme, Function func)
+int check_validity(char *nme, IntFunc func)
 {
   char *p;
   tcl_bind_list_t *tl;
@@ -540,6 +545,17 @@ static int builtin_5int STDVAR
   return TCL_OK;
 }
 
+static int builtin_cron STDVAR
+{
+  Function F = (Function) cd;
+
+  BADARGS(6, 6, " min hrs dom mon weekday");
+
+  CHECKVALIDITY(builtin_cron);
+  F(atoi(argv[1]), atoi(argv[2]), atoi(argv[3]), atoi(argv[4]), atoi(argv[5]));
+  return TCL_OK;
+}
+
 static int builtin_char STDVAR
 {
   Function F = (Function) cd;
@@ -660,6 +676,16 @@ static int builtin_dcc STDVAR
   return TCL_OK;
 }
 
+static int builtin_log STDVAR
+{
+  Function F = (Function) cd;
+
+  BADARGS(3, 3, " lvl chan msg");
+
+  CHECKVALIDITY(builtin_log);
+  F(argv[1], argv[2], argv[3]);
+  return TCL_OK;
+}
 
 /* Trigger (execute) a Tcl proc
  *
@@ -696,10 +722,7 @@ static inline int trigger_bind(const char *proc, const char *param,
 
   if (x == TCL_ERROR) {
     /* FIXME: we really should be able to log longer errors */
-    if (strlen(interp->result) > 400)
-      interp->result[400] = 0;
-
-    putlog(LOG_MISC, "*", "Tcl error [%s]: %s", proc, interp->result);
+    putlog(LOG_MISC, "*", "Tcl error [%s]: %.*s", proc, 400, tcl_resultstring());
 
     return BIND_EXECUTED;
   }
@@ -707,10 +730,10 @@ static inline int trigger_bind(const char *proc, const char *param,
   /* FIXME: This is an ugly hack. It is not documented as a
    *        'feature' because it will eventually go away.
    */
-  if (!strcmp(interp->result, "break"))
+  if (!strcmp(tcl_resultstring(), "break"))
     return BIND_QUIT;
 
-  return (atoi(interp->result) > 0) ? BIND_EXEC_LOG : BIND_EXECUTED;
+  return (tcl_resultint() > 0) ? BIND_EXEC_LOG : BIND_EXECUTED;
 }
 
 
@@ -722,7 +745,7 @@ static inline int trigger_bind(const char *proc, const char *param,
 static inline int check_bind_match(const char *match, char *mask,
                                    int match_type)
 {
-  switch (match_type & 0x03) {
+  switch (match_type & 0x07) {
   case MATCH_PARTIAL:
     return (!egg_strncasecmp(match, mask, strlen(match)));
     break;
@@ -735,6 +758,12 @@ static inline int check_bind_match(const char *match, char *mask,
   case MATCH_MASK:
     return (wild_match_per(mask, match));
     break;
+  case MATCH_MODE:
+    return (wild_match_partial_case(mask, match));
+    break;
+  case MATCH_CRON:
+    return (cron_match(mask, match));
+    break;
   default:
     /* Do nothing */
     break;
@@ -799,7 +828,7 @@ int check_tcl_bind(tcl_bind_list_t *tl, const char *match,
             /* Either this is a non-partial match, which means we
              * only want to execute _one_ bind ...
              */
-            if ((match_type & 0x03) != MATCH_PARTIAL ||
+            if ((match_type & 0x07) != MATCH_PARTIAL ||
               /* ... or this happens to be an exact match. */
               !egg_strcasecmp(match, tm->mask)) {
               cnt = 1;
@@ -824,7 +853,7 @@ int check_tcl_bind(tcl_bind_list_t *tl, const char *match,
           x = trigger_bind(tc->func_name, param, tm->mask);
 
           if (match_type & BIND_ALTER_ARGS) {
-            if (interp->result == NULL || !interp->result[0])
+            if (tcl_resultempty())
               return x;
           } else if ((match_type & BIND_STACKRET) && x == BIND_EXEC_LOG) {
             /* If we have multiple commands/triggers, and if any of the
@@ -851,7 +880,7 @@ int check_tcl_bind(tcl_bind_list_t *tl, const char *match,
   if (result)           /* BIND_STACKRET */
     return result;
 
-  if ((match_type & 0x03) == MATCH_MASK || (match_type & 0x03) == MATCH_CASE)
+  if ((match_type & 0x07) == MATCH_MASK || (match_type & 0x07) == MATCH_CASE)
     return BIND_EXECUTED;
 
   /* Hit counter */
@@ -861,7 +890,7 @@ int check_tcl_bind(tcl_bind_list_t *tl, const char *match,
   /* Now that we have found at least one bind, we can update the
    * preferred entries information.
    */
-  if (tm_p) {
+  if (tm_p && tm_p->next) {
     tm = tm_p->next;            /* Move mask to front of bind's mask list. */
     tm_p->next = tm->next;      /* Unlink mask from list. */
     tm->next = tl->first;       /* Readd mask to front of list. */
@@ -988,10 +1017,10 @@ const char *check_tcl_filt(int idx, const char *text)
                      MATCH_MASK | BIND_USE_ATTR | BIND_STACKABLE |
                      BIND_WANTRET | BIND_ALTER_ARGS);
   if (x == BIND_EXECUTED || x == BIND_EXEC_LOG) {
-    if (interp->result == NULL || !interp->result[0])
+    if (tcl_resultempty())
       return "";
     else
-      return interp->result;
+      return tcl_resultstring();
   } else
     return text;
 }
@@ -1019,7 +1048,7 @@ void check_tcl_listen(const char *cmd, int idx)
   Tcl_SetVar(interp, "_n", (char *) s, 0);
   x = Tcl_VarEval(interp, cmd, " $_n", NULL);
   if (x == TCL_ERROR)
-    putlog(LOG_MISC, "*", "error on listen: %s", interp->result);
+    putlog(LOG_MISC, "*", "error on listen: %s", tcl_resultstring());
 }
 
 void check_tcl_chjn(const char *bot, const char *nick, int chan,
@@ -1110,12 +1139,45 @@ void check_tcl_time(struct tm *tm)
                  MATCH_MASK | BIND_STACKABLE);
 }
 
+void check_tcl_cron(struct tm *tm)
+{
+  char y[15];
+
+  egg_snprintf(y, sizeof y, "%02d", tm->tm_min);
+  Tcl_SetVar(interp, "_cron1", (char *) y, 0);
+  egg_snprintf(y, sizeof y, "%02d", tm->tm_hour);
+  Tcl_SetVar(interp, "_cron2", (char *) y, 0);
+  egg_snprintf(y, sizeof y, "%02d", tm->tm_mday);
+  Tcl_SetVar(interp, "_cron3", (char *) y, 0);
+  egg_snprintf(y, sizeof y, "%02d", tm->tm_mon + 1);
+  Tcl_SetVar(interp, "_cron4", (char *) y, 0);
+  egg_snprintf(y, sizeof y, "%02d", tm->tm_wday);
+  Tcl_SetVar(interp, "_cron5", (char *) y, 0);
+  egg_snprintf(y, sizeof y, "%02d %02d %02d %02d %02d", tm->tm_min, tm->tm_hour,
+               tm->tm_mday, tm->tm_mon + 1, tm->tm_wday);
+  check_tcl_bind(H_cron, y, 0,
+                 " $_cron1 $_cron2 $_cron3 $_cron4 $_cron5",
+                 MATCH_CRON | BIND_STACKABLE);
+}
+
 void check_tcl_event(const char *event)
 {
   Tcl_SetVar(interp, "_event1", (char *) event, 0);
   check_tcl_bind(H_event, event, 0, " $_event1", MATCH_EXACT | BIND_STACKABLE);
 }
 
+void check_tcl_log(int lv, char *chan, char *msg)
+{
+  char mask[512];
+
+  snprintf(mask, sizeof mask, "%s %s", chan, msg);
+  Tcl_SetVar(interp, "_log1", masktype(lv), 0);
+  Tcl_SetVar(interp, "_log2", chan, 0);
+  Tcl_SetVar(interp, "_log3", msg, 0);
+  check_tcl_bind(H_log, mask, 0, " $_log1 $_log2 $_log3",
+                 MATCH_MASK | BIND_STACKABLE);
+}
+
 void tell_binds(int idx, char *par)
 {
   tcl_bind_list_t *tl, *tl_kind;
@@ -1145,7 +1207,7 @@ void tell_binds(int idx, char *par)
     patmatc = 1;
 
   dprintf(idx, MISC_CMDBINDS);
-  dprintf(idx, "  TYPE FLGS     COMMAND              HITS BINDING (TCL)\n");
+  dprintf(idx, "  TYPE FLAGS    COMMAND              HITS BINDING (TCL)\n");
 
   for (tl = tl_kind ? tl_kind : bind_table_list; tl;
        tl = tl_kind ? 0 : tl->next) {
