@@ -1,11 +1,11 @@
 /*
  * tclhash.h
  *
- * $Id: tclhash.h,v 1.19 2008-02-16 21:41:04 guppy Exp $
+ * $Id: tclhash.h,v 1.25 2010/07/12 15:40:52 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,6 +38,13 @@ typedef struct tcl_cmd_b {
   u_8bit_t attributes;          /* Flags for this entry. TC_* */
 } tcl_cmd_t;
 
+struct threaddata {
+  int (*mainloopfunc)(int);     /* main loop function */
+  sock_list *socklist;          /* tcl socket list for threads, else NULL */
+  struct timeval blocktime;     /* maximum time to block in select() */
+  int mainthread;               /* Is this the main thread? */
+  int MAXSOCKS;
+};
 
 #define TBM_DELETED  0x0001     /* This mask was deleted. */
 
@@ -60,7 +67,7 @@ typedef struct tcl_bind_list_b {
   tcl_bind_mask_t *first;       /* Pointer to registered binds for this list. */
   char name[5];                 /* Name of the bind. */
   u_8bit_t flags;               /* Flags for this element. HT_* */
-  Function func;                /* Function used as the Tcl calling interface
+  IntFunc func;                 /* Function used as the Tcl calling interface
                                  * for procs actually representing C functions. */
 } tcl_bind_list_t, *p_tcl_bind_list;
 
@@ -73,7 +80,7 @@ void init_bind(void);
 void kill_bind(void);
 int expmem_tclhash(void);
 
-tcl_bind_list_t *add_bind_table(const char *nme, int flg, Function func);
+tcl_bind_list_t *add_bind_table(const char *nme, int flg, IntFunc func);
 void del_bind_table(tcl_bind_list_t *tl_which);
 
 tcl_bind_list_t *find_bind_table(const char *nme);
@@ -90,11 +97,13 @@ const char *check_tcl_filt(int, const char *);
 int check_tcl_note(const char *, const char *, const char *);
 void check_tcl_listen(const char *, int);
 void check_tcl_time(struct tm *);
+void check_tcl_cron(struct tm *);
 void tell_binds(int, char *);
 void check_tcl_nkch(const char *, const char *);
 void check_tcl_away(const char *, int, const char *);
 void check_tcl_chatactbcst(const char *, int, const char *, tcl_bind_list_t *);
 void check_tcl_event(const char *);
+void check_tcl_log(int, char *, char *);
 
 #define check_tcl_chat(a, b, c) check_tcl_chatactbcst(a ,b, c, H_chat)
 #define check_tcl_act(a, b, c) check_tcl_chatactbcst(a, b, c, H_act)
@@ -111,10 +120,10 @@ void check_tcl_loadunld(const char *, tcl_bind_list_t *);
 void rem_builtins(tcl_bind_list_t *, cmd_t *);
 void add_builtins(tcl_bind_list_t *, cmd_t *);
 
-int check_validity(char *, Function);
+int check_validity(char *, IntFunc);
 extern p_tcl_bind_list H_chat, H_act, H_bcst, H_chon, H_chof;
 extern p_tcl_bind_list H_load, H_unld, H_dcc, H_bot, H_link;
-extern p_tcl_bind_list H_away, H_nkch, H_filt, H_disc, H_event;
+extern p_tcl_bind_list H_away, H_nkch, H_filt, H_disc, H_event, H_log;
 
 #endif /* MAKING_MODS */
 
