@@ -2,11 +2,11 @@
  * tclmisc.c -- handles:
  *   Tcl stubs for everything else
  *
- * $Id: tclmisc.c,v 1.59 2008-02-16 21:41:04 guppy Exp $
+ * $Id: tclmisc.c,v 1.70 2010/07/01 16:10:49 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -86,11 +86,8 @@ static int tcl_logfile STDVAR
     /* They just want a list of the logfiles and modes */
     for (i = 0; i < max_logs; i++)
       if (logs[i].filename != NULL) {
-        strcpy(s, masktype(logs[i].mask));
-        strcat(s, " ");
-        strcat(s, logs[i].chname);
-        strcat(s, " ");
-        strcat(s, logs[i].filename);
+        snprintf(s, sizeof s, "%s %s %s", masktype(logs[i].mask),
+                 logs[i].chname, logs[i].filename);
         Tcl_AppendElement(interp, s);
       }
       return TCL_OK;
@@ -98,6 +95,20 @@ static int tcl_logfile STDVAR
 
   BADARGS(4, 4, " ?logModes channel logFile?");
 
+  if (*argv[1] && !*argv[2]) {
+    Tcl_AppendResult(interp,
+                     "log modes set, but no channel specified", NULL);
+    return TCL_ERROR;
+  }
+  if (*argv[2] && !strchr(CHANMETA, *argv[2]) && strcmp(argv[2], "*")) {
+    Tcl_AppendResult(interp, "invalid channel prefix", NULL);
+    return TCL_ERROR;
+  }
+  if (*argv[2] && strchr(argv[2], ' ')) {
+    Tcl_AppendResult(interp, "channel names cannot contain spaces", NULL);
+    return TCL_ERROR;
+  }
+
   for (i = 0; i < max_logs; i++)
     if ((logs[i].filename != NULL) && (!strcmp(logs[i].filename, argv[3]))) {
       logs[i].flags &= ~LF_EXPIRING;
@@ -179,7 +190,7 @@ static int tcl_putloglev STDVAR
   int lev = 0;
   char logtext[501];
 
-  BADARGS(4, 4, " level channel text");
+  BADARGS(4, 4, " level(s) channel text");
 
   lev = logmodes(argv[1]);
   if (!lev) {
@@ -378,7 +389,7 @@ static int tcl_unixtime STDVAR
 
   BADARGS(1, 1, "");
 
-  egg_snprintf(s, sizeof s, "%li", now2);
+  egg_snprintf(s, sizeof s, "%li", (long) now2);
   Tcl_AppendResult(irp, s, NULL);
   return TCL_OK;
 }
@@ -430,19 +441,26 @@ static int tcl_myip STDVAR
 
 static int tcl_rand STDVAR
 {
+  long i;
   unsigned long x;
   char s[11];
 
   BADARGS(2, 2, " limit");
 
-  if (atol(argv[1]) <= 0) {
+  i = atol(argv[1]);
+
+  if (i <= 0) {
     Tcl_AppendResult(irp, "random limit must be greater than zero", NULL);
     return TCL_ERROR;
+  } else if (i > RANDOM_MAX) {
+    Tcl_AppendResult(irp, "random limit must be less than ", RANDOM_MAX, NULL);
+    return TCL_ERROR;
   }
 
-  x = randint((unsigned long) (atol(argv[1])));
+  x = randint(i);
 
   egg_snprintf(s, sizeof s, "%lu", x);
+
   Tcl_AppendResult(irp, s, NULL);
   return TCL_OK;
 }
@@ -547,7 +565,7 @@ static int tcl_unames STDVAR
 
   if (uname(&un) < 0) {
 #endif
-    unix_n = "*unkown*";
+    unix_n = "*unknown*";
     vers_n = "";
 #ifdef HAVE_UNAME
   } else {
@@ -685,7 +703,7 @@ static int tcl_md5 STDVAR
     return TCL_ERROR;
   }
 #  ifdef USE_TCL_BYTE_ARRAYS
-  string = (unsigned char *)Tcl_GetByteArrayFromObj(objv[1], &len);
+  string = (char *)Tcl_GetByteArrayFromObj(objv[1], &len);
 #  else
   string = Tcl_GetStringFromObj(objv[1], &len);
 #  endif /* USE_TCL_BYTE_ARRAYS */
@@ -705,6 +723,39 @@ static int tcl_md5 STDVAR
   return TCL_OK;
 }
 
+static int tcl_matchaddr STDVAR
+{
+  BADARGS(3, 3, " mask address");
+
+  if (match_addr(argv[1], argv[2]))
+    Tcl_AppendResult(irp, "1", NULL);
+  else
+    Tcl_AppendResult(irp, "0", NULL);
+  return TCL_OK;
+}
+
+static int tcl_matchcidr STDVAR
+{
+  BADARGS(4, 4, " block address prefix");
+
+  if (cidr_match(argv[1], argv[2], atoi(argv[3])))
+    Tcl_AppendResult(irp, "1", NULL);
+  else
+    Tcl_AppendResult(irp, "0", NULL);
+  return TCL_OK;
+}
+
+static int tcl_matchstr STDVAR
+{
+  BADARGS(3, 3, " pattern string");
+
+  if (wild_match(argv[1], argv[2]))
+    Tcl_AppendResult(irp, "1", NULL);
+  else
+    Tcl_AppendResult(irp, "0", NULL);
+  return TCL_OK;
+}
+
 tcl_cmds tclmisc_objcmds[] = {
 #ifdef USE_TCL_OBJ
   {"md5", tcl_md5},
@@ -750,5 +801,8 @@ tcl_cmds tclmisc_cmds[] = {
   {"binds",               tcl_binds},
   {"callevent",       tcl_callevent},
   {"stripcodes",     tcl_stripcodes},
+  {"matchaddr",       tcl_matchaddr},
+  {"matchcidr",       tcl_matchcidr},
+  {"matchstr",         tcl_matchstr},
   {NULL,                       NULL}
 };
