@@ -2,11 +2,11 @@
  * tcluser.c -- handles:
  *   Tcl stubs for the user-record-oriented commands
  *
- * $Id: tcluser.c,v 1.44 2008-02-16 21:41:04 guppy Exp $
+ * $Id: tcluser.c,v 1.51 2010/07/01 16:10:49 thommey Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -78,6 +78,7 @@ static int tcl_passwdOk STDVAR
 
 static int tcl_chattr STDVAR
 {
+  int of, ocf = 0;
   char *chan, *chg, work[100];
   struct flag_record pls, mns, user;
   struct userrec *u;
@@ -121,9 +122,11 @@ static int tcl_chattr STDVAR
     Tcl_AppendResult(irp, "no such channel", NULL);
     return TCL_ERROR;
   }
+  /* Retrieve current flags */
   get_user_flagrec(u, &user, chan);
   /* Make changes */
   if (chg) {
+    of = user.global;
     pls.match = user.match;
     break_down_flags(chg, &pls, &mns);
     /* No-one can change these flags on-the-fly */
@@ -139,13 +142,19 @@ static int tcl_chattr STDVAR
     user.udef_global = (user.udef_global | pls.udef_global)
                        & ~mns.udef_global;
     if (chan) {
+      ocf = user.chan;
       user.chan = chan_sanity_check((user.chan | pls.chan) & ~mns.chan,
                                     user.global);
       user.udef_chan = (user.udef_chan | pls.udef_chan) & ~mns.udef_chan;
+
     }
     set_user_flagrec(u, &user, chan);
+    check_dcc_attrs(u, of);
+    if (chan)
+      check_dcc_chanattrs(u, chan, user.chan, ocf);
   }
-  /* Retrieve current flags and return them */
+  user.chan &= ~BOT_SHARE; /* actually not a user flag, hide it */
+  /* Build flag string */
   build_flags(work, &user, NULL);
   Tcl_AppendResult(irp, work, NULL);
   return TCL_OK;
@@ -203,6 +212,7 @@ static int tcl_botattr STDVAR
     Tcl_AppendResult(irp, "no such channel", NULL);
     return TCL_ERROR;
   }
+  /* Retrieve current flags */
   get_user_flagrec(u, &user, chan);
   /* Make changes */
   if (chg) {
@@ -220,7 +230,12 @@ static int tcl_botattr STDVAR
     }
     set_user_flagrec(u, &user, chan);
   }
-  /* Retrieve current flags and return them */
+  /* Only user flags can be set per channel, not bot ones,
+     so BOT_SHARE is a hack to allow botattr |+s */
+  user.chan &= BOT_SHARE;
+  user.udef_chan = 0; /* User definable bot flags are global only,
+                         anything here is a regular flag, so hide it. */
+  /* Build flag string */
   build_flags(work, &user, NULL);
   Tcl_AppendResult(irp, work, NULL);
   return TCL_OK;
@@ -239,7 +254,7 @@ static int tcl_matchattr STDVAR
     get_user_flagrec(u, &user, argv[3]);
     plus.match = user.match;
     break_down_flags(argv[2], &plus, &minus);
-    f = (minus.global ||minus.udef_global || minus.chan || minus.udef_chan ||
+    f = (minus.global || minus.udef_global || minus.chan || minus.udef_chan ||
          minus.bot);
     if (flagrec_eq(&plus, &user)) {
       if (!f)
@@ -257,11 +272,18 @@ static int tcl_matchattr STDVAR
 
 static int tcl_adduser STDVAR
 {
+  unsigned char *p;
+
   BADARGS(2, 3, " handle ?hostmask?");
 
   if (strlen(argv[1]) > HANDLEN)
     argv[1][HANDLEN] = 0;
-  if ((argv[1][0] == '*') || get_user_by_handle(userlist, argv[1]))
+  for (p = (unsigned char *) argv[1]; *p; p++)
+    if (*p <= 32 || *p == '@')
+      *p = '?';
+
+  if ((argv[1][0] == '*') || strchr(BADHANDCHARS, argv[1][0]) ||
+      get_user_by_handle(userlist, argv[1]))
     Tcl_AppendResult(irp, "0", NULL);
   else {
     userlist = adduser(userlist, argv[1], argv[2], "-", default_flags);
@@ -279,9 +301,12 @@ static int tcl_addbot STDVAR
 
   if (strlen(argv[1]) > HANDLEN)
     argv[1][HANDLEN] = 0;
-  if (get_user_by_handle(userlist, argv[1]))
-    Tcl_AppendResult(irp, "0", NULL);
-  else if (argv[1][0] == '*')
+  for (p = argv[1]; *p; p++)
+    if ((unsigned char) *p <= 32 || *p == '@')
+      *p = '?';
+
+  if ((argv[1][0] == '*') || strchr(BADHANDCHARS, argv[1][0]) ||
+      get_user_by_handle(userlist, argv[1]))
     Tcl_AppendResult(irp, "0", NULL);
   else {
     userlist = adduser(userlist, argv[1], "none", "-", USER_BOT);
@@ -394,7 +419,7 @@ static int tcl_chhandle STDVAR
   else {
     strncpyz(newhand, argv[2], sizeof newhand);
     for (i = 0; i < strlen(newhand); i++)
-      if ((newhand[i] <= 32) || (newhand[i] >= 127) || (newhand[i] == '@'))
+      if (((unsigned char) newhand[i] <= 32) || (newhand[i] == '@'))
         newhand[i] = '?';
     if (strchr(BADHANDCHARS, newhand[0]) != NULL)
       x = 0;
@@ -505,7 +530,7 @@ static int tcl_getuser STDVAR
   struct userrec *u;
   struct user_entry *e;
 
-  BADARGS(3, 999, " handle type");
+  BADARGS(3, -1, " handle type");
 
   if (!(et = find_entry_type(argv[2])) && egg_strcasecmp(argv[2], "HANDLE")) {
     Tcl_AppendResult(irp, "No such info type: ", argv[2], NULL);
@@ -536,7 +561,7 @@ static int tcl_setuser STDVAR
   int r;
   module_entry *me;
 
-  BADARGS(3, 999, " handle type ?setting....?");
+  BADARGS(3, -1, " handle type ?setting....?");
 
   if (!(et = find_entry_type(argv[2]))) {
     Tcl_AppendResult(irp, "No such info type: ", argv[2], NULL);
@@ -564,7 +589,7 @@ static int tcl_setuser STDVAR
   }
   r = et->tcl_set(irp, u, e, argc, argv);
   /* Yeah... e is freed, and we read it... (tcl: setuser hand HOSTS none) */
-  if ((!e->u.list) && (list_delete((struct list_type **) &(u->entries),
+  if ((!e->u.list) && (egg_list_delete((struct list_type **) &(u->entries),
       (struct list_type *) e)))
     nfree(e);
     /* else maybe already freed... (entry_type==HOSTS) <drummer> */
