@@ -4,11 +4,11 @@
  *   a bunch of functions to find and change user records
  *   change and check user (and channel-specific) flags
  *
- * $Id: userrec.c,v 1.57 2008-02-16 21:41:05 guppy Exp $
+ * $Id: userrec.c,v 1.64 2010/07/05 12:07:05 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -45,7 +45,7 @@ struct userrec *lastuser = NULL;   /* last accessed user record     */
 maskrec *global_bans = NULL, *global_exempts = NULL, *global_invites = NULL;
 struct igrec *global_ign = NULL;
 int cache_hit = 0, cache_miss = 0; /* temporary cache accounting    */
-int strict_host = 0;
+int strict_host = 1;
 int userfile_perm = 0600;         /* Userfile permissions
                                    * (default rw-------) */
 
@@ -168,21 +168,23 @@ int count_users(struct userrec *bu)
  */
 char *fixfrom(char *s)
 {
-  char *p = NULL;
+  static char uhost[UHOSTLEN];
+  char *p = uhost;
 
   if (!s || !*s || strict_host)
     return s;
 
-  if ((p = strchr(s, '!'))) {
-    if (!*(++p))
-      return s;	/* There's nothing following "!". */
-  } else
-    p = s; /* There's no nick. */
+  while (*s) {
+    *p++ = *s;
+    if (*s == '!' && strchr("~+-^=", s[1]) && s[2] != '@') {
+      strcpy(p, s + 2);
+      return uhost;
+    }
+    s++;
+  }
 
-  if (strchr("~+-^=", *p) && *(p + 1) != '@')
-    memmove(p, p + 1, strlen(p)); /* NUL is included without +1. */
-
-  return s;
+  *p = 0;
+  return uhost;
 }
 
 struct userrec *check_dcclist_hand(char *handle)
@@ -323,11 +325,11 @@ struct userrec *get_user_by_host(char *host)
   }
   cache_miss++;
   strncpyz(host2, host, sizeof host2);
-  fixfrom(host);
+  host = fixfrom(host);
   for (u = userlist; u; u = u->next) {
     q = get_user(&USERENTRY_HOSTS, u);
     for (; q; q = q->next) {
-      i = wild_match(q->extra, host);
+      i = match_useraddr(q->extra, host);
       if (i > cnt) {
         ret = u;
         cnt = i;
@@ -597,7 +599,8 @@ int change_handle(struct userrec *u, char *newh)
   strncpyz(s, u->handle, sizeof s);
   strncpyz(u->handle, newh, sizeof u->handle);
   for (i = 0; i < dcc_total; i++)
-    if (dcc[i].type != &DCC_BOT && !egg_strcasecmp(dcc[i].nick, s)) {
+    if ((dcc[i].type == &DCC_CHAT || dcc[i].type == &DCC_CHAT_PASS) &&
+        !egg_strcasecmp(dcc[i].nick, s)) {
       strncpyz(dcc[i].nick, newh, sizeof dcc[i].nick);
       if (dcc[i].type == &DCC_CHAT && dcc[i].u.chat->channel >= 0) {
         chanout_but(-1, dcc[i].u.chat->channel,
@@ -657,7 +660,7 @@ struct userrec *adduser(struct userrec *bu, char *handle, char *host,
      *   but its much easier to use here...  (drummer)
      *   Only use it if we have a host :) (dw)
      */
-    fixfrom(host);
+    host = fixfrom(host);
 
     p = strchr(host, ',');
     while (p != NULL) {
@@ -902,3 +905,24 @@ void user_del_chan(char *dname)
     }
   }
 }
+
+/* Check if the console flags specified in md are permissible according
+ * to the given flagrec. If the FR_CHAN flag is not set in fr->match,
+ * only global user flags will be considered.
+ * Returns: md with all unallowed flags masked out.
+ */
+int check_conflags(struct flag_record *fr, int md)
+{
+  if (!glob_owner(*fr))
+    md &= ~(LOG_RAW | LOG_SRVOUT | LOG_BOTNET | LOG_BOTSHARE);
+  if (!glob_master(*fr)) {
+    md &= ~(LOG_FILES | LOG_LEV1 | LOG_LEV2 | LOG_LEV3 | LOG_LEV4 |
+            LOG_LEV5 | LOG_LEV6 | LOG_LEV7 | LOG_LEV8 | LOG_DEBUG |
+            LOG_WALL);
+    if ((fr->match & FR_CHAN) && !chan_master(*fr))
+      md &= ~(LOG_MISC | LOG_CMDS);
+  }
+  if (!glob_botmast(*fr))
+    md &= ~LOG_BOTS;
+  return md;
+}
