@@ -10,11 +10,11 @@
  *
  * dprintf'ized, 9nov1995
  *
- * $Id: users.c,v 1.51 2008-02-16 21:41:05 guppy Exp $
+ * $Id: users.c,v 1.59 2010/02/07 17:21:14 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -58,7 +58,7 @@ int match_ignore(char *uhost)
   struct igrec *ir;
 
   for (ir = global_ign; ir; ir = ir->next)
-    if (wild_match(ir->igmask, uhost))
+    if (match_useraddr(ir->igmask, uhost))
       return 1;
   return 0;
 }
@@ -204,7 +204,7 @@ void tell_ignores(int idx, char *match)
   dprintf(idx, "%s:\n", IGN_CURRENT);
   for (; u; u = u->next) {
     if (match[0]) {
-      if (wild_match(match, u->igmask) ||
+      if (cmp_usermasks(match, u->igmask) ||
           wild_match(match, u->msg) || wild_match(match, u->user))
         display_ignore(idx, k, u);
       k++;
@@ -485,17 +485,17 @@ void tell_user(int idx, struct userrec *u, int master)
   fr.udef_global = u->flags_udef;
   build_flags(s, &fr, NULL);
   if (module_find("notes", 0, 0)) {
-    Tcl_SetVar(interp, "user", u->handle, 0);
-    if (Tcl_VarEval(interp, "notes ", "$user", NULL) == TCL_OK)
-      n = atoi(interp->result);
+    Tcl_SetVar(interp, "_user", u->handle, 0);
+    if (Tcl_VarEval(interp, "notes ", "$_user", NULL) == TCL_OK)
+      n = tcl_resultint();
   }
   li = get_user(&USERENTRY_LASTON, u);
   if (!li || !li->laston)
     strcpy(s1, "never");
   else {
     now2 = now - li->laston;
-    if (now2 > 86400)
-      egg_strftime(s1, 7, "%d %b", localtime(&li->laston));
+    if (now2 >= 86400)
+      egg_strftime(s1, 11, "%Y-%m-%d", localtime(&li->laston));
     else
       egg_strftime(s1, 6, "%H:%M", localtime(&li->laston));
   }
@@ -513,8 +513,8 @@ void tell_user(int idx, struct userrec *u, int master)
         strcpy(s1, "never");
       else {
         now2 = now - (ch->laston);
-        if (now2 > 86400)
-          egg_strftime(s1, 7, "%d %b", localtime(&ch->laston));
+        if (now2 >= 86400)
+          egg_strftime(s1, 11, "%Y-%m-%d", localtime(&ch->laston));
         else
           egg_strftime(s1, 6, "%H:%M", localtime(&ch->laston));
       }
@@ -674,7 +674,7 @@ int readuserfile(char *file, struct userrec **ret)
   struct userrec *bu, *u = NULL;
   struct chanset_t *cst = NULL;
   int i;
-  char ignored[512];
+  char ignored[LOGLINEMAX]; /* putlog() will truncate anything larger anyway */
   struct flag_record fr;
   struct chanuserrec *cr;
 
@@ -794,8 +794,10 @@ int readuserfile(char *file, struct userrec **ret)
             strcpy(s1, lasthand);
             strcat(s1, " ");
             if (strstr(ignored, s1) == NULL) {
-              strcat(ignored, lasthand);
-              strcat(ignored, " ");
+              strncat(ignored, lasthand,
+                      sizeof(ignored) - 1 - strlen(ignored));
+              strncat(ignored, " ",
+                      sizeof(ignored) - 1 - strlen(ignored));
             }
             lasthand[0] = 0;
           } else {
@@ -820,8 +822,10 @@ int readuserfile(char *file, struct userrec **ret)
             strcpy(s1, lasthand);
             strcat(s1, " ");
             if (strstr(ignored, s1) == NULL) {
-              strcat(ignored, lasthand);
-              strcat(ignored, " ");
+              strncat(ignored, lasthand,
+                      sizeof(ignored) - 1 - strlen(ignored));
+              strncat(ignored, " ",
+                      sizeof(ignored) - 1 - strlen(ignored));
             }
             lasthand[0] = 0;
           } else {
@@ -846,8 +850,10 @@ int readuserfile(char *file, struct userrec **ret)
             strcpy(s1, lasthand);
             strcat(s1, " ");
             if (strstr(ignored, s1) == NULL) {
-              strcat(ignored, lasthand);
-              strcat(ignored, " ");
+              strncat(ignored, lasthand,
+                      sizeof(ignored) - 1 - strlen(ignored));
+              strncat(ignored, " ",
+                      sizeof(ignored) - 1 - strlen(ignored));
             }
             lasthand[0] = 0;
           } else {
@@ -879,7 +885,7 @@ int readuserfile(char *file, struct userrec **ret)
                 list->next = NULL;
                 list->extra = user_malloc(strlen(s) + 1);
                 strcpy(list->extra, s);
-                list_append((&ue->u.list), list);
+                egg_list_append((&ue->u.list), list);
                 ok = 1;
               }
             if (!ok) {
@@ -963,7 +969,7 @@ int readuserfile(char *file, struct userrec **ret)
     struct user_entry *e;
 
     if (!(u->flags & USER_BOT) && !egg_strcasecmp(u->handle, botnetnick)) {
-      putlog(LOG_MISC, "*", "(!) I have an user record, but without +b");
+      putlog(LOG_MISC, "*", "(!) I have a user record, but without +b");
       /* u->flags |= USER_BOT; */
     }
 
