@@ -2,11 +2,11 @@
  * users.h
  *   structures and definitions used by users.c and userrec.c
  *
- * $Id: users.h,v 1.18 2008-02-16 21:41:05 guppy Exp $
+ * $Id: users.h,v 1.20 2010/02/07 17:21:14 pseudo Exp $
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2008 Eggheads Development Team
+ * Copyright (C) 1999 - 2010 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,9 +36,9 @@ struct list_type {
         (b)->next = *(a);                                               \
         *(a) = (b);                                                     \
 }
-int list_append(struct list_type **, struct list_type *);
-int list_delete(struct list_type **, struct list_type *);
-int list_contains(struct list_type *, struct list_type *);
+int egg_list_append(struct list_type **, struct list_type *);
+int egg_list_delete(struct list_type **, struct list_type *);
+int egg_list_contains(struct list_type *, struct list_type *);
 
 
 /* New userfile format stuff
